;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Command/backtrace.scm          */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Jul 28 21:34:02 1999                          */
;*    Last change :  Mon Oct  4 12:01:23 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The backtrace command                                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_backtrace
   (import tools_error
	   tools_tools
	   tools_io
	   patient_mangling
	   patient_invoke
	   patient_value
	   gdb_tools
	   gdb_invoke
	   gdb_annotate
	   engine_param
	   command_command)
   (export (display-stack-frames ::bstring)
	   (display-frame frame)
	   (stack-frame-number::int sframe)
	   (frame=? ::obj ::obj)))

;*---------------------------------------------------------------------*/
;*    display-stack-frames ...                                         */
;*---------------------------------------------------------------------*/
(define (display-stack-frames cmd)
   ;; before getting a new stack we have to reset the current one
   (gdb-reset-stack-frames!)
   ;; we store the current stack frame
   (let ((frame (gdb-current-frame)))
      ;; then we emit the command
      (gdb-call-for-output cmd)
      (let ((stack (gdb-stack-frames)))
	 ;; we have to restore the current stack frame
	 (gdb-restore-current-frame! frame)
	 ;; and now we can display the stack
	 (for-each (lambda (x) (display-frame x)) (reverse stack))
	 ;; but we have to forget about what we have just computed
	 (gdb-command-status #f))))

;*---------------------------------------------------------------------*/
;*    *display-frame-armed?* ...                                       */
;*---------------------------------------------------------------------*/
(define *display-frame-armed?* #f)

;*---------------------------------------------------------------------*/
;*    display-frame ...                                                */
;*---------------------------------------------------------------------*/
(define (display-frame frame)
   (if (not *display-frame-armed?*)
       (begin
	  (set! *display-frame-armed?* #t)
	  (match-case frame
	     ((?c-ident ?args ?num ?pos ?pref-num)
	      (define (display-bgl-frame scm-name)
		 (console-echo scm-name))
 	      (define (display-c-frame c-name)
		 (console-echo c-name))
	      (if (and (not (string=? pref-num ""))
		       (not (char=? (string-ref pref-num 0) #\space)))
		  (begin
		     (console-echo pref-num)
		     (console-echo (if (< num 10) "  " " "))))
	      (let ((i (bdb-demangle c-ident)))
		 (if (string? i)
		     (console-echo i)
		     (console-echo c-ident)))
	      (if (pair? args)
		  (console-echo "(...)")
		  (console-echo "()"))
	      (match-case pos
		 ((?file . ?line)
		  (console-echo " at ")
		  (let ((str (string-append file ":" line)))
		     (console-echo-edit str str))))
	      (console-newline)))
	  (set! *display-frame-armed?* #f))))

;*---------------------------------------------------------------------*/
;*    stack-frame-number ...                                           */
;*---------------------------------------------------------------------*/
(define (stack-frame-number frame)
   (match-case frame
      ((?- ?- ?- ?- ?pref-num)
       (if (and (>fx (string-length pref-num) 0)
		(char=? (string-ref pref-num 0) #\#))
	   (string->integer (string-until-at pref-num 1 #\space 0))
	   0))
      (else
       0)))

;*---------------------------------------------------------------------*/
;*    frame=? ...                                                      */
;*    -------------------------------------------------------------    */
;*    This function returns #t iff the two stack frames F1 and F2      */
;*    are the same stack frame                                         */
;*---------------------------------------------------------------------*/
(define (frame=? f1 f2)
   (equal? f1 f2))
   
;*---------------------------------------------------------------------*/
;*    *backtrace-help* ...                                             */
;*---------------------------------------------------------------------*/
(define *backtrace-help* 
   "Print backtrace of all stack frames, or innermost COUNT frames.
With a negative argument, print outermost -COUNT frames.")

;*---------------------------------------------------------------------*/
;*    The backtrace command                                            */
;*---------------------------------------------------------------------*/
(let ((action (lambda (cmd source level env)
		 (display-stack-frames source))))
   ;; backtrace
   (bind-toplevel-command! "backtrace"
			   2
			   action
			   *backtrace-help*)
   ;; bt
   (bind-toplevel-command! "bt"
			   2
			   action
			   *backtrace-help*))

	   


