;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Command/break.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 16:25:36 1999                          */
;*    Last change :  Mon Oct  4 13:15:15 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    In this implementation of break, we assume that Scheme           */
;*    breakpoints are entered using upper case strings and C           */
;*    breakpoints are lower case strings.                              */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_break
   (import tools_error
	   tools_tools
	   tools_io
	   patient_mangling
	   bee_etags
	   command_command
	   command_parse
	   command_expr
	   gdb_invoke
	   engine_param
	   (gdb-current-function gdb_annotate))
   (export (display-breakpoint-table table)
	   (compare-breakpoint-tables old new)
	   (break-condition ::pair-nil)
	   (breakpoint-changed!)
	   (breakpoint-changed?::bool)))

;*---------------------------------------------------------------------*/
;*    *breakpoint-changed?* ...                                        */
;*    -------------------------------------------------------------    */
;*    This variable is set to #t when an break command is issued       */
;*    (e.g. delete, disable, enabled, break, ...).                     */
;*---------------------------------------------------------------------*/
(define *breakpoint-changed?* #f)

;*---------------------------------------------------------------------*/
;*    breakpoint-changed? ...                                          */
;*---------------------------------------------------------------------*/
(define (breakpoint-changed?::bool)
   *breakpoint-changed?*)

;*---------------------------------------------------------------------*/
;*    breakpoint-changed! ...                                          */
;*---------------------------------------------------------------------*/
(define (breakpoint-changed!)
   (set! *breakpoint-changed?* #t))

;*---------------------------------------------------------------------*/
;*    display-breakpoint-table ...                                     */
;*    -------------------------------------------------------------    */
;*    Prints out a breakpoint table has parsed by the --annotate       */
;*    parser. This function must implements some name demangling       */
;*    of the what field. That field display a summary of the           */
;*    breakpoint location. That description contains the function      */
;*    name :-(.                                                        */
;*---------------------------------------------------------------------*/
(define (display-breakpoint-table table)
   (define (get-tabbing-list)
      '(4 16 5 4 12 0 0 0 0 0 0 0 0 0 0))
   (match-case table
      ((?headers . ?records)
       (define (what-value value)
	  ;; This function computes the shape of the WHAT value of
	  ;; a breakpoint record. This require a little bit a name demangling.
	  (let* ((ident     (string-until-at value 3 #\space 0))
		 (scm-ident (bdb-demangle ident)))
	     (let* ((start (string-from-at value 3 #\space 1))
		    (loc   (substring start 3 (string-length start))))
		(values
		 (if (string? scm-ident)
		     (string-append "in "
				    scm-ident
				    (string-from-at value 3 #\space 0))
		     value)
		 loc))))
       (define (display/width string width)
	  (let ((add (-fx width (string-length string))))
	     (console-echo string)
	     (if (>fx add 0)
		 (console-echo (make-string add #\space)))))
       (define (display-breakpoint-table-headers headers)
	  (for-each (lambda (x w) (display/width x w))
		    headers
		    (get-tabbing-list))
	  (console-newline))
       (define (display-breakpoint-record-value/edit string edit width)
	  (let ((add (-fx width (string-length string))))
	     (console-echo-edit string edit)
	     (if (>fx add 0)
		 (console-echo (make-string add #\space)))))
       (define (display-breakpoint-record-value record tabbing)
	  (display/width record tabbing))
       (define (display-breakpoint-table-record record len)
	  ;; we display the first element with tabing
	  (let loop ((i       0)
		     (record  record)
		     (tabbing (get-tabbing-list)))
	     (cond
		((=fx i len)
		 (console-newline)
		 ;; we display all the other elements in separate lines
		 (for-each (lambda (x)
			      (console-echo x) (console-newline))
			   record))
		((=fx i (-fx len 1))
		 ;; this is the WHAT value, the one that has to be demangled
		 (let ((tab (if (pair? tabbing) (car tabbing) 0)))
		    (multiple-value-bind (value loc)
		       (what-value (car record))
		       (if loc
			   (display-breakpoint-record-value/edit value loc tab)
			   (display-breakpoint-record-value value tab)))
		    (loop (+fx i 1)
			  (cdr record)
			  (cdr tabbing))))
		(else
		 (let ((tab (if (pair? tabbing) (car tabbing) 0)))
		    (display-breakpoint-record-value (car record) tab)
		    (loop (+fx i 1)
			  (cdr record)
			  (cdr tabbing)))))))
       (define (display-breakpoint-table-records records len)
	  (for-each (lambda (x) (display-breakpoint-table-record x len))
		    records))
       (display-breakpoint-table-headers headers)
       (display-breakpoint-table-records records (length headers)))
      ((kwote no-breakpoint)
       ;; no breakpoint yet
       (console-echo #"No breakpoints or watchpoints.\n"))
      (else
       (error "display-breakpoint-table" "Illegal table" table))))

;*---------------------------------------------------------------------*/
;*    break-command-parse ...                                          */
;*    -------------------------------------------------------------    */
;*    If the argument of the breakpoint is an identifier and if that   */
;*    identifier the one of a Bigloo function we try successively to   */
;*    find out the source file position of that identifier using the   */
;*    read etags file. If this fail we try a name mangling. The name   */
;*    mangling succeeds if the program is already started. Otherwise   */
;*    it fails and the breakpoint will be rejected.                    */
;*---------------------------------------------------------------------*/
(define (break-command-parse base cmd-list source::bstring level::int env)
   (match-case cmd-list
      (()
       ;; mark the breakpoints as changed
       (breakpoint-changed!)
       ;; issue the command
       (console-echo (gdb-call-for-output "break")))
      ((?fun)
       ;; mark the breakpoints as changed
       (breakpoint-changed!)
       ;; this is a breakpoint to a function
       (if (bigloo-ident? fun)
	   (let ((fpos (bigloo-symbol-etags-source-location fun)))
	      ;; before operating a name mangling we check
	      ;; if we know the source file position of that identifier
	      (if (string? fpos)
		  ;; yes we do
		  (let ((cmd (string-append base " " fpos)))
		     (console-echo (gdb-call-for-output cmd)))
		  ;; no we don't thus we try some name mangling to get
		  ;; the C name associated to that function
		  (let ((c-name (bdb-mangle fun)))
		     (if (string? c-name)
			 (let ((cmd (string-append base " " c-name)))
			    (console-echo (gdb-call-for-output cmd)))
			 ;; we have found nothing the symbol is unbound
			 (bdb-error "break"
				    "Can't find Bigloo identifier"
				    fun)))))
	   ;; this is a plain C identifier or a source file position
	   (console-echo (gdb-call-for-output source))))
      ((?- . ?rest)
       (parse-error "Junk at end of arguments." (car rest) source 2))))

;*---------------------------------------------------------------------*/
;*    break-condition ...                                              */
;*    -------------------------------------------------------------    */
;*    Return the condition expression associated to a breakpoint       */
;*    records.                                                         */
;*---------------------------------------------------------------------*/
(define (break-condition records)
   (let* ((stop-markup     #"\tstop only if ")
	  (stop-markup-len (string-length stop-markup)))
      (let loop ((records records))
	 (cond
	    ((null? records)
	     #f)
	    ((substring=? stop-markup (car records) stop-markup-len)
	     (substring (car records)
			stop-markup-len
			(string-length (car records))))
	    (else
	     (loop (cdr records)))))))
  
;*---------------------------------------------------------------------*/
;*    compare-breakpoint-tables ...                                    */
;*    -------------------------------------------------------------    */
;*    This function reset the *BREAKPOINT-CHANGED?* variable. That is  */
;*    after comparing to breakpoint tables, the breakpoints are as     */
;*    unchanged.                                                       */
;*---------------------------------------------------------------------*/
(define (compare-breakpoint-tables old new)
   (set! *breakpoint-changed?* #f)
   (cond
      ((not (pair? old))
       (if (pair? new)
	   (values '() '() (cdr new))
	   (values '() '() '())))
      ((not (pair? new))
       (values (cdr old) '() '()))
      (else
       (let loop ((orec    (cdr old))
		  (nrec    (cdr new))
		  (removed '())
		  (changed '())
		  (added   '()))
	  (cond
	     ((and (null? orec) (null? nrec))
	      (values removed changed added))
	     ((null? orec)
	      (values removed changed (append nrec added)))
	     ((null? nrec)
	      (values (append orec removed) changed added))
	     (else
	      (let* ((o (car orec))
		     (n (car nrec)))
		 (match-case o
		    ((?on ?- ?- ?oe . ?er)
		     (match-case n
			((?nn ?- ?- ?ne . ?or)
			 (let ((onum (string->integer on))
			       (nnum (string->integer nn)))
 			    (cond
			       ((and (=fx onum nnum)
				     (string=? oe ne)
				     (equal? (break-condition er)
					     (break-condition or)))
				;; nothing changed
				(loop (cdr orec)
				      (cdr nrec)
				      removed
				      changed
				      added))
			       ((=fx onum nnum)
				;; the state has changed
				(loop (cdr orec)
				      (cdr nrec)
				      removed
				      (cons n changed)
				      added))
			       ((<fx onum nnum)
				;; a breakpoint has been remove
				(loop (cdr orec)
				      nrec
				      (cons o removed)
				      changed
				      added))
			       ((>fx onum nnum)
				;; a breakpoint has been added
				(loop orec
				      (cdr nrec)
				      removed
				      changed
				      (cons n added))))))
			(else
			 (loop (cdr orec)
			       (cdr nrec)
			       removed
			       changed
			       added))))
		    (else
		     (loop (cdr orec)
			   (cdr nrec)
			   removed
			   changed
			   added))))))))))

;*---------------------------------------------------------------------*/
;*    condition-parser ...                                             */
;*---------------------------------------------------------------------*/
(define (condition-parser cmd-list source level end)
   (match-case cmd-list
      (() 
       (console-error "Argument required (breakpoint number)."))
      ((?num)
       (breakpoint-changed!)
       (console-echo (gdb-call-for-output source)))
      ((?num . ?-)
       (breakpoint-changed!)
       ;; we seek for the beginning of the expression
       (let* ((bdb-exp (string-from (string-from source #\space 1) #\space 1))
	      (gdb-exp (bdb-expr->gdb-expr bdb-exp (gdb-current-function)))
	      (cmd     (string-append "cond " num " " gdb-exp)))
	  (console-echo "WARNING: THE CONDITION EXPRESSION OF THE BREAKPOINT")
	  (console-newline)
	  (console-echo "IS NOT DEMANGLED IN THE ENVIRONMENT OF THE BREAKPOINT")
	  (console-newline)
	  (console-echo "BUT THE CURRENT ENVIRONMENT. THIS WILL BE FIXED...")
	  (console-newline)
	  (console-echo (gdb-call-for-output cmd))))))
       
;*---------------------------------------------------------------------*/
;*    The break command                                                */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "break"
			1
			(lambda (cmd-list source level env)
			   (break-command-parse "break"
						cmd-list
						source
						level
						env))
"Set breakpoint at specified line or function.
Argument may be line number, function name, or \"*\" and an address.
If line number is specified, break at start of code for that line.
If function is specified, break at start of code for that function.
If an address is specified, break at that exact address.
With no arg, uses current execution address of selected stack frame.
This is useful for breaking on return to a stack frame.

If the function name is upper case, then a breakpoint to a Bigloo
function is set. Otherwise, the name is considered as a C identifier.

Multiple breakpoints at one place are permitted, and useful if conditional.

Do \"help breakpoints\" for info on other commands dealing with breakpoints.")

;*---------------------------------------------------------------------*/
;*    generic-break-command-parse ...                                  */
;*    -------------------------------------------------------------    */
;*    This function is just a wrapper. It calls the real gdb command   */
;*    but it set the BREAKPOINT-CHANGED first.                         */
;*---------------------------------------------------------------------*/
(define (generic-break-command-parse cmd-list source::bstring level::int env)
   (breakpoint-changed!)
   (console-echo (gdb-call-for-output source)))

;*---------------------------------------------------------------------*/
;*    The delete command                                               */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "delete"
			1
			(lambda (cmd-list source level env)
			   (generic-break-command-parse cmd-list
							source
							level
							env))
			"Delete some breakpoints or auto-display expressions.
Arguments are breakpoint numbers with spaces in between.
To delete all breakpoints, give no argument.

Also a prefix command for deletion of other GDB objects.

List of delete subcommands:

delete breakpoints -- Delete some breakpoints or auto-display expressions
delete display -- Cancel some expressions to be displayed when program stops
delete tracepoints -- Delete specified tracepoints

Type \"help delete\" followed by delete subcommand name for full documentation.
Command name abbreviations are allowed if unambiguous.")

;*---------------------------------------------------------------------*/
;*    The enable command                                               */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "enable"
			2
			(lambda (cmd-list source level env)
			   (generic-break-command-parse cmd-list
							source
							level
							env))
			"Enable some breakpoints.
Give breakpoint numbers (separated by spaces) as arguments.
With no subcommand, breakpoints are enabled until you command otherwise.
This is used to cancel the effect of the \"disable\" command.
With a subcommand you can enable temporarily.

List of enable subcommands:

enable delete -- Enable breakpoints and delete when hit
enable display -- Enable some expressions to be displayed when program stops
enable once -- Enable breakpoints for one hit
enable tracepoints -- Enable specified tracepoints

Type \"help enable\" followed by enable subcommand name for full documentation.
Command name abbreviations are allowed if unambiguous.")

;*---------------------------------------------------------------------*/
;*    The disable command                                              */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "disable"
			3
			(lambda (cmd-list source level env)
			   (generic-break-command-parse cmd-list
							source
							level
							env))
			"Disable some breakpoints.
Give breakpoint numbers (separated by spaces) as arguments.
With no subcommand, breakpoints are disabled until you command otherwise.
This is used to cancel the effect of the \"disable\" command.
With a subcommand you can disable temporarily.

List of disable subcommands:

disable delete -- Disable breakpoints and delete when hit
disable display -- Disable some expressions to be displayed when program stops
disable once -- Disable breakpoints for one hit
disable tracepoints -- Disable specified tracepoints

Type \"help disable\" followed by disable subcommand name for full documentation.
Command name abbreviations are allowed if unambiguous.")

;*---------------------------------------------------------------------*/
;*    the condition command                                            */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "condition"
			4
			condition-parser
			"Specify breakpoint number N to break only if COND is true.
Usage is `condition N COND', where N is an integer and COND is an
expression to be evaluated whenever breakpoint N is reached.")
