;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Command/display.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Aug  4 11:13:35 1999                          */
;*    Last change :  Thu Sep 23 09:10:36 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The handling of gdb DISPLAYs.                                    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_display
   (import command_expr
	   tools_error
	   tools_tools
	   tools_io
	   patient_mangling
	   patient_invoke
	   patient_value
	   gdb_tools
	   gdb_invoke
	   gdb_annotate
	   engine_param
	   command_command)
   (export (display-display-table! disp)
	   (display->string disp)
	   (display-number disp)
	   (set-display-display-function! ::procedure)))

;*---------------------------------------------------------------------*/
;*    *display-table* ...                                              */
;*---------------------------------------------------------------------*/
(define *display-table* '())

;*---------------------------------------------------------------------*/
;*    *display-register-mode* ...                                      */
;*---------------------------------------------------------------------*/
(define *display-register-mode* 'echo)

;*---------------------------------------------------------------------*/
;*    *display-display* ...                                            */
;*---------------------------------------------------------------------*/
(define *display-display* console-display-display)

;*---------------------------------------------------------------------*/
;*    set-display-display-function! ...                                */
;*---------------------------------------------------------------------*/
(define (set-display-display-function! fun)
   (set! *display-display* fun))

;*---------------------------------------------------------------------*/
;*    display-number ...                                               */
;*---------------------------------------------------------------------*/
(define (display-number disp)
   (match-case disp
      ((?num ?fmt ?expr ?value)
       (string->integer num))
      (else
       0)))
   
;*---------------------------------------------------------------------*/
;*    display->string ...                                              */
;*---------------------------------------------------------------------*/
(define (display->string disp)
   (match-case disp
      ((?num ?fmt ?expr ?value)
       (string-append num ": " expr " = " value))
      (else
       "")))
   
;*---------------------------------------------------------------------*/
;*    console-display-display ...                                      */
;*---------------------------------------------------------------------*/
(define (console-display-display disp)
   (define (display-display disp)
      (console-echo (display->string disp))
      (console-newline))
   (for-each display-display disp))
	 
;*---------------------------------------------------------------------*/
;*    display-display-table! ...                                       */
;*---------------------------------------------------------------------*/
(define (display-display-table! disp)
   (case *display-register-mode*
      ((eager)
       (console-display-display disp))
      ((echo)
       (*display-display* disp))))

;*---------------------------------------------------------------------*/
;*    display-parser ...                                               */
;*---------------------------------------------------------------------*/
(define (display-parser cmd-list source level env)
   (let ((mode *display-register-mode*))
      (set! *display-register-mode* 'eager)
      (if (null? cmd-list)
	  (console-echo (gdb-call-for-output source))
	  (let* ((bdb-expr (string-from-at source 1 #\space 1))
		 (gdb-expr (bdb-expr->gdb-expr bdb-expr
					       (gdb-current-function))))
	     ;; if bdb-expr and gdb-expr are the same
	     ;; it is a real displaying...
	     (if (string=? bdb-expr gdb-expr)
		 (console-echo (gdb-call-for-output source))
		 ;; they differ, it is a Bigloo display
		 (let* ((cmd (string-append "display (char *)bdb_print( "
					    gdb-expr
					    (if *write-circle?*
						", 1"
						", 0")
					    " )")))
		    (console-echo (gdb-call-for-output cmd))))))
      (set! *display-register-mode* mode)))

;*---------------------------------------------------------------------*/
;*    *display-help* ...                                               */
;*---------------------------------------------------------------------*/
(define *display-help*
   "Print value of expression EXP each time the program stops.")

;*---------------------------------------------------------------------*/
;*    *cdisplay-help* ...                                              */
;*---------------------------------------------------------------------*/
(define *cdisplay-help*
   "Print value of expression EXP each time the program stops.
/FMT may be used before EXP as in the \"print\" command.
/FMT \"i\" or \"s\" or including a size-letter is allowed,
as in the \"x\" command, and then EXP is used to get the address to examine
and examining is done as in the \"x\" command.

With no argument, display all currently requested auto-display expressions.
Use \"undisplay\" to cancel display requests previously made.")
   
;*---------------------------------------------------------------------*/
;*    The display command                                              */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "display"
			4
			display-parser
			*display-help*)

;*---------------------------------------------------------------------*/
;*    The cdisplay command                                             */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "cdisplay"
			5
			(lambda (cmd source level env)
			   (console-echo
			    (gdb-call-for-output
			     (substring source 1 (string-length source)))))
			*cdisplay-help*)
