;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Command/explore.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Aug  5 15:42:04 1999                          */
;*    Last change :  Mon Oct  4 13:31:30 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The "explore" command implementation.                            */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_explore
   (import command_expr
	   tools_error
	   tools_tools
	   tools_io
	   patient_mangling
	   patient_invoke
	   patient_value
	   gdb_tools
	   gdb_invoke
	   gdb_annotate
	   engine_param
	   command_command)
   (export (will-explore-value! ::bstring)))

;*---------------------------------------------------------------------*/
;*    console-print ...                                                */
;*---------------------------------------------------------------------*/
(define (console-print . obj)
   (for-each console-echo obj)
   (console-newline))

;*---------------------------------------------------------------------*/
;*    explore-tty-echo ...                                             */
;*---------------------------------------------------------------------*/
(define (explore-tty-echo explore)
   (define (explore-link-chain/base link base)
      (for-each (lambda (x)
		   (console-echo "   ")
		   (if (substring=? "0x" x 2)
		       (let ((base (string-until x #\space 0)))
			  (console-echo-explore base base)
			  (console-echo (string-from x #\space 0)))
		       (console-echo x))
		   (console-newline))
		link))
   (define (explore-link-chain/w-base link base)
      (for-each (lambda (x)
		   (console-echo "   ")
		   (let ((str (string-from x #\[ 0)))
		      (if (substring=? "0x" x 2)
			  (let ((base (string-until x #\space 0)))
			     (console-echo-explore str base))
			  (console-echo str)))
		   (console-newline))
		link))
   (define (explore-link-chain link base)
      (if *base-address?*
	  (explore-link-chain/base link base)
	  (explore-link-chain/w-base link base)))
   (match-case explore
      ((?kind ?gen ?size ?fun ?lnum ?type ?link ?base (?file . ?line))
       (case kind
	  ((0)
	   (console-print "Unreferenced object! ("
			  (integer->string base 16) ")"))
	  ((1)
	   (console-print "Undebugged object! ("
			  (integer->string base 16) ")"))
	  (else
	   (console-echo "type          : ")
	   (if (string=? type "<string buffer>")
	       (console-print (integer->string base 16))
	       (begin
		  (console-echo-dprint type
				       (string-append
					"0x"
					(let ((s (integer->string base 16)))
					   (substring s 2 (string-length s)))))
		  (console-newline)))
	   (console-echo "Producer      : ")
	   (cond
	      ((substring=? fun "BDB:" 4)
	       (console-print fun))
	      ((=fx line 0)
	       (console-echo-edit fun fun)
	       (console-newline))
	      (else
	       (let ((edit (string-append file ":" (integer->string line))))
		  (console-echo-edit fun edit)
		  (console-newline))))
	   (console-print "Generation    : " (integer->string gen))
	   (console-print "Size in bytes : " (integer->string size))
	   (if *base-address?*
	       (begin
		  (console-echo "base address  : " )
		  (if (string=? type "<string buffer>")
		      (console-print (integer->string base 16))
		      (let ((num (integer->string base 16)))
			 (console-echo-dprint
			  (integer->string base 16)
			  (string-append
			   "0x"
			   (let ((s (integer->string base 16)))
			      (substring s 2 (string-length s)))))
			 (console-newline)))))
	   (explore-link-chain link base))))))

;*---------------------------------------------------------------------*/
;*    *bigloo-type-shape* ...                                          */
;*---------------------------------------------------------------------*/
(define *bigloo-type-shape* (string-append "(" *bigloo-type* ")"))

;*---------------------------------------------------------------------*/
;*    *will-explore-value* ...                                         */
;*---------------------------------------------------------------------*/
(define *will-explore-value* #unspecified)

;*---------------------------------------------------------------------*/
;*    will-explore-value! ...                                          */
;*---------------------------------------------------------------------*/
(define (will-explore-value! value::bstring)
   (set! *will-explore-value* value))

;*---------------------------------------------------------------------*/
;*    explore-value ...                                                */
;*---------------------------------------------------------------------*/
(define (explore-value value::bstring)
   (if (not (substring=? *bigloo-type-shape*
			 value
			 (string-length *bigloo-type-shape*)))
       (begin
	  (console-echo "No heap value")
	  (console-newline))
       (let ((len (string-length value)))
	  (let loop ((i (-fx len 1)))
	     (let ((c (string-ref value i)))
		(if (or (char=? c #\space) (char=? c #\)))
		    (let* ((addr (substring value (+fx i 1) len))
			   (explore (patient-call "bdb_heap_explore"
						  addr
						  (if (>=fx *verbose* 4)
						      "1"
						      "0"))))
		       (explore-tty-echo explore))
 		    (loop (-fx i 1))))))))

;*---------------------------------------------------------------------*/
;*    explore-parser ...                                               */
;*---------------------------------------------------------------------*/
(define (explore-parser cmd-list source level env)
   (set! *will-explore-value* #f)
   (if (null? cmd-list)
       (bdb-notify-error "explore" "Missing arguments" source)
       (let* ((bdb-expr (string-from-at source 1 #\space 1))
	      (gdb-expr (bdb-expr->gdb-expr bdb-expr (gdb-current-function))))
	  ;; if bdb-expr and gdb-expr are the same it is a real exploring...
	  (gdb-enable-explore-print!)
	  (let ((cmd (string-append "print " gdb-expr)))
	     (gdb-call-for-output cmd)
	     (if (string? *will-explore-value*)
		 (begin
		    (explore-value *will-explore-value*)
		    (set! *will-explore-value* #f))))
	  (gdb-disable-print!))))

;*---------------------------------------------------------------------*/
;*    explore-value-offset ...                                         */
;*---------------------------------------------------------------------*/
(define (explore-value-offset addr::bstring)
   (let ((explore (patient-call "bdb_heap_explore_offset"
				addr
				(if (>=fx *verbose* 4) "1" "0"))))
      (explore-tty-echo explore)))
   
;*---------------------------------------------------------------------*/
;*    explore-offset-parser ...                                        */
;*---------------------------------------------------------------------*/
(define (explore-offset-parser cmd-list source level env)
   (set! *will-explore-value* #f)
   (if (null? cmd-list)
       (bdb-notify-error "explore_offset" "Missing arguments" source)
       (let ((gdb-expr (string-from-at source 1 #\space 1)))
	  (gdb-enable-explore-print!)
	  (let ((cmd (string-append "print " gdb-expr)))
	     (gdb-call-for-output cmd)
	     (if (string? *will-explore-value*)
		 (begin
		    (explore-value-offset *will-explore-value*)
		    (set! *will-explore-value* #f))))
	  (gdb-disable-print!))))

;*---------------------------------------------------------------------*/
;*    *explore-help* ...                                               */
;*---------------------------------------------------------------------*/
(define *explore-help*
   "Explore in the heap the value of expression EXP.
Variables accessible are those of the lexical environment of the selected
stack frame, plus all those whose scope is global or an entire file.")

;*---------------------------------------------------------------------*/
;*    *explore-offset-help* ...                                        */
;*---------------------------------------------------------------------*/
(define *explore-offset-help*
   "Private command.")

;*---------------------------------------------------------------------*/
;*    The print command                                                */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "explore"
			3
			explore-parser
			*explore-help*)

;*---------------------------------------------------------------------*/
;*    The print command                                                */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "explore_offset"
			9
			explore-offset-parser
			*explore-offset-help*)

