;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/kbdb/Command/file.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 16:00:47 1999                          */
;*    Last change :  Tue Aug 17 18:51:42 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The file command implementation                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_file
   (import command_command
	   command_expr
	   gdb_invoke
	   engine_param
	   patient_mangling
	   tools_io
	   (gdb-reset-stack! gdb_annotate)
	   (read-etags-file bee_etags))
   (export (set-previous-file! ::bstring)
	   (get-previous-file)))

;*---------------------------------------------------------------------*/
;*    *previous-file* ...                                              */
;*---------------------------------------------------------------------*/
(define *previous-file* #unspecified)

;*---------------------------------------------------------------------*/
;*    set-previous-file! ...                                           */
;*---------------------------------------------------------------------*/
(define (set-previous-file! file)
   (set! *previous-file* file))

;*---------------------------------------------------------------------*/
;*    get-previous-file ...                                            */
;*---------------------------------------------------------------------*/
(define (get-previous-file)
   *previous-file*)
	 
;*---------------------------------------------------------------------*/
;*    make-file-callback ...                                           */
;*---------------------------------------------------------------------*/
(define (make-file-callback file)
   ;; first, we register the file for a futur file command
   (set-previous-file! file)
   ;; then emit the command
   (console-echo (gdb-call-for-output (string-append "file " file)))
   ;; we reset the whole thing
   (gdb-reset-stack!)
   ;; try to re-read the etags description
   (read-etags-file))

;*---------------------------------------------------------------------*/
;*    file-command-parse ...                                           */
;*---------------------------------------------------------------------*/
(define (file-command-parse cmd-list source::bstring level::int env)
   (match-case cmd-list
      (()
       ;; we cannot simply invoke the bdb command, we have to
       ;; check first if a file has already been loaded. If it
       ;; has, file with no argument means re-load that file
       (if (string? *previous-file*)
	   ;; a reload
	   (make-file-callback *previous-file*)
	   ;; file with no argument! After all, why not...
	   (begin
	      (console-echo (gdb-call-for-output "file"))
	      (gdb-reset-stack!)
	      ;; try to re-read the etags description
	      (read-etags-file))))
      ((?file . ?-)
       ;; because this is a change of file, we have to forget
       ;; about everything we know of name demangling.
       (reset-demangling-cache!)
       ;; we also have to reset the expression table
       (reset-expression-table!)
       ;; we do invoke the gdb command
       (make-file-callback file))))

;*---------------------------------------------------------------------*/
;*    The file command                                                 */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "file"
			3
			file-command-parse
			"Use FILE as program to be debugged.
It is read for its symbols, for getting the contents of pure memory,
and it is the program executed when you use the `run' command.
If FILE cannot be found as specified, your execution directory path
($PATH) is searched for a command of that name.
No arg means to have no executable file and no symbols.")
