;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Command/help.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 11:40:27 1999                          */
;*    Last change :  Wed Sep 15 16:37:13 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The help command (it is really necessary to bother with it...).  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module command_help
   (import command_command
	   gdb_invoke
	   engine_param
	   tools_io))

;*---------------------------------------------------------------------*/
;*    print-command-help ...                                           */
;*---------------------------------------------------------------------*/
(define (print-command-help cmd::command)
   (with-access::command cmd (name help)
      (console-echo name)
      (console-echo " -- ")
      (console-echo help)
      (console-newline)))
   
;*---------------------------------------------------------------------*/
;*    main-help-callback ...                                           */
;*---------------------------------------------------------------------*/
(define (main-help-callback)
   (lambda ()
      (console-echo (gdb-call-for-output "help"))))

;*---------------------------------------------------------------------*/
;*    gdb-help-callback ...                                            */
;*---------------------------------------------------------------------*/
(define (gdb-help-callback line)
   (lambda ()
      (gdb-call-for-output line)))

;*---------------------------------------------------------------------*/
;*    sub-help-command ...                                             */
;*---------------------------------------------------------------------*/
(define (sub-help-command sub-cmd)
   (lambda ()
      ;; display the information relative to that command
      (print-command-help sub-cmd)
      (with-access::command sub-cmd (name env)
	 (if (pair? env)
	     (begin
		;; they are subcommands
		(console-newline)
		(console-echo "List of ")
		(console-echo name)
		(console-echo " subcommands:")
		(console-newline)
		(console-newline)
		(for-each (lambda (cmd)
			     (with-access::command cmd (help)
				(console-echo help)
				(console-newline)))
			  env))))))

;*---------------------------------------------------------------------*/
;*    help-command-parse ...                                           */
;*---------------------------------------------------------------------*/
(define (help-command-parse cmd-list source::bstring level::int env)
   (let loop ((cmd-list  cmd-list)
	      (help      (main-help-callback))
	      (env       (toplevel-command-env))
	      (err-level level))
      (if (null? cmd-list)
	  (help)
	  (let* ((sub-cmd-id (car cmd-list))
		 (sub-cmd    (find-command sub-cmd-id env)))
	     (if (not (command? sub-cmd))
		 (gdb-help-callback source)
		 (loop (cdr cmd-list)
		       (sub-help-command sub-cmd)
		       (command-env sub-cmd)
		       (+fx err-level 1)))))))

;*---------------------------------------------------------------------*/
;*    The help command                                                 */
;*---------------------------------------------------------------------*/
(bind-toplevel-command! "help"
			1
			help-command-parse
			"Print list of commands.")
			   
	   
