;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Engine/engine.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 09:27:53 1999                          */
;*    Last change :  Tue Aug 31 12:04:29 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The bdb engine (i.e. whatever to be done before the repl).       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module engine_engine
   (include "Engine/hooks.sch")
   (extern  (include "signal.h"))
   (import  engine_param
	    engine_repl
	    tools_error
	    gdb_proc
	    gdb_invoke
	    gdb_annotate
	    patient_server
	    bee_etags
	    bee_invoke
	    command_run)
   (export  (engine ::procedure)
	    (bdb-exit ::int)
	    (add-quit-hook! ::procedure)))

;*---------------------------------------------------------------------*/
;*    *quit-hooks* ...                                                 */
;*    -------------------------------------------------------------    */
;*    Hooks to be executed on bdb termination.                         */
;*---------------------------------------------------------------------*/
(define *quit-hooks* '())

;*---------------------------------------------------------------------*/
;*    add-quit-hook! ...                                               */
;*---------------------------------------------------------------------*/
(define (add-quit-hook! fun::procedure)
   (set! *quit-hooks* (cons fun *quit-hooks*)))

;*---------------------------------------------------------------------*/
;*    engine ...                                                       */
;*---------------------------------------------------------------------*/
(define (engine repl)
   ;; Bee acknowledgement
   (if *bee-client?*
       (bee-notify "INITIALIZATION"))
   ;; we setup the signal handling (that is C-c)
   (init-signal-handler!)
   ;; we if we debugging a Bigloo program, we add an run hook in order
   ;; to set a breakpoint in the error notifying function
   (add-run-hook! (lambda ()
		     (if *bigloo-enabled?*
			 (let ((li (string-append "break " *bigloo-failure*)))
			    ;; we must not register that breakpoint in the
			    ;; history list because run always sets it!
			    (gdb-call-ignore-output li)
			    ;; whatever happened during the breakpoint, we
			    ;; don't print the result of the command
			    (gdb-command-status #f)))))
   ;; try to read the etags description
   (read-etags-file)
   ;; we prepare for the exit machinery
   (bind-exit (exit)
      (set! *bdb-exit* exit)
      ;; then we start the gdb process
      (if (gdb-start)
	  (unwind-protect
	     (begin
		;; before entering the repl, we have to execute
		;; all the pre hooks and the post hooks in order
		;; to ensure a correct initialization.
		(run-hooks *pre-command-hook*)
		(run-hooks *post-command-hook*)
		(repl))
	     (begin
		(process-send-signal (get-gdb-process) SIGINT)
		(gdb-call-ignore-output "kill")
		(gdb-stop)
		(run-hooks *quit-hooks*)
		(close-server!))))))

;*---------------------------------------------------------------------*/
;*    *bdb-exit* ...                                                   */
;*    -------------------------------------------------------------    */
;*    This variable is set when the repl is started. It is a pointer   */
;*    to a lexical escape from the repl or the Bigloo exit function.   */
;*---------------------------------------------------------------------*/
(define *bdb-exit* #f)

;*---------------------------------------------------------------------*/
;*    bdb-exit ...                                                     */
;*---------------------------------------------------------------------*/
(define (bdb-exit val)
   (if (procedure? *bdb-exit*)
       (*bdb-exit* val)
       (*exit* val)))

;*---------------------------------------------------------------------*/
;*    init-signal-handler! ...                                         */
;*---------------------------------------------------------------------*/
(define (init-signal-handler!)
   (let ((kill-my-self (lambda (n)
                          (bdb-error "engine"
				     "Illegal signal caught, aborting"
				     n)
                          (exit -1))))
      (signal sigfpe kill-my-self)
      (signal sigill kill-my-self)
      (signal sigbus kill-my-self)
      (signal sigsegv kill-my-self)
      (signal sigint (lambda (signal) 'ignore))))
   
