;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Engine/param.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 07:15:39 1999                          */
;*    Last change :  Mon Oct  4 11:52:04 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Configuration variables                                          */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module engine_param
   (import  tools_date)
   (export  ;; version and author
            *bdb-version*
	    *bdb-name*
	    *bdb-level*
	    *bdb-author*
	    *bdb-email*
	    *bdb-date*
	    ;; gdb settings
	    *gdb*
	    *gdb-options*
	    *gdb-prompt*
	    ;; gdb frame arguments parsing
	    *gdb-par-clo*
	    *gdb-arg-begin*
	    *gdb-no-arg*
	    *gdb-arg-name*
	    *gdb-arg-=*
	    *gdb-arg-,*
	    *gdb-arg-end*
	    *gdb-arg-value*
	    ;; arguments parsing
	    *exec*
	    *args*
	    *root-directory*
	    *afile*
	    *etags*
	    ;; misc
	    *verbose*
	    *active-prompt*
	    *bigloo-type*
	    *bigloo-failure*
	    *pre-command-hook*
	    *post-command-hook*
	    *exit*
	    *gdb-run-hook*
	    ;; programming environment
	    *bee-client?*
	    *emacs-client?*
	    *bigloo-enabled?*
	    *heap-explorer?*
	    *base-address?*
	    *force-tty?*
	    *tty-init-hook*
	    ;; pretty printing
	    *write-circle?*
	    ;; input and output
	    *prompt*
	    *pin*
	    *pout*
	    *perr*
	    *bout*
	    *bin*))

;*---------------------------------------------------------------------*/
;*    Version and authoring ...                                        */
;*---------------------------------------------------------------------*/
(define *bdb-version*       "1.0")
(define *bdb-level*         #\a)
(define *bdb-name*          (string-append "Bdb (v" *bdb-version*")"))
(define *bdb-author*        "Manuel Serrano")
(define *bdb-email*         "Manuel.Serrano@unice.fr")
(define *bdb-date*          (bdb-date))

;*---------------------------------------------------------------------*/
;*    Gdb configuration                                                */
;*---------------------------------------------------------------------*/
(define *gdb*               "gdb")
(define *gdb-options*       '("--quiet" "--fullname" "--nx" "--annotate=2"))
(define *gdb-prompt*        '(#"pre-prompt\n" #"(gdb) \n" #"prompt\n"))

;*---------------------------------------------------------------------*/
;*    Gdb argument frame parsing                                       */
;*---------------------------------------------------------------------*/
(define *gdb-par-clo*       #")\n")
(define *gdb-no-arg*        #" ()\n")
(define *gdb-arg-begin*     #"arg-begin\n")
(define *gdb-arg-name*      #"arg-name-end\n")
(define *gdb-arg-=*         #"=\n")
(define *gdb-arg-,*         #", \n")
(define *gdb-arg-value*     #"arg-value")
(define *gdb-arg-end*       #"arg-end\n")

;*---------------------------------------------------------------------*/
;*    Arguments parsing ...                                            */
;*---------------------------------------------------------------------*/
(define *exec*              #f)
(define *args*              '())
(define *root-directory*    ".")
(define *afile*             ".afile")
(define *etags*             ".etags")

;*---------------------------------------------------------------------*/
;*    Misc                                                             */
;*---------------------------------------------------------------------*/
(define *verbose*           0)
(define *active-prompt*     #t)
(define *bigloo-type*       "union scmobj *")
(define *bigloo-failure*    "bigloo_abort")
(define *post-command-hook* '())
(define *pre-command-hook*  '())
(define *exit*              exit)
(define *gdb-run-hook*      '())

;*---------------------------------------------------------------------*/
;*    Programming environment                                          */
;*---------------------------------------------------------------------*/
(define *bee-client?*       #f)
(define *emacs-client?*     #f)
(define *bigloo-enabled?*   #t)
(define *heap-explorer?*    #t)
(define *force-tty?*        #f)
(define *tty-init-hook*     '())
(define *base-address?*     #f)

;*---------------------------------------------------------------------*/
;*    Pretty printing                                                  */
;*---------------------------------------------------------------------*/
(define *write-circle?*     #t)

;*---------------------------------------------------------------------*/
;*    Input and output                                                 */
;*---------------------------------------------------------------------*/
(define *prompt*            "(bdb) ")
(define *pin*               (current-input-port))
(define *pout*              (current-output-port))
(define *perr*              (current-error-port))
(define *bout*              (current-output-port))
(define *bin*               (current-input-port))

