;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Gdb/annotate.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 17:27:11 1999                          */
;*    Last change :  Mon Oct  4 12:01:40 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The GDB annotate output command handling.                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module gdb_annotate
   (import gdb_invoke
	   tools_tools
	   tools_speek
	   tools_io
	   engine_param
	   command_backtrace
	   command_print
	   command_display
	   command_explore
	   gdb_proc
	   patient_mangling
	   (gdb-expr->bdb-expr command_expr))
   (export (find-annotate-parser line)
	   (gdb-reset-stack-frames!)
	   (gdb-reset-stack!)
	   (gdb-command-status ::bool)
	   (gdb-error?)
	   (gdb-stack-frames)
	   (gdb-breakpoint-table)
	   (gdb-current-function)
	   (gdb-current-frame)
	   (gdb-restore-current-frame! frame)
	   (gdb-current-frame-number::int)
	   (gdb-enable-print!)
	   (gdb-enable-bigloo-print!)
	   (gdb-enable-explore-print!)
	   (gdb-disable-print!)
	   (gdb-read-error)))

;*---------------------------------------------------------------------*/
;*    *gdb-stopped?* ...                                               */
;*    -------------------------------------------------------------    */
;*    That variable is true iff a command as raised an stopped. This   */
;*    is used to find out the kind of status that must be displayed.   */
;*---------------------------------------------------------------------*/
(define *gdb-stopped?*       #f)
(define *gdb-signal?*        #f)
(define *gdb-stopped-output* #f)
(define *gdb-started?*       #f)
(define *gdb-exited?*        #f)

;*---------------------------------------------------------------------*/
;*    *gdb-print?* ...                                                 */
;*---------------------------------------------------------------------*/
(define *gdb-print?*         #f)

;*---------------------------------------------------------------------*/
;*    gdb-enable-print! ...                                            */
;*---------------------------------------------------------------------*/
(define (gdb-enable-print!)
   (set! *gdb-print?* #t))

;*---------------------------------------------------------------------*/
;*    gdb-enable-bigloo-print! ...                                     */
;*---------------------------------------------------------------------*/
(define (gdb-enable-bigloo-print!)
   (set! *gdb-print?* 'bigloo))

;*---------------------------------------------------------------------*/
;*    gdb-enable-explore-print! ...                                    */
;*---------------------------------------------------------------------*/
(define (gdb-enable-explore-print!)
   (set! *gdb-print?* 'explore))

;*---------------------------------------------------------------------*/
;*    gdb-disable-print! ...                                           */
;*---------------------------------------------------------------------*/
(define (gdb-disable-print!)
   (set! *gdb-print?* #f))

;*---------------------------------------------------------------------*/
;*    gdb-command-status ...                                           */
;*    -------------------------------------------------------------    */
;*    After the invocation of a GDB command, that function display     */
;*    the new status. This function works in collaboration with the    */
;*    stopped parser.                                                  */
;*---------------------------------------------------------------------*/
(define (gdb-command-status verb?::bool)
   (define (reset-command-status!)
      (set! *gdb-stopped-output* #f)
      (set! *gdb-source-line* #unspecified)
      (set! *gdb-stack-frame-parsed?* #f)
      (set! *gdb-error?* #f)
      (set! *gdb-exited?* #f)
      (set! *gdb-signal?* #f)
      (gdb-disable-print!))
   (cond
      (*gdb-exited?*
       ;; only reset then
       (reset-command-status!))
      (*gdb-error?*
       ;; reset then
       (reset-command-status!)
       ;; read the error
       (let ((err (gdb-read-error)))
	  (if verb?
	      (console-error err))))
      ((not verb?)
       ;; a simple reset
       (reset-command-status!))
      (else
       (let ((gdb-source-line *gdb-source-line*)
	     (gdb-stack-frame-parsed? *gdb-stack-frame-parsed?*)
	     (gdb-stack-source *gdb-stack-source*)
	     (gdb-current-stack-frame *gdb-current-stack-frame*)
	     (gdb-stopped-output *gdb-stopped-output*)
	     (gdb-stopped? *gdb-stopped?*)
	     (gdb-started? *gdb-started?*)
	     (gdb-signal? *gdb-signal?*))
	  (reset-command-status!)
	  (verbose 5 "status: " gdb-stopped? " " gdb-started? " "
		   gdb-stopped-output
		   " " gdb-source-line #\Newline)
	  (if (and (not gdb-stopped?) (not gdb-started?)
		   (or (not gdb-signal?) (not *bigloo-enabled?*)))
	      ;; this means that we have parsed a frame without emitting
	      ;; a started command, we thus display that frame.
	      (display-frame gdb-current-stack-frame)
	      (if (pair? gdb-stopped-output)
		  (match-case gdb-stopped-output
		     ((breakpoint ?break ?fun ?args ?file ?num)
		      (console-echo break)
		      ;; The function where the breakpoint occured.
		      ;; We can't demangle the name if we have stopped
		      ;; because of a signal.
		      (let ((i (and (or (not gdb-signal?)
					(not *bigloo-enabled?*))
				    (bdb-demangle fun))))
			 (if (string? i)
			     (console-echo i)
			     (console-echo fun)))
		      (if (pair? args)
			  (console-echo "(...)")
			  (console-echo "()"))
		      (console-echo " at ")
		      (let ((str (string-append file ":" num)))
			 (console-echo-edit str str))
		      (console-newline))
		     ((frame ?frame)
		      (if (or (not gdb-signal?)
			      (not *bigloo-enabled?*))
			  (display-frame frame))))))
	  ;; we source line where we have stopped
	  (cond
	     ((and gdb-signal? *bigloo-enabled?*)
	      ;; when we have stopped because of a signal we can't
	      ;; do much (in particular, we can't demangle), thus we
	      ;; store that bdb has lost the stack...
	      (gdb-reset-stack-frames!)
	      (gdb-reset-stack!))
	     ((string? gdb-source-line)
	      ;; displaying the source line takes in charge the
	      ;; display of the newline
	      (console-source gdb-source-line))
	     (gdb-stack-frame-parsed?
	      (if (pair? gdb-stack-source)
		  (begin
		     (console-echo (car gdb-stack-source))
		     (console-echo ":")
		     (console-echo (cdr gdb-stack-source)))
		  (console-echo "No such file or directory."))
	      (console-newline)))))))
	  
;*---------------------------------------------------------------------*/
;*    find-annotate-parser ...                                         */
;*---------------------------------------------------------------------*/
(define (find-annotate-parser line)
   (define (substring-assoc obj alist)
      (let loop ((alist alist))
	 (if (pair? alist)
	     (if (substring=? (caar alist) obj (string-length (caar alist)))
		 (car alist)
		 (loop (cdr alist)))
	     #f)))
   (let ((cell (substring-assoc line *annotate-parser*)))
      (if (pair? cell)
	  (cdr cell)
	  #f)))

;*---------------------------------------------------------------------*/
;*    *annotate-parser* ...                                            */
;*---------------------------------------------------------------------*/
(define *annotate-parser* '())

;*---------------------------------------------------------------------*/
;*    add-annotate-parser! ...                                         */
;*---------------------------------------------------------------------*/
(define (add-annotate-parser! start-markup parser)
   (set! *annotate-parser*
	 (cons (cons start-markup parser) *annotate-parser*)))

;*---------------------------------------------------------------------*/
;*    The variables that hold informations about the stack frames.     */
;*---------------------------------------------------------------------*/
;; The stack frame. This variable is reset each time we set the
;; stack frame numbered 0 in FRAME-BEGIN-PARSER
(define *gdb-stack-frames*        '())
;; the current stack frame (not necessarily the top of stack one).
(define *gdb-current-stack-frame* #f)
;; this variable is #t iff we have parsed a stack frame
(define *gdb-stack-frame-parsed?* #f)
;; The other variables are temporary, they are used to construct the
;; stack frame. The value of these variables can't be used by someone
;; out of this module
(define *gdb-stack-function* #unspecified)
(define *gdb-stack-args*     #unspecified)
(define *gdb-stack-source*   #unspecified)

;*---------------------------------------------------------------------*/
;*    gdb-stack-frames ...                                             */
;*---------------------------------------------------------------------*/
(define (gdb-stack-frames)
   *gdb-stack-frames*)

;*---------------------------------------------------------------------*/
;*    gdb-reset-stack-frames! ...                                      */
;*---------------------------------------------------------------------*/
(define (gdb-reset-stack-frames!)
   (set! *gdb-stack-source* #unspecified)
   (set! *gdb-stack-frames* '()))

;*---------------------------------------------------------------------*/
;*    gdb-reset-stack! ...                                             */
;*---------------------------------------------------------------------*/
(define (gdb-reset-stack!)
   (gdb-reset-stack-frames!)
   (set! *gdb-current-stack-frame* #f))

;*---------------------------------------------------------------------*/
;*    gdb-top-stack-frame ...                                          */
;*    -------------------------------------------------------------    */
;*    Returns a description of the top stack frames.                   */
;*---------------------------------------------------------------------*/
(define (gdb-top-stack-frame)
   (let ((stack (gdb-stack-frames)))
      (if (pair? stack)
	  (car (last-pair stack))
	  #f)))

;*---------------------------------------------------------------------*/
;*    gdb-current-function ...                                         */
;*---------------------------------------------------------------------*/
(define (gdb-current-function)
   (if (pair? *gdb-current-stack-frame*)
       (car *gdb-current-stack-frame*)
       #f))

;*---------------------------------------------------------------------*/
;*    gdb-current-frame ...                                            */
;*---------------------------------------------------------------------*/
(define (gdb-current-frame)
   *gdb-current-stack-frame*)

;*---------------------------------------------------------------------*/
;*    gdb-restore-current-frame! ...                                   */
;*---------------------------------------------------------------------*/
(define (gdb-restore-current-frame! frame)
   (set! *gdb-current-stack-frame* frame))

;*---------------------------------------------------------------------*/
;*    gdb-current-frame-number ...                                     */
;*---------------------------------------------------------------------*/
(define (gdb-current-frame-number::int)
   (if (pair? *gdb-current-stack-frame*)
       (stack-frame-number (car *gdb-current-stack-frame*))
       -1))

;*---------------------------------------------------------------------*/
;*    *gdb-breakpoint* ...                                             */
;*---------------------------------------------------------------------*/
(define *gdb-breakpoint* #unspecified)

;*---------------------------------------------------------------------*/
;*    *gdb-source-line* ...                                            */
;*---------------------------------------------------------------------*/
(define *gdb-source-line* #unspecified)

;*---------------------------------------------------------------------*/
;*    *gdb-error?* ...                                                 */
;*---------------------------------------------------------------------*/
(define *gdb-error?* #f)

;*---------------------------------------------------------------------*/
;*    gdb-error? ...                                                   */
;*---------------------------------------------------------------------*/
(define (gdb-error?)
   *gdb-error?*)

;*---------------------------------------------------------------------*/
;*    read-one-arg ...                                                 */
;*---------------------------------------------------------------------*/
(define (read-one-arg)
   ;; we read its name first
   (let* ((line (ignore-one-line-parser))
	  (name (string-until line #\Newline 0)))
      ;; we ignore the arg-name line
      (ignore-one-line-parser)
      ;; we ignore the = line
      (ignore-one-line-parser)
      ;; we read the value markup
      (let ((line (ignore-one-line-parser)))
	 (if (substring=? *gdb-arg-value*
			  line
			  (string-length *gdb-arg-value*))
	     (let* ((line  (ignore-one-line-parser))
		    (value (string-until line #\Newline 0)))
		;; we check for the correct completion
		(let ((line (ignore-one-line-parser)))
		   (if (substring=? *gdb-arg-end*
				    line
				    (string-length *gdb-arg-end*))
		       (cons name value)
		       (error "frame-args-parser"
			      "Illegal line (arg-name-end expected)"
			      (string-until line #\Newline 0)))))
	     (error "frame-args-parser"
		    "Illegal line (arg-value expected)"
		    (string-until line #\Newline 0))))))

;*---------------------------------------------------------------------*/
;*    frame-args-parser ...                                            */
;*    -------------------------------------------------------------    */
;*    Parse an annotated frame description. We do the parsing here     */
;*    because it is much easier to fetch the argument values when      */
;*    written in that format than using the regular `info args'        */
;*    command. In consequence, `info args' simply uses the result      */
;*    of the following parsing.                                        */
;*---------------------------------------------------------------------*/
(define (frame-args-parser _)
   ;; first we skip the par-open line
   (let ((line (ignore-one-line-parser)))
      (if (substring=? *gdb-no-arg* line (string-length *gdb-no-arg*))
	  (set! *gdb-stack-args* '())
	  (let ((markup *gdb-par-clo*)
		(len    (string-length *gdb-par-clo*)))
	     ;; then, we read the argument until the par-clo line
	     (let loop ((line (ignore-one-line-parser))
			(args '()))
		(cond
		   ((=fx (string-length line) 0)
		    (loop (ignore-one-line-parser) 
			  args))
		   ((substring=? markup line len)
		    ;; we are done, we have parser all the arguments
		    (set! *gdb-stack-args* (reverse! args))
		    #f)
		   ((substring=? *gdb-arg-begin*
				 line
				 (string-length *gdb-arg-begin*))
		    ;; this is a new argument
		    (let ((a (read-one-arg)))
		       (loop (ignore-one-line-parser)
			     (cons a args))))
		   ((substring=? *gdb-arg-,* line (string-length *gdb-arg-,*))
		    ;; this is an argument separator
		    (loop (ignore-one-line-parser)
			  args))
		   (else
		    (error "frame-args-parser"
			   "Illegal line (arg-begin or , exptected)"
			   (string-until line #\Newline 0)))))))))

;*---------------------------------------------------------------------*/
;*    breakpoints-invalid-parser ...                                   */
;*    -------------------------------------------------------------    */
;*    Display everything until the next command.                       */
;*---------------------------------------------------------------------*/
(define (breakpoints-invalid-parser _)
   'nothing-at-all)

;*---------------------------------------------------------------------*/
;*    breakpoint-parser ...                                            */
;*---------------------------------------------------------------------*/
(define (breakpoint-parser _)
   (ignore-one-line-parser)
   (set! *gdb-breakpoint* (string-until (ignore-one-line-parser) #\Newline 0))
   #f)

;*---------------------------------------------------------------------*/
;*    frame-source-begin-parser ...                                    */
;*---------------------------------------------------------------------*/
(define (frame-source-begin-parser _)
   ;; the at line
   (ignore-one-line-parser)
   ;; the frame-source-file line
   (ignore-one-line-parser)
   ;; the file name
   (let* ((line (ignore-one-line-parser))
	  (file (string-until line #\Newline 0)))
      ;; the end marker
      (ignore-one-line-parser)
      ;; the colon line
      (let ((line (ignore-one-line-parser)))
	 (if (not (substring=? line ":" 1))
	     (error "frame-source-parser"
		    "Illegal line (: expected)"
		    (string-until line #\Newline 0))))
      ;; the frame-source-line
      (ignore-one-line-parser)
      ;; the line number
      (let* ((line (ignore-one-line-parser))
	     (num  (string-until line #\Newline 0)))
	 (set! *gdb-stack-source* (cons file num))
	 ;; the frame-source-end line
	 (let ((line (ignore-one-line-parser))
	       (mark #"frame-source-end\n"))
	    (if (not (substring=? line
				  mark
				  (string-length mark)))
		(error "frame-source-parser"
		       "Illegal line (frame-source-end expected)"
		       (string-until line #\Newline 0))))))
   #f)

;*---------------------------------------------------------------------*/
;*    frame-begin-parser ...                                           */
;*    -------------------------------------------------------------    */
;*    we have to fetch the frame number here, it is important          */
;*    because when we will be parsing a function number or an argument */
;*    list that number will tell us the depth in the stack.            */
;*---------------------------------------------------------------------*/
(define (frame-begin-parser line)
   ;; We mark that at least one stack frame has been
   ;; parsed. This variable shows that a source line
   ;; has to be displayed.
   (set! *gdb-stack-frame-parsed?* #t)
   (let* ((snum (string-until-at
		 line
		 (+ 1 (string-length #"frame-begin"))
		 #\space
		 0))
	  (num (string->integer snum))
	  (nli (ignore-one-line-parser))
	  (pref (string-until nli #\Newline 0)))
      ;; read everything until the end of the frame
      ((make-collect-parser #"frame-end\n") nli)
      ;; ignore the following line
      (ignore-one-line-parser)
      ;; now that we are done we can record
      ;; the stack frame
      (let ((frame (list *gdb-stack-function*
			 *gdb-stack-args*
			 num
			 *gdb-stack-source*
			 pref)))
	 (set! *gdb-stack-frames* (cons frame *gdb-stack-frames*))
	 (if *gdb-started?*
	     (begin
		(set! *gdb-started?* #f)
		(set! *gdb-current-stack-frame* frame)))))
   #f)

;*---------------------------------------------------------------------*/
;*    frame-function-name-parser ...                                   */
;*    -------------------------------------------------------------    */
;*    because of module evaluation order, we cannot guarantee here     */
;*    that IGNORE-ONE-LINE-PARSER has already been initialized.        */
;*---------------------------------------------------------------------*/
(define (frame-function-name-parser _)
   (let ((line (ignore-one-line-parser)))
      ;; ignore-one-line-parser does not
      ;; extract the valuable part of the line,
      ;; thus we have to do it here.
      (set! *gdb-stack-function* (string-until line #\Newline 0)))
   #f)

;*---------------------------------------------------------------------*/
;*    source-parser ...                                                */
;*---------------------------------------------------------------------*/
(define (source-parser line)
   (define (bee-client-source-representation)
      (string-append #"" (string-until-at line
					    (string-length #"source ")  
					    #\Newline
					    0)))
   (define (user-source-representation)
      (let* ((start (string-length #"source "))
	     (file  (string-until-at line start #\: 0))
	     (lnum  (string-until-at line (+ start (string-length file) 1)
				     #\: 0))
	     (pos   (string-until-at line (+ start
					     (string-length file)
					     (string-length lnum)
					     2)
				     #\: 0))
	     (beg   (string-until-at line (+ start
					     (string-length file)
					     (string-length lnum)
					     (string-length pos)
					     3)
				     #\: 0))
	     (addr  (string-until-at line (+ start
					     (string-length file)
					     (string-length lnum)
					     (string-length pos)
					     (string-length beg)
					     4)
				     #\: 0))) 
	 (fetch-source-line file lnum)))
   (ignore-one-line-parser)
   (set! *gdb-source-line*
	 (if (or *bee-client?* *emacs-client?*)
	     (bee-client-source-representation)
	     (user-source-representation)))
   #f)

;*---------------------------------------------------------------------*/
;*    stopped-parser ...                                               */
;*---------------------------------------------------------------------*/
(define (stopped-parser _)
   (ignore-one-line-parser)
   ;; we are stopped
   (set! *gdb-stopped?* #t)
   (cond
      ((string? *gdb-stopped-output*)
       'nothing-to-do)
      (*gdb-error?*
       ;; nothging to do because the error is handle by the
       ;; GDB-COMMAND-STATUS function
       'nothing-to-do)
      (else
       (set! *gdb-stopped-output* 
	     (if (string? *gdb-breakpoint*)
		 ;; have we stop because of a breakpoint? If
		 ;; the answer is yes, we store the information
		 ;; used to display that stop...
		 (begin
		    (let* ((frame (gdb-top-stack-frame))
			   (fun   (car frame))
			   (args  (cadr frame))
			   (pos   (cadddr frame)))
		       (if (pair? pos)
			   (list 'breakpoint *gdb-breakpoint* fun args
				 (car pos) (cdr pos))
			   (list 'breakpoint *gdb-breakpoint* fun args
				 "???" "???"))))
		 ;; ...otherwise we store the current frame
		 ;; for a possible display in GDB-COMMAND-STATUS
		 (if *gdb-stack-frame-parsed?*
		     (list 'frame *gdb-current-stack-frame*))))))
   #f)

;*---------------------------------------------------------------------*/
;*    error-begin-parser ...                                           */
;*    -------------------------------------------------------------    */
;*    Warning this markup is emitted on stderr, not on stdout. Thus,   */
;*    we have to implement a special function to display error.        */
;*    Errors have been detected by the means of the ^z^zerror markup   */
;*    on stdout, afterward, we have to read the error.                 */
;*---------------------------------------------------------------------*/
(define (error-begin-parser l)
   (error "error-begin-parser" "Illegal markup" (string-until l #\Newline 0)))

;*---------------------------------------------------------------------*/
;*    gdb-read-error ...                                               */
;*    -------------------------------------------------------------    */
;*    Just read two line on gdb stderr.                                */
;*---------------------------------------------------------------------*/
(define (gdb-read-error)
   (set! *gdb-error?* #f)
   (let* ((proc (get-gdb-process))
	  (perr (process-error-port proc)))
      (read-unbuffered-line perr #t)
      (let ((l (read-unbuffered-line perr #t))
	    (m #"error-begin\n"))
	 (if (not (substring=? m l (string-length m)))
	     ""
	     (let loop ((l (read-unbuffered-line perr #t)))
		(if (not (char=? (string-ref l 0) #\Newline))
		    (string-until l #\Newline 1)
		    (loop (read-unbuffered-line perr #t))))))))

;*---------------------------------------------------------------------*/
;*    error-parser ...                                                 */
;*---------------------------------------------------------------------*/
(define (error-parser _)
   (set! *gdb-error?* #t)
   #f)

;*---------------------------------------------------------------------*/
;*    frames-invalid-parser ...                                        */
;*    -------------------------------------------------------------    */
;*    because of module evaluation order, we cannot guarantee here     */
;*    that IGNORE-ONE-LINE-PARSER has already been initialized.        */
;*---------------------------------------------------------------------*/
(define (frames-invalid-parser _)
   ;; we parse everything until a blank line, we accummulate the
   ;; printed lines as user output
   (ignore-one-line-parser)
   #f)

;*---------------------------------------------------------------------*/
;*    echo-parser ...                                                  */
;*---------------------------------------------------------------------*/
(define (echo-parser line)
   (console-echo (string-until line #\Newline 1)))

;*---------------------------------------------------------------------*/
;*    echo-ignore-one-line-parser ...                                  */
;*---------------------------------------------------------------------*/
(define (echo-ignore-one-line-parser line)
   (echo-parser line)
   (ignore-one-line-parser))
   
;*---------------------------------------------------------------------*/
;*    echo-ignore-one-line-parser-nl ...                               */
;*---------------------------------------------------------------------*/
(define (echo-ignore-one-line-parser-nl line)
   (echo-parser line)
   (console-echo (string-until (ignore-one-line-parser) #\Newline 1)))
   
;*---------------------------------------------------------------------*/
;*    post-prompt-parser ...                                           */
;*---------------------------------------------------------------------*/
(define (post-prompt-parser _)
   (set! *gdb-stack-frames* '())
   (set! *gdb-error?* #f)
   (set! *gdb-stopped?* #f)
   (set! *gdb-started?* #t)
   (set! *gdb-exited?* #f)
   (set! *gdb-stack-frame-parsed?* #f)
;*    (set! *gdb-source-line* #unspecified)                            */
   (set! *gdb-breakpoint* #unspecified)
   #f)

;*---------------------------------------------------------------------*/
;*    starting-parser ...                                              */
;*---------------------------------------------------------------------*/
(define (starting-parser _)
   (set! *gdb-started?* #t)
   (set! *gdb-stopped-output* #f)
   (ignore-one-line-parser)
   #f)

;*---------------------------------------------------------------------*/
;*    read-print-value ...                                             */
;*    -------------------------------------------------------------    */
;*    Read a value (for instance, a printed value). It could be        */
;*    multi line value if the value is a structure.                    */
;*---------------------------------------------------------------------*/
(define (read-print-value stop pp-margin)
   (define (read-print-struct-field line)
      ;; the startup markup
      (check-line "field-begin " line)
      ;; the name
      (let ((name (string-until (ignore-one-line-parser) #\Newline 0)))
	 ;; the field-name-end line
	 (check-line "field-name-end" (ignore-one-line-parser))
	 ;; the = line
	 (check-line " =" (ignore-one-line-parser))
	 ;; the beginning of the value
	 (check-line "field-value" (ignore-one-line-parser))
	 (let ((value (read-print-value "field-end" (+fx pp-margin 3))))
	    (string-append name " = " value))))
   (define (read-print-struct start)
      ;; a structure a sequence field names and values (values can
      ;; be structure too...)
      (let loop ((line (string-until (ignore-one-line-parser) #\Newline 0))
		 (res  start))
	 (cond
	    ((string=? line "}")
	     ;; we are done
	     (let ((res (string-append res line)))
		((make-collect-parser stop) line)
		res))
	    ((string=? line ", ")
	     ;; more to be read
	     (loop (string-until (ignore-one-line-parser) #\Newline 0)
		   (string-append res line)))
	    (else
	     ;; we have to read one struct field
	     (let ((field (read-print-struct-field line)))
		(loop (string-until (ignore-one-line-parser) #\Newline 0)
		      (string-append res field)))))))
   (let* ((start (string-until (ignore-one-line-parser) #\Newline 0))
	  (stop-len (string-length stop))
	  (err "error")
	  (err-len (string-length err)))
      (if (string=? start "{")
	  ;; this is a structure, we have to read field by field
	  (read-print-struct start)
	  (let loop ((line (string-until (ignore-one-line-parser) #\Newline 0))
		     (res  start))
	     (cond
		((or (string=? line "")
		     (char=? (string-ref line 0) #\Newline))
		 (loop (string-until (ignore-one-line-parser) #\Newline 0)
		       res))
		((substring=? line stop stop-len)
		 res)
		((substring=? line err err-len)
		 res)
		(else
		 (loop (string-until (ignore-one-line-parser) #\Newline 0)
		       (string-append res #"\n" line))))))))

;*---------------------------------------------------------------------*/
;*    value-history-begin-parser ...                                   */
;*---------------------------------------------------------------------*/
(define (value-history-begin-parser _)
   (let ((hist-num (string-until (ignore-one-line-parser) #\Newline 0)))
      (ignore-one-line-parser)
      (let ((value (read-print-value #"value-history-end" 0)))
	 (if *gdb-print?*
	     (begin
		;; we only display the output if we are executing a
		;; print command (that is gdb-enable-print! has been
		;; invoked before the execution of the gdb command that
		;; as operated the print.
		(case *gdb-print?*
		   ((bigloo)
		    (console-echo hist-num)
		    (console-echo (gdb-value->bdb-value value))
		    (console-newline))
		   ((explore)
		    (will-explore-value! value))
		   (else
		    (console-echo hist-num)
		    (console-echo value)
		    (console-newline)))))
	 (ignore-one-line-parser)))
   #f))

;*---------------------------------------------------------------------*/
;*    *gdb-breakpoint-table* ...                                       */
;*---------------------------------------------------------------------*/
(define *gdb-breakpoint-table* #f)

;*---------------------------------------------------------------------*/
;*    gdb-breakpoint-table ...                                         */
;*---------------------------------------------------------------------*/
(define (gdb-breakpoint-table)
   *gdb-breakpoint-table*)

;*---------------------------------------------------------------------*/
;*    breakpoint-headers-parser ...                                    */
;*    -------------------------------------------------------------    */
;*    This function is the main function used to parser breakpoint     */
;*    table (e.g. the output of the "info break" command).             */
;*    -------------------------------------------------------------    */
;*    This function does not produce any output. It stores the         */
;*    breakpoint table in a variable that can be used by the command   */
;*    that invoked "info break".                                       */
;*---------------------------------------------------------------------*/
(define (breakpoint-headers-parser _)
   (let* ((table-markup         #"breakpoints-table\n")
	  (table-markup-len     (string-length table-markup))
	  (table-end-markup     #"breakpoints-table-end\n")
	  (table-end-markup-len (string-length table-end-markup))
	  (record-markup        #"record\n")
	  (record-markup-len    (string-length record-markup))
	  (field-markup         #"field")
	  (field-markup-len     (string-length field-markup)))
      (define (read-headers)
	 ;; read all the table headers
	 (let loop ((line     (ignore-one-line-parser))
		    (headers '()))
	    (cond
	       ((substring=? field-markup line field-markup-len)
		(let ((new-header (string-until (ignore-one-line-parser)
						#\Newline
						0)))
		   (loop (ignore-one-line-parser)
			 (cons new-header headers))))
	       ((substring=? table-markup line table-markup-len)
		(ignore-one-line-parser)
		(reverse! headers))
	       ((or (string=? line "") (char=? (string-ref line 0) #\Newline))
		(loop (ignore-one-line-parser)
		      headers))
	       (else
		(error "breakpoint-headers-parser"
		       "Illegal header line"
		       (string-until line #\Newline 0))))))
      (define (read-record field-len)
	 ;; read one breakpoint record
	 (let loop ((i    0)
		    (vals '()))
	    (cond
	       ((=fx i field-len)
		;; we have to read addition informations
		(let loop ((line (ignore-one-line-parser))
			   (vals vals))
		   (cond
		      ((or (substring=? record-markup line
					record-markup-len)
			   (substring=? table-end-markup line
					table-end-markup-len))
		       ;; we are done
		       (values (reverse! vals) line))
		      ((or (= (string-length line) 0)
			   (char=? (string-ref line 0) #\Newline)
			   (substring=? field-markup line
					field-markup-len))
		       (loop (ignore-one-line-parser)
			     vals))
		      (else
		       (let ((value (string-until line #\Newline 0)))
			  (loop (ignore-one-line-parser)
				(cons value vals)))))))
 	       (else
		(ignore-one-line-parser)
		(loop (+fx i 1)
		      (cons (string-until (ignore-one-line-parser) #\Newline 0)
			    vals))))))
      (define (read-records field-len)
	 ;; read all breakpoint records
	 (let loop ((line (ignore-one-line-parser))
		    (res  '()))
	    (cond
	       ((substring=? record-markup line record-markup-len)
		(ignore-one-line-parser)
		(multiple-value-bind (record next-line)
		   (read-record field-len)
		   (loop next-line
			 (cons record res))))
	       ((substring=? table-end-markup line table-end-markup-len)
		(reverse! res))
	       ((char=? (string-ref line 0) #\Newline)
		(loop (ignore-one-line-parser)
		      res))
	       (else
		(error "breakpoint-records-parser"
		       "Illegal records line"
		       (string-until line #\Newline 0))))))
      (let* ((headers (read-headers))
	     (records (read-records (length headers))))
	 (set! *gdb-breakpoint-table* (cons headers records))))
   #f)

;*---------------------------------------------------------------------*/
;*    breakpoint-table-parser ...                                      */
;*    -------------------------------------------------------------    */
;*    If we find this markup it means that the user has requested      */
;*    an table breakpoint inspection before any breakpoint has been    */
;*    set.                                                             */
;*---------------------------------------------------------------------*/
(define (breakpoint-table-parser _)
   (set! *gdb-breakpoint-table* 'no-breakpoint)
   #f)

;*---------------------------------------------------------------------*/
;*    exited-parser ...                                                */
;*---------------------------------------------------------------------*/
(define (exited-parser _)
   (ignore-one-line-parser)
   (set! *gdb-exited?* #t)
   ;; the remaining lines are optional, we simply make some house keeping
   (set! *gdb-stack-frames* '())
   (set! *gdb-current-stack-frame* #f)
   (set! *gdb-source-line* #f)
   (set! *gdb-stopped-output* #f)
   #f)

;*---------------------------------------------------------------------*/
;*    check-line ...                                                   */
;*    -------------------------------------------------------------    */
;*    This function parse one display entry entry.                     */
;*---------------------------------------------------------------------*/
(define (check-line markup line)
   (if (not (substring=? markup line (string-length markup)))
       (error "display-parser"
	      (string-append "Illegal line (" markup " expected)")
	      (string-until line #\Newline 0))))

;*---------------------------------------------------------------------*/
;*    display-begin-parser ...                                         */
;*    -------------------------------------------------------------    */
;*    This function accumulates all the table display. When the        */
;*    end markup is found, the table is then displayed.                */
;*---------------------------------------------------------------------*/
(define (display-begin-parser line)
   (define (bdb-print-expr? expr)
      ;; this is a rough implementation. we consider expr to be
      ;; a bdb expression as soon as it is a call to a static non
      ;; debugged function.
      (let ((pref "(char *) ({<text variable, no debug info>}"))
	 (substring=? pref expr (string-length pref))))
   (define (bdb-print-expr expr)
      ;; we have to fetch the expression inside the last parenthesis
      ;; and to demangle it. this is not 100% correct because we don't
      ;; make a specical case of strings.
      (let* ((len  (string-length expr))
	     (stop (let loop ((i (-fx len 1)))
		      (cond
			 ;; we are sure to find a ")" otherwise bdb-print-expr?
			 ;; would have returned #f
			 ((char=? (string-ref expr i) #\))
			  i)
			 (else
			  (loop (-fx i 1)))))))
	 (let loop ((i (-fx stop 1))
		    (p 1))
	    (cond
	       ((=fx i -1)
		;; strange. this should be an error...
		expr)
	       ((char=? (string-ref expr i) #\()
		(if (=fx p 1)
		    ;; we got it
		    (gdb-expr->bdb-expr (substring expr (+fx i 1) stop))
		    ;; not yet...
		    (loop (-fx i 1)
			  (-fx p 1))))
	       ((char=? (string-ref expr i) #\))
		(loop (-fx i 1)
		      (+fx p 1)))
	       (else
		(loop (-fx i 1)
		      p))))))
   (define (read-value-exp)
      ;; Read a value in a display. That is read all lines until the
      ;; end-mark line and collect what has been read.
      (let* ((end-mark #"display-expression-end")
	     (end-mark-len (string-length end-mark)))
	 (let loop ((exp (string-until (ignore-one-line-parser)
				       #\Newline 0)))
	    ;; we read until the display-expression-end line
	    ;; the display-expression-end line
	    (let ((new-exp (ignore-one-line-parser)))
	       (if (not (substring=? end-mark new-exp end-mark-len))
		   (let* ((str (string-until new-exp #\Newline 0))
			  (len (string-length str)))
		      ;; we have to remove the starting blank chars
		      ;; introduced by Gdb (grrrr...)
		      (let liip ((i 0))
			 (cond
			    ((=fx i len)
			     (loop (string-append exp str)))
			    ((or (char=? (string-ref str i) #\Space)
				 (char=? (string-ref str i) #\Tab))
			     (liip (+fx i 1)))
			    (else
			     (loop (string-append exp
						  (substring str i len)))))))
		   exp)))))
   (define (parse-display)
      (let ((lnum (string-until (ignore-one-line-parser) #\Newline 0)))
	 ;; the display-number-end line
	 (check-line #"display-number-end" (ignore-one-line-parser))
	 ;; the colon line
	 (check-line ":" (ignore-one-line-parser))
	 ;; the display format line
	 (check-line #"display-format" (ignore-one-line-parser))
	 (let ((fmt (string-until (ignore-one-line-parser) #\Newline 0)))
	    ;; the first display-expression line
	    (check-line #"display-expression" (ignore-one-line-parser))
	    (let ((exp (read-value-exp)))
	       ;; the = line
	       (check-line " =" (ignore-one-line-parser))
	       ;; the second display-expression line
	       (check-line #"display-expression"
			   (ignore-one-line-parser))
	       ;; the expression value
	       (let ((value (read-print-value "display-end" 0)))
		  (if (bdb-print-expr? exp)
		      (list lnum
			    fmt
			    (bdb-print-expr exp)
			    (gdb-value->bdb-value value))
		      (list lnum fmt exp value)))))))
   (let* ((end-markup #"stopped\n")
	  (end-markup-len (string-length end-markup))
	  (end-markup2 #"pre-prompt\n")
	  (end-markup2-len (string-length end-markup2)))
      (let loop ((line line)
		 (disp '()))
	 (cond
	    ((or (string=? line "") (char=? (string-ref line 0) #\Newline))
	     (loop (ignore-one-line-parser) disp))
	    ((substring=? end-markup line end-markup-len)
	     (display-display-table! disp)
	     #f)
	    ((substring=? end-markup2 line end-markup2-len)
	     (display-display-table! disp)
	     line)
	    (else
	     (let ((d (parse-display)))
		(loop (ignore-one-line-parser)
		      (cons d disp))))))))

;*---------------------------------------------------------------------*/
;*    arg-begin-parser ...                                             */
;*    -------------------------------------------------------------    */
;*    The name is printed here un demangled. I guess this is want      */
;*    one want because this parse is used on command such as           */
;*    "info frame" which prints rather low level informations (such    */
;*    as register values).					       */
;*---------------------------------------------------------------------*/
(define (arg-begin-parser _)
   (let* ((arg (read-one-arg))
	  (_   (ignore-one-line-parser)))
      (console-echo (car arg))
      (console-echo " = " )
      (console-echo (cdr arg))
      (console-newline))
   #f)

;*---------------------------------------------------------------------*/
;*    signal-parser ...                                                */
;*---------------------------------------------------------------------*/
(define (signal-parser _)
   ;; we mark that a signal has been raised
   (set! *gdb-signal?* #t)
   ;; this function parse one display entry entry
   (define (check-line markup line)
      (if (not (substring=? markup line (string-length markup)))
	  (error "display-parser"
		 (string-append "Illegal line (" markup " expected)")
		 (string-until line #\Newline 0))))
   ;; the dummy line
   (ignore-one-line-parser)
   ;; the Program received signal line
   (ignore-one-line-parser)
   ;; the name
   (check-line #"signal-name" (ignore-one-line-parser))
   (let ((sig (string-until (ignore-one-line-parser) #\Newline 0)))
      (check-line #"signal-name-end" (ignore-one-line-parser))
      ;; the command line
      (check-line "," (ignore-one-line-parser))
      ;; the signal string 
      (check-line #"signal-string" (ignore-one-line-parser))
      (let ((msg (string-until (ignore-one-line-parser) #\Newline 0)))
 	 (console-error "Signal received: ")
	 (console-echo sig)
	 (console-echo " ")
	 (console-echo msg)
	 (console-newline)
	 (check-line #"signal-string-end" (ignore-one-line-parser))
	 (ignore-one-line-parser)))
   #f)
   
;*---------------------------------------------------------------------*/
;*    All the annotate parsers.                                        */
;*---------------------------------------------------------------------*/
(add-annotate-parser! #"starting\n" starting-parser)
(add-annotate-parser! #"post-prompt\n" post-prompt-parser)
(add-annotate-parser! #"stopped\n" stopped-parser)
(add-annotate-parser! #"pre-prompt\n" (make-ignore-parser #"prompt\n"))
(add-annotate-parser! #"frames-invalid\n" frames-invalid-parser)
(add-annotate-parser! #"breakpoints-invalid\n" breakpoints-invalid-parser)
(add-annotate-parser! #"frame-function-name\n" frame-function-name-parser)
(add-annotate-parser! #"error-begin\n" error-begin-parser)
(add-annotate-parser! #"error\n" error-parser)
(add-annotate-parser! #"frame-source-begin\n" frame-source-begin-parser)
(add-annotate-parser! #"frame-args\n" frame-args-parser)
(add-annotate-parser! #"frame-begin" frame-begin-parser)
(add-annotate-parser! #"breakpoint " breakpoint-parser)
(add-annotate-parser! #"source " source-parser)
(add-annotate-parser! #"frame-end\n" (lambda (_)
					  (ignore-one-line-parser)
					  #f))
(add-annotate-parser! #"value-history-begin" value-history-begin-parser)
(add-annotate-parser! #"breakpoints-headers\n" breakpoint-headers-parser)
(add-annotate-parser! #"breakpoints-table-end\n" breakpoint-table-parser)
(add-annotate-parser! #"exited" exited-parser)
(add-annotate-parser! #"display-begin" display-begin-parser)
(add-annotate-parser! #"arg-begin" arg-begin-parser)
(add-annotate-parser! #"signal" signal-parser)
(add-annotate-parser! #"quit" (lambda (_)
				   (ignore-one-line-parser)
				   #f))
;; non annotate markup commands
(add-annotate-parser! #"Starting program: " echo-ignore-one-line-parser)
(add-annotate-parser! #"Breakpoint " echo-ignore-one-line-parser)
