;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Gdb/ginvoke.scm                */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 13:11:10 1999                          */
;*    Last change :  Tue Aug 31 11:53:52 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Invoking GDB from BDB.                                           */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module gdb_invoke
   (import gdb_proc
	   gdb_annotate
	   tools_speek
	   tools_tools
	   tools_io
	   engine_param
	   (bdb-repl-prompt engine_repl))
   (export (gdb-call-for-output . args)
	   (gdb-call-ignore-output ::bstring)
	   (gdb-call ::bstring ::procedure)
	   (flush-gdb-pending-output ::bool)
	   (input-port->file::file iport::input-port)
	   ignore-one-line-parser
	   (make-ignore-parser ::bstring)
	   (make-collect-parser ::bstring)
	   (read-unbuffered-line ::input-port ::bool)))

;*---------------------------------------------------------------------*/
;*    gdb-call-for-output ...                                          */
;*    -------------------------------------------------------------    */
;*    Invokes a GDB command and prints out the result of the           */
;*    evaluation of that command                                       */
;*---------------------------------------------------------------------*/
(define (gdb-call-for-output . args)
   ;; first, we construct the command line
   (let ((cmd (match-case args
		 (()
		  (error "gdb-call-for-ouput" "Illegal empty command" args))
		 ((?str)
		  str)
		 (else
		  (let loop ((cmd (reverse args))
			     (res ""))
		     (if (null? cmd)
			 res
			 (loop (cdr cmd)
			       (string-append (car cmd) " " res))))))))
      (gdb-call cmd (lambda (pin) (read-until-next-prompt)))))
   
;*---------------------------------------------------------------------*/
;*    gdb-call-ignore-output ...                                       */
;*---------------------------------------------------------------------*/
(define (gdb-call-ignore-output cmd)
   (let ((old-console-echo (get-console-echo-function)))
      (set-console-echo-function! (lambda (x) x))
      ;; emit the command
      (gdb-call cmd (lambda (pin)
		       (flush-gdb-pending-output #f)))
      ;; flush the command output silently
      (gdb-command-status #f)
      ;; we restore the regular echo function
      (set-console-echo-function! old-console-echo)
      (not (gdb-error?))))

;*---------------------------------------------------------------------*/
;*    *gdb-call-armed* ...                                             */
;*---------------------------------------------------------------------*/
(define *gdb-call-armed* #f)

;*---------------------------------------------------------------------*/
;*    gdb-call ...                                                     */
;*---------------------------------------------------------------------*/
(define (gdb-call cmd::bstring filter::procedure)
   (if (and #f *gdb-call-armed*)
       (error "internal-error:gdb-call:Illegal re-entrant gdb-call"
	      *gdb-call-armed*
	      cmd)
       (begin
	  (set! *gdb-call-armed* cmd)
	  (verbose 4 "sending: [" cmd "]" #\Newline)
	  (let* ((proc (get-gdb-process))
		 (pin  (process-input-port proc))
		 (pout (process-output-port proc)))
	     (fprint pin cmd)
	     (flush-output-port pin)
	     (let ((res (filter pout)))
		(set! *gdb-call-armed* #f)
		res)))))

;*---------------------------------------------------------------------*/
;*    *buffer* ...                                                     */
;*---------------------------------------------------------------------*/
(define *buffer* (make-string 255 #\-))

;*---------------------------------------------------------------------*/
;*    enlarge-buffer! ...                                              */
;*---------------------------------------------------------------------*/
(define (enlarge-buffer!)
   (let* ((old-len (string-length *buffer*))
	  (new-len (*fx old-len 2))
	  (new-buf (make-string new-len)))
      (blit-string! *buffer* 0 new-buf 0 old-len)
      (set! *buffer* new-buf)))

;*---------------------------------------------------------------------*/
;*    input-port->file ...                                             */
;*---------------------------------------------------------------------*/
(define (input-port->file::file iport::input-port)
   (pragma::file "INPUT_PORT( $1 ).file" iport))

;*---------------------------------------------------------------------*/
;*    getc ...                                                         */
;*---------------------------------------------------------------------*/
(define (getc::char file::file block?)
   (pragma "extern int (*bdb_getc)(FILE *)")
   (if block?
       (pragma::char "getc( $1 )" file)
       (pragma::char "bdb_getc( $1 )" file)))

;*---------------------------------------------------------------------*/
;*    read-unbuffered-line ...                                         */
;*    -------------------------------------------------------------    */
;*    Read a line unbuffered.                                          */
;*---------------------------------------------------------------------*/
(define (read-unbuffered-line iport::input-port block?)
   (let ((file::file (input-port->file iport)))
      (let loop ((i   0)
		 (len (string-length *buffer*))
		 (buf *buffer*))
	 (if (=fx i len)
	     (begin
		(enlarge-buffer!)
		(loop i (string-length *buffer*) *buffer*))
	     (let ((c (getc file block?)))
		(if (char=? c #\Newline)
		    (begin
		       (string-set! buf i c)
		       buf)
		    (begin
		       (string-set! buf i c)
		       (loop (+fx i 1)
			     len buf))))))))
	    
;*---------------------------------------------------------------------*/
;*    flush-gdb-pending-output ...                                     */
;*    -------------------------------------------------------------    */
;*    We read everything until we find the gdb prompt.                 */
;*    -------------------------------------------------------------    */
;*    That is we read line by line the gdb output until we find the    */
;*    last three lines that gdb prints for its prompt.                 */
;*---------------------------------------------------------------------*/
(define (flush-gdb-pending-output verb?)
   (flush-until *gdb-prompt* (process-output-port (get-gdb-process)))
   (if (gdb-error?)
       (if verb?
	   (let ((err (gdb-read-error)))
	      ;; reads the error and displays it
	      (console-error err)
	      ;; we redisplay the console prompt
	      (console-prompt (bdb-repl-prompt)))
	   (gdb-read-error))))

;*---------------------------------------------------------------------*/
;*    flush-gdb-starting-output ...                                    */
;*---------------------------------------------------------------------*/
(define (flush-gdb-starting-output pin)
   (flush-until-command pin))

;*---------------------------------------------------------------------*/
;*    flush-until ...                                                  */
;*---------------------------------------------------------------------*/
(define (flush-until stop-lines::pair pin)
   (let loop ((line   (read-unbuffered-line pin #f))
	      (prompt stop-lines))
	 (verbose 5 "flush ["
		  (string-for-read (string-until line #\Newline 1))
		  #"]\n")
	 (verbose 5 "flush ["
		  (find-annotate-parser (string-until line #\Newline 1))
		  #"]\n")
      (if (substring=? (car prompt) line (string-length (car prompt)))
	  ;; we have a match, we skip to the next line until we
	  ;; are with all the prompt
	  (let ((new-prompt (cdr prompt)))
	     (if (null? new-prompt)
		 #t
		 (loop (read-unbuffered-line pin #f) new-prompt)))
	  (let* ((line   (string-until line #\Newline 1))
		 (parser (find-annotate-parser line)))
	     (if (procedure? parser)
		 ;; We have to parser with the parser
		 ;; then, we will keep going with the
		 ;; current parser. The result of this
		 ;; parser is ignored because the local
		 ;; parser is supposed to make a side
		 ;; effect with its input.
		 (parser line))
	     ;; no match, we restart from the beginning
	     (loop (read-unbuffered-line pin #f) stop-lines)))))

;*---------------------------------------------------------------------*/
;*    flush-until-command ...                                          */
;*    -------------------------------------------------------------    */
;*    Flush all io until the first command.                            */
;*---------------------------------------------------------------------*/
(define (flush-until-command pin)
   (let loop ((line (read-unbuffered-line pin #f)))
      (verbose 5 "flush-cmd ["
	       (string-for-read (string-until line #\Newline 1))
	       #"]\n")
      (verbose 5 "flush-cmd ["
	       (find-annotate-parser (string-until line #\Newline 1))
	       #"]\n")
      (let* ((line   (string-until line #\Newline 1))
	     (parser (find-annotate-parser line)))
	 (if (procedure? parser)
	     ;; We have to parser with the parser
	     ;; then, we will keep going with the
	     ;; current parser. The result of this
	     ;; parser is ignored because the local
	     ;; parser is supposed to make a side
	     ;; effect with its input.
	     (parser line)
	     ;; no match, we restart from the beginning
	     (loop (read-unbuffered-line pin #f))))))

;*---------------------------------------------------------------------*/
;*    read-until-next-prompt ...                                       */
;*    -------------------------------------------------------------    */
;*    This function start collecting when it first sees an non         */
;*    blank line.                                                      */
;*    -------------------------------------------------------------    */
;*    During this collect, we always skip blank lines because they     */
;*    can't be part of any user output.                                */
;*---------------------------------------------------------------------*/
(define (read-until-next-prompt)
   ;; the main loop
   (let* ((proc       (get-gdb-process))
	  (pin        (process-output-port proc))
	  (stop-lines *gdb-prompt*))
      ;; flush until the starting prompt
      (flush-gdb-starting-output pin)
      ;; we read line by line until the next prompt
      (let loop ((line     (read-unbuffered-line pin #f))
		 (prompt   stop-lines)
		 (miss-buf '())
		 (res      '()))
	 (verbose 5 "until ["
		  (string-for-read (string-until line #\Newline 1))
		  #"]\n")
	 (verbose 5 "until ["
		  (find-annotate-parser (string-until line #\Newline 1))
		  #"]\n")
	 (cond
	    ((and (>fx (string-length line) 0)
		  (substring=? (car prompt) line (string-length (car prompt))))
	     ;; we have a match, we skip to the next line until we
	     ;; are with all the prompt
	     (let ((new-prompt (cdr prompt)))
		(if (null? new-prompt)
		    (apply string-append (reverse! res))
		    ;; take care to the evaluation order here!
		    (let ((new-miss-buff (cons (string-until line #\Newline 1)
					       miss-buf)))
		       (loop (read-unbuffered-line pin #f)
			     new-prompt
			     new-miss-buff
			     res)))))
	    (else
	     (let* ((line   (string-until line #\Newline 1))
		    (parser (find-annotate-parser line)))
		(if (procedure? parser)
		    ;; We have to parser with the parser
		    ;; then, we will keep going with the
		    ;; current parser. The result of this
		    ;; parser is ignored because the local
		    ;; parser is supposed to make a side
		    ;; effect with its input.
		    (let ((line (parser line)))
		       (loop (if (string? line)
				 line
				 (read-unbuffered-line pin #f))
			     stop-lines
			     '()
			     (if (pair? miss-buf)
				 (append (reverse! miss-buf) res)
				 res)))
		    ;; no match, we restart from the beginning
		    (let ((collect? (> (string-length line) 1)))
		       (loop (read-unbuffered-line pin #f)
			     stop-lines
			     '()
			     (if collect?
				 (cons line
				       (if (pair? miss-buf)
					   (append (reverse! miss-buf) res)
					   res))
				 res))))))))))

;*---------------------------------------------------------------------*/
;*    ignore-one-line-parser ...                                       */
;*    -------------------------------------------------------------    */
;*    This function does not extract the relevant part from the        */
;*    string. If the result of this function is useful the caller,     */
;*    must strip off the end of the string.                            */
;*---------------------------------------------------------------------*/
(define (ignore-one-line-parser)
   (let* ((proc (get-gdb-process))
	  (pin  (process-output-port proc))
	  (line (read-unbuffered-line pin #f)))
      (verbose 5 "ignore-one ["
	       (string-for-read (string-until line #\Newline 1))
	       #"]\n")
      (verbose 5 "ignore-one ["
	       (find-annotate-parser (string-until line #\Newline 1))
	       #"]\n")
      line))
     
;*---------------------------------------------------------------------*/
;*    make-ignore-parser ...                                           */
;*    -------------------------------------------------------------    */
;*    Ignore every line until the markup.                              */
;*---------------------------------------------------------------------*/
(define (make-ignore-parser markup)
   (lambda (_)
      (let* ((proc (get-gdb-process))
	     (pin  (process-output-port proc))
	     (len  (string-length markup)))
	 (let loop ((line (read-unbuffered-line pin #f)))
	    (verbose 5 "ignore ["
		     (string-for-read (string-until line #\Newline 1))
		     #"]\n")
	    (verbose 5 "ignore ["
		     (find-annotate-parser (string-until line #\Newline 1))
		     #"]\n")
	    (if (or (=fx (string-length line) 0)
		    (not (substring=? markup line len)))
		(loop (read-unbuffered-line pin #f)))))))

;*---------------------------------------------------------------------*/
;*    make-collect-parser ...                                          */
;*    -------------------------------------------------------------    */
;*    Collect all the line until the markup.                           */
;*---------------------------------------------------------------------*/
(define (make-collect-parser markup)
   (lambda (_)
      ;; the main loop
      (let* ((proc (get-gdb-process))
	     (pin  (process-output-port proc))
	     (len  (string-length markup)))
	 (let loop ((line (read-unbuffered-line pin #f))
		    (res  '()))
	    (verbose 5 "collect ["
		     (string-for-read (string-until line #\Newline 1))
		     #"]\n")
	    (verbose 5 "collect ["
		     (find-annotate-parser (string-until line #\Newline 1))
		     #"]\n")
	    (cond
	       ((and (>fx (string-length line) 0)
		     (substring=? markup line len))
		(apply string-append (reverse! res)))
	       (else
		(let* ((line   (string-until line #\Newline 1))
		       (parser (find-annotate-parser line)))
		   (if (procedure? parser)
		       ;; We have to parser with the parser
		       ;; then, we will keep going with the
		       ;; current parser. The result of this
		       ;; parser is ignored because the local
		       ;; parser is supposed to make a side
		       ;; effect with its input.
		       (begin
			  (parser line)
			  (if (gdb-error?)
			      ;; during a collect, if an error is raised
			      ;; we have to stop every thing
			      (apply string-append (reverse! res))
			      (loop (read-unbuffered-line pin #f)
				    res)))
		       ;; no match, we restart from the beginning
		       (loop (read-unbuffered-line pin #f)
			     (cons line res))))))))))
   
