;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Gdb/tools.scm                  */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Jul 28 15:41:13 1999                          */
;*    Last change :  Tue Aug 31 09:42:36 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    General GDB parsing tools. This module implements small GDB      */
;*    facilities to gather small informations about expressions.       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module gdb_tools
   (import tools_tools)
   (export (gdb-typeof::bstring ::bstring)
	   (gdb-current-scope))
   (import gdb_invoke))

;*---------------------------------------------------------------------*/
;*    gdb-typeof ...                                                   */
;*    -------------------------------------------------------------    */
;*    Ask GDB for the type of a variable. Returns it as a string.      */
;*---------------------------------------------------------------------*/
(define (gdb-typeof::bstring var::bstring)
   (let* ((cmd (string-append "whatis " var))
	  (res (gdb-call-for-output cmd)))
      ;; whatis format is <TYPE> = <TYPE VALUE>. to fetch the type
      ;; value we skip everything until the = characters
      (string-until (string-from res #\= 2) #\Newline 0)))

;*---------------------------------------------------------------------*/
;*    gdb-current-scope ...                                            */
;*    -------------------------------------------------------------    */
;*    Returns the current scope (this function does do any parsing,    */
;*    it returns a plain string as produced by gdb).                   */
;*---------------------------------------------------------------------*/
(define (gdb-current-scope)
   ;; we have to compute the current line number
   (let ((line (gdb-call-for-output "info line")))
      (if (substring=? "Line " line 5)
	  (let* ((lnum (string-until-at line 5 #\space 0))
		 (cmd  (string-append "info scope " lnum)))
	     (if (= (string->integer lnum) 0)
		 ;; there is no line for that file
		 #f
		 ;; then we can request an "info scope"
		 (gdb-call-for-output cmd)))
	  #f)))
   
