;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Init/parse-args.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Thu Jun 29 11:56:20 1995                          */
;*    Last change :  Tue Aug 31 09:15:32 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Command-line parsing                                             */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module init_parse-args
   (include "Init/args.sch")
   (export  (parse-args args::pair))
   (import  engine_param
	    tools_version))

;*---------------------------------------------------------------------*/
;*    parse-args ...                                                   */
;*---------------------------------------------------------------------*/
(define (parse-args cmd-args)
   (bind-exit (exit)
      (args-parse (cdr cmd-args)
	 ;; help
	 (section "Help")
	 (("--help" (synopsis "-help|--help" "Print this message."))
	  (help args-parse-usage)
	  (exit 0))
	 (("-help")
	  (help args-parse-usage)
	  (exit 0))
	 (("--bare-help")
	  (bare-help args-parse-usage)
	  (exit #f))
	 (("?")
	  (help args-parse-usage))
	 ;; gdb compatibility
	 (section "Gdb compatibility")
	 (("--quiet" (synopsis "Do not print version number on startup."))
	  (set! *verbose* -1))
	 ;; language
	 (section "Language")
	 (("-bigloo" (synopsis "Enables Bigloo mode (default)"))
	  (set! *bigloo-enabled?* #t)
	  (set! *heap-explorer?* #t))
	 (("-c" (synopsis "Disables Bigloo mode"))
	  (set! *bigloo-enabled?* #f)
	  (set! *heap-explorer?* #f))
	 ;; bdb debugging configuration
	 (section "Bdb debug configuration")
	 (("-v" (synopsis "-v[23]" "Set verbose mode on."))
	  (set! *verbose* 1))
	 (("-v2")
	  (set! *verbose* 2))
	 (("-v3")
	  (set! *verbose* 3))
	 (("-t" (synopsis "-t[2]" "Enables tracing mode"))
	  (set! *verbose* 4))
	 (("-t2")
	  (set! *verbose* 5))
	 (("-active-prompt" (synopsis "Enables active prompt (default)"))
	  (set! *active-prompt* #t))
	 (("-no-active-prompt" (synopsis "Disables active prompt"))
	  (set! *active-prompt* #f))
	 ;; bdb environment configuration
	 (section "Bdb environment")
	 (("--bee" (synopsis "--bee" "Runs Bdb as a Bee client"))
	  (set! *bee-client?* #t))
	 (("--emacs" (synopsis "--emacs|--fullname" "Runs Bdb as an Emacs client"))
	  (set! *emacs-client?* #t))
	 (("--fullname")
	  (set! *emacs-client?* #t))
	 (("--root-dir" ?dir (synopsis "Sets the Bee root directory"))
	  (set! *root-directory* dir))
	 (("--afile" ?afile (synopsis "Sets the Bee afile file name"))
	  (set! *afile* afile))
	 (("--etags" ?etags (synopsis "Sets the Bee btags file name"))
	  (set! *etags* etags))
	 (("--no-demangling" (synopsis "Disable demangling (i.e., C mode)"))
	  (set! *bigloo-enabled?* #f))
	 (("--no-heap-explorer" (synopsis "Disable heap explorer (i.e., C mode)"))
	  (set! *heap-explorer?* #f))
	 ;; executable
	 (else
	  (if (string? *exec*)
	      (set! *args* (cons else *args*))
	      (set! *exec* else))))))
	
;*---------------------------------------------------------------------*/
;*    query ...                                                        */
;*---------------------------------------------------------------------*/
(define (query)
   (version)
   (newline)
   (exit 0))

;*---------------------------------------------------------------------*/
;*    help ...                                                         */
;*---------------------------------------------------------------------*/
(define (help usage)
   (version)
   (print "usage: bdb [options] executable-file")
   (newline)
   (bare-help usage))

;*---------------------------------------------------------------------*/
;*    bare-help ...                                                    */
;*---------------------------------------------------------------------*/
(define (bare-help usage)
   (usage)
   (newline)
   (print "Runtime Command file:")
   (print "   - ~/.bdbrc"))


   
