;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/src/Patient/mangling.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 21:08:59 1999                          */
;*    Last change :  Tue Aug 31 09:24:19 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The name Bdb mangling/demangling.                                */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module patient_mangling
   (import tools_tools
	   engine_param
	   patient_invoke
	   bee_invoke
	   command_expr
	   gdb_annotate)
   (export (bdb-demangle ::bstring)
	   (bdb-demangle2 ::bstring ::bstring)
	   (bdb-mangle ::bstring)
	   (bdb-mangle2 ::bstring ::bstring)
	   (bdb-mangle-for-funcall ::bstring)
	   (reset-demangling-cache!)
	   (bigloo-function? ::bstring)))

;*---------------------------------------------------------------------*/
;*    global-info                                                      */
;*---------------------------------------------------------------------*/
(define-struct global-info
   ;; the Bigloo identifier for the variable
   scm-name
   ;; the c identifier for the variable
   c-name
   ;; the list of locals contained in the global
   locals)

;*---------------------------------------------------------------------*/
;*    local-info                                                       */
;*---------------------------------------------------------------------*/
(define-struct local-info
   ;; the Bigloo identifier for the variable
   scm-name
   ;; the c identifier for the variable
   c-name)

;*---------------------------------------------------------------------*/
;*    bdb-demangle ...                                                 */
;*    -------------------------------------------------------------    */
;*    To demangle a name, we check if we have already demangle         */
;*    that name and thus if the name is in the local cache.            */
;*    Otherwise we have to request the patient for a demangling.       */
;*---------------------------------------------------------------------*/
(define (bdb-demangle c-ident::bstring)
   (if *bigloo-enabled?*
       (let ((ld (local-demangle c-ident)))
	  (if (global-info? ld)
	      (global-info-scm-name ld)
	      (let ((rd (remote-demangle c-ident)))
		 (if (string? rd)
		     (let ((glo (global-info rd c-ident '())))
			(put-local-demangle! glo)
			rd)
		     ;; if we have found no name demangling for that
		     ;; function and if the execution is started it
		     ;; means that it is a real C function.
		     (begin
			(if (gdb-current-function)
			    (mark-unmangled! c-ident))
			#f)))))
       #f))

;*---------------------------------------------------------------------*/
;*    bdb-demangle2 ...                                                */
;*    -------------------------------------------------------------    */
;*    To demangle a name, we check if we have already demangle         */
;*    that name and thus if the name is in the local cache.            */
;*    Otherwise we have to request the patient for a demangling.       */
;*---------------------------------------------------------------------*/
(define (bdb-demangle2 c-ident::bstring loc::bstring)
   (define (demangle2/glo glo)
      (let ((ld (local-demangle-local glo loc)))
	 (if (local-info? ld)
	     (local-info-scm-name ld)
	     (let ((rd (remote-demangle2 c-ident loc)))
		(if (string? rd)
		    (let ((loc (local-info rd loc)))
		       (put-local-demangle-local! glo loc)
		       rd)
		    #f)))))
   (if *bigloo-enabled?*
       (let ((glo (local-demangle c-ident)))
	  (if (global-info? glo)
	      (demangle2/glo glo)
	      (let ((glo (remote-demangle c-ident)))
		 (if (global-info? glo)
		     (demangle2/glo glo)
		     #f))))
       #f))

;*---------------------------------------------------------------------*/
;*    bdb-mangle ...                                                   */
;*    -------------------------------------------------------------    */
;*    See DEMANGLE                                                     */
;*---------------------------------------------------------------------*/
(define (bdb-mangle bgl-ident::bstring)
   (if *bigloo-enabled?*
       (let ((ld (local-mangle bgl-ident)))
	  (if (global-info? ld)
	      (global-info-c-name ld)
	      (let ((rd (remote-mangle bgl-ident)))
		 (if (string? rd)
		     (let ((glo (global-info bgl-ident rd '())))
			(put-local-demangle! glo)
			rd)
		     #f))))
       #f))

;*---------------------------------------------------------------------*/
;*    bdb-mangle2 ...                                                  */
;*    -------------------------------------------------------------    */
;*    We don't maintain a cache here. This function is only called     */
;*    when evaluating expression.                                      */
;*    -------------------------------------------------------------    */
;*    The first argument is a C identifier. It is not demangled! The   */
;*    reason is that this function is called if (GDB-CURRENT-FUNCTION) */
;*    as C-IDENT argument.                                             */
;*---------------------------------------------------------------------*/
(define (bdb-mangle2 c-ident::bstring bgl-ident::bstring)
   (if *bigloo-enabled?*
       (remote-mangle2 c-ident bgl-ident)
       #f))

;*---------------------------------------------------------------------*/
;*    bdb-mangle-for-funcall ...                                       */
;*    -------------------------------------------------------------    */
;*    We don't maintain a cache here. This function is only called     */
;*    when evaluating expression.                                      */
;*---------------------------------------------------------------------*/
(define (bdb-mangle-for-funcall bgl-ident::bstring)
   (if *bigloo-enabled?*
       (remote-mangle-for-funcall bgl-ident)
       #f))

;*---------------------------------------------------------------------*/
;*    *mangling-cache* ...                                             */
;*---------------------------------------------------------------------*/
(define *demangling-cache* #unspecified)
(define *mangling-cache* #unspecified)
(define *unmangled-cache* #unspecified)

;*---------------------------------------------------------------------*/
;*    reset-demangling-cache! ...                                      */
;*---------------------------------------------------------------------*/
(define (reset-demangling-cache!)
   (if *bigloo-enabled?*
       (begin
	  (set! *demangling-cache*
		(make-hash-table 1024
				 (lambda (o) (string->0..2^x-1 o 10))
				 global-info-c-name
				 string=?
				 64))
	  (set! *mangling-cache*
		(make-hash-table 1024
				 (lambda (o) (string->0..2^x-1 o 10))
				 global-info-scm-name
				 string=?
				 64))
	  (set! *unmangled-cache*
		(make-hash-table 1024
				 (lambda (o) (string->0..2^x-1 o 10))
				 (lambda (x) x)
				 string=?
				 64)))))

;*---------------------------------------------------------------------*/
;*    local-demangle ...                                               */
;*---------------------------------------------------------------------*/
(define (local-demangle c::bstring)
   (and (hash-table? *demangling-cache*)
	(get-hash c *demangling-cache*)))

;*---------------------------------------------------------------------*/
;*    local-mangle ...                                                 */
;*---------------------------------------------------------------------*/
(define (local-mangle bgl::bstring)
   (and (hash-table? *mangling-cache*)
	(get-hash bgl *mangling-cache*)))

;*---------------------------------------------------------------------*/
;*    put-local-demangle! ...                                          */
;*---------------------------------------------------------------------*/
(define (put-local-demangle! glo::struct)
   (if (or (not (hash-table? *demangling-cache*))
	   (not (hash-table? *mangling-cache*)))
       (reset-demangling-cache!))
   (put-hash! glo *demangling-cache*)
   (put-hash! glo *mangling-cache*))

;*---------------------------------------------------------------------*/
;*    remote-demangle ...                                              */
;*    -------------------------------------------------------------    */
;*    The mangling for that identifier is not in the local cache       */
;*    yet, we have to request the patient.                             */
;*---------------------------------------------------------------------*/
(define (remote-demangle c::bstring)
   ;; in bee client mode, we do request the patient, the ask the bee
   (if (and (gdb-current-function)
	    (not (mark-unmangled? c)))
       (patient-call "bdb_demangle" (string-append "\"" c "\""))
       #f))
   
;*---------------------------------------------------------------------*/
;*    remote-demangle2 ...                                             */
;*    -------------------------------------------------------------    */
;*    The mangling for that identifier is not in the local cache       */
;*    yet, we have to request the patient.                             */
;*---------------------------------------------------------------------*/
(define (remote-demangle2 c loc)
   ;; in bee client mode, we do request the patient, the ask the bee
   (if (gdb-current-function)
       (patient-call "bdb_demangle2"
		     (string-append "\"" c "\"")
		     (string-append "\"" loc "\""))
       #f))
   
;*---------------------------------------------------------------------*/
;*    remote-mangle ...                                                */
;*    -------------------------------------------------------------    */
;*    The mangling for that identifier is not in the local cache       */
;*    yet, we have to request the patient.                             */
;*---------------------------------------------------------------------*/
(define (remote-mangle bgl::bstring)
   ;; we ask the patient directly
   (if (gdb-current-function)
       (patient-call "bdb_mangle" (string-append "\"" bgl "\""))
       #f))
   
;*---------------------------------------------------------------------*/
;*    remote-mangle2 ...                                               */
;*    -------------------------------------------------------------    */
;*    The mangling for that identifier is not in the local cache       */
;*    yet, we have to request the patient.                             */
;*---------------------------------------------------------------------*/
(define (remote-mangle2 c::bstring bgl::bstring)
   (if (gdb-current-function)
       (patient-call "bdb_mangle2"
 		     (string-append "\"" c "\"")
		     (string-append "\"" bgl "\""))
       #f))
   
;*---------------------------------------------------------------------*/
;*    remote-mangle-for-funcall ...                                    */
;*    -------------------------------------------------------------    */
;*    The mangling for that identifier is not in the local cache       */
;*    yet, we have to request the patient.                             */
;*---------------------------------------------------------------------*/
(define (remote-mangle-for-funcall bgl::bstring)
   (if (gdb-current-function)
       (patient-call "bdb_mangle_for_funcall" (string-append "\"" bgl "\""))
       #f))
   
;*---------------------------------------------------------------------*/
;*    bigloo-function? ...                                             */
;*    -------------------------------------------------------------    */
;*    A function is a Bigloo function iff we are able to demangle it.  */
;*---------------------------------------------------------------------*/
(define (bigloo-function? name::bstring)
   (string? (bdb-demangle name)))

;*---------------------------------------------------------------------*/
;*    local-demangle-local ...                                         */
;*---------------------------------------------------------------------*/
(define (local-demangle-local glo::struct c-ident::bstring)
   (let loop ((locals (global-info-locals glo)))
      (cond
	 ((null? locals)
	  #f)
	 ((string=? (local-info-c-name (car locals)) c-ident)
	  (car locals))
	 (else
	  (loop (cdr locals))))))

;*---------------------------------------------------------------------*/
;*    put-local-demangle-local! ...                                    */
;*---------------------------------------------------------------------*/
(define (put-local-demangle-local! glo::struct loc::struct)
   (global-info-locals-set! glo (cons loc (global-info-locals glo))))

;*---------------------------------------------------------------------*/
;*    local-mangle-local ...                                           */
;*---------------------------------------------------------------------*/
(define (local-mangle-local glo::struct bgl-ident::bstring)
   (let loop ((locals (global-info-locals glo)))
      (cond
	 ((null? locals)
	  #f)
	 ((string=? (local-info-scm-name (car locals)) bgl-ident)
	  (car locals))
	 (else
	  (loop (cdr locals))))))

;*---------------------------------------------------------------------*/
;*    put-local-mangle-local! ...                                      */
;*---------------------------------------------------------------------*/
(define (put-local-mangle-local! glo::struct loc::struct)
   (global-info-locals-set! glo (cons loc (global-info-locals glo))))

;*---------------------------------------------------------------------*/
;*    mark-unmangled! ...                                              */
;*---------------------------------------------------------------------*/
(define (mark-unmangled! c::bstring)
   (if (not (hash-table? *unmangled-cache*))
       (reset-demangling-cache!))
   (put-hash! c *unmangled-cache*))

;*---------------------------------------------------------------------*/
;*    mark-unmangled? ...                                              */
;*---------------------------------------------------------------------*/
(define (mark-unmangled? c::bstring)
   (and (hash-table? *unmangled-cache*)
	(get-hash c *unmangled-cache*)))
