;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bdk/kbdb/kbdb/Patient/pinvoke.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 17:02:51 1999                          */
;*    Last change :  Thu Aug  5 19:29:59 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Invoking the patient.                                            */
;*    -------------------------------------------------------------    */
;*    To invoke the patient, we go through gdb, we construct a         */
;*    call command. When calling a patient function, we send a         */
;*    port to establish a connection by the means of socket on         */
;*    that port.                                                       */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module patient_invoke
   (import engine_param
	   patient_server
	   gdb_invoke
	   gdb_annotate)
   (export (patient-call fun::bstring . args)))

;*---------------------------------------------------------------------*/
;*    patient-call ...                                                 */
;*    -------------------------------------------------------------    */
;*    Call a service from the patient a read the result via a socket.  */
;*---------------------------------------------------------------------*/
(define (patient-call fun . args)
   (if (not (gdb-current-function))
       ;; when we are not running the function, we cannot request for
       ;; the patient help.
       #f
       ;; first, we construct the command line
       (let* ((pnum (integer->string (get-server-number)))
	      (cmd  (match-case args
		       (()
			(string-append "call " fun "(" pnum ")"))
		       ((?str)
			(string-append "call " fun "(" pnum "," str ")"))
		       ((?str ?arg)
			(string-append "call " fun "(" pnum "," str "," arg ")"))
		       (else
			(let loop ((cmd (reverse args))
				   (res ")"))
			   (if (null? (cdr cmd))
			       (string-append "call " fun "(" pnum ","
					      (car cmd) res)
			       (loop (cdr cmd)
				     (string-append ", " (car cmd) res))))))))
	  ;; we now call gdb using the ignore filter the result
	  ;; if this command raise an error, it will be catched
	  ;; by the standard reader and the call will thus returns
	  ;; #f instead of #t on normal termination.
	  (if (gdb-call-for-output cmd)
	      ;; we now read on the socket for the client answer
	      (read-client-reply)
	      #f))))
	  

