;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bdb/bdb/Patient/server.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jul 27 17:12:42 1999                          */
;*    Last change :  Thu Oct 14 14:17:08 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Establish a socket server to manage to talk the patient.         */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module patient_server
   (import tools_error
	   tools_speek
	   gdb_invoke)
   (export (close-server!)
	   (read-client-reply)
	   (get-server-number)))

;*---------------------------------------------------------------------*/
;*    *server* ...                                                     */
;*---------------------------------------------------------------------*/
(define *server* #unspecified)
(define *connected?* #f)

;*---------------------------------------------------------------------*/
;*    get-server-number ...                                            */
;*---------------------------------------------------------------------*/
(define (get-server-number)
   (if (socket? *server*)
       (socket-port-number *server*)
       (begin
	  (init-server!)
	  (get-server-number))))

;*---------------------------------------------------------------------*/
;*    read-client-reply ...                                            */
;*---------------------------------------------------------------------*/
(define (read-client-reply)
   ;; wait for the client connection
   (if (not *connected?*)
       (begin
	  (verbose 4 #"waiting for connection...")
	  (flush-output-port (current-output-port))
	  (socket-accept-connection *server*)
	  (set! *connected?* #t)
	  (verbose 4 #"ok.\n")))
   ;; then read the client output
   (let* ((kind (read (socket-input *server*))))
      (if (boolean? kind)
	  (let ((reply (read (socket-input *server*))))
	     (if kind
		 (begin
		    (verbose 4 "Read from patient [" reply "]" #\Newline)
		    (flush-output-port (socket-output *server*))
		    reply)
		 (let ((reply (string->obj reply)))
		    (verbose 4 "Read from patient [" reply "]" #\Newline)
		    (flush-output-port (socket-output *server*))
		    reply)))
	  (let ((p (open-output-string)))
	     (write kind p)
	     (error "read-client-reply"
		    "Illegal reply"
		    (close-output-port p))))))

;*---------------------------------------------------------------------*/
;*    init-server! ...                                                 */
;*    -------------------------------------------------------------    */
;*    This function setup the socker server that will communicate      */
;*    with the bigloo running process (the client).                    */
;*    -------------------------------------------------------------    */
;*    This function is called the first time there is a need for       */
;*    the server port number. This is why this particular function     */
;*    is not exported.                                                 */
;*---------------------------------------------------------------------*/
(define (init-server!)
   (if (not (socket? *server*))
       (begin
	  (set! *server* (make-server-socket))
	  (if (not (socket? *server*))
	      (bdb-error "bdb" "Can't setup server" *server*)))))

;*---------------------------------------------------------------------*/
;*    close-server! ...                                                */
;*    -------------------------------------------------------------    */
;*    We are done with bdb, we stop the server.                        */
;*    -------------------------------------------------------------    */
;*    This function is only called once, on termination of BDB.        */
;*---------------------------------------------------------------------*/
(define (close-server!)
   (if (socket? *server*)
       (begin
	  (verbose 4 #"Closing server.\n")
	  (socket-shutdown *server*)
	  (set! *connected?* #f)
	  (set! *server* #unspecified))))


