;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bdb/bdb/Tools/tools.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Jul 28 15:50:51 1999                          */
;*    Last change :  Mon Aug 16 10:12:10 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Various tools (string tokenification, ...).                      */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module tools_tools
   (extern (export dummy "GC_get_back_ptr_info"))
   (export (dummy ::long ::long ::long))
   (export (num->string::bstring ::obj)
	   (string-from::bstring ::bstring ::char ::int)
	   (string-from-at::bstring ::bstring ::int ::char ::int)
	   (string-until::bstring ::bstring ::char ::int)
	   (string-until-at::bstring ::bstring ::int ::char ::int)
	   (bigloo-ident?::bool ::bstring)
	   (fetch-source-line::bstring ::bstring ::bstring)))

;*---------------------------------------------------------------------*/
;*    num->string ...                                                  */
;*---------------------------------------------------------------------*/
(define (num->string num)
   (if (< num 0.01)
       "0.0"
       (let* ((num (if (and (flonum? num) (>fl num 100.0))
                       (inexact->exact num)
                       num))
              (str (number->string num)))
          (if (and (flonum? num) (>fx (string-length str) 4))
              (substring str 0 4)
              str))))

;*---------------------------------------------------------------------*/
;*    string-from ...                                                  */
;*    -------------------------------------------------------------    */
;*    Skip all characters until MARK, returns the rest of the          */
;*    string. Delta is how many characters to skip from the marker.    */
;*---------------------------------------------------------------------*/
(define (string-from string mark delta)
   (let ((len (string-length string)))
      (let loop ((i 0))
	 (cond
	    ((=fx i len)
	     "")
	    ((char=? (string-ref string i) mark)
	     (substring string (+fx i delta) len))
	    (else
	     (loop (+fx i 1)))))))

;*---------------------------------------------------------------------*/
;*    string-from-at ...                                               */
;*    -------------------------------------------------------------    */
;*    Skip all characters until MARK, returns the rest of the          */
;*    string. Delta is how many characters to skip from the marker.    */
;*---------------------------------------------------------------------*/
(define (string-from-at string start mark delta)
   (let ((len (string-length string)))
      (let loop ((i start))
	 (cond
	    ((=fx i len)
	     "")
	    ((char=? (string-ref string i) mark)
	     (substring string (+fx i delta) len))
	    (else
	     (loop (+fx i 1)))))))

;*---------------------------------------------------------------------*/
;*    string-until ...                                                 */
;*    -------------------------------------------------------------    */
;*    Skip all characters after MARK, returns the rest of the          */
;*    string. Delta is how many characters to skip from the marker.    */
;*---------------------------------------------------------------------*/
(define (string-until string mark delta)
   (string-until-at string 0 mark delta))

;*---------------------------------------------------------------------*/
;*    string-until-at ...                                              */
;*    -------------------------------------------------------------    */
;*    Skip all characters after MARK, returns the rest of the          */
;*    string. Delta is how many characters to skip from the marker.    */
;*---------------------------------------------------------------------*/
(define (string-until-at string start mark delta)
   (let ((len (string-length string)))
      (let loop ((i start))
	 (cond
	    ((>=fx i len)
	     "")
	    ((char=? (string-ref string i) mark)
	     (substring string start (+fx i delta)))
	    (else
	     (loop (+fx i 1)))))))

;*---------------------------------------------------------------------*/
;*    bigloo-ident? ...                                                */
;*    -------------------------------------------------------------    */
;*    The only way to write a correct parser here is to use the        */
;*    Bigloo regular-expression for identifier, simply removing lower  */
;*    case symbols.                                                    */
;*---------------------------------------------------------------------*/
(define (bigloo-ident? str)
   (let* ((g (regular-grammar ((letter  (in ("AZ") (#a128 #a255)))
			       (special (in "!@~$%^&*></-_+\\|=?.:"))
			       (id      (: (* digit)
					   (or letter special)
					   (* (or letter
						  special
						  digit
						  (in ",'`"))))))
		(id (=fx (the-length) (string-length str)))
		(else #f)))
	  (p (open-input-string str))
	  (r (read/rp g p)))
      (close-input-port p)
      r))
	 
;*---------------------------------------------------------------------*/
;*    fetch-source-line ...                                            */
;*    -------------------------------------------------------------    */
;*    Extract one source line. If an error is raised, produced a       */
;*    gdb like error string message.                                   */
;*---------------------------------------------------------------------*/
(define (fetch-source-line::bstring file::bstring sline::bstring)
   (let ((line (string->integer sline)))
      (if (not (file-exists? file))
	  (string-append "No source file named " file ".")
	  (let ((port (open-input-file file)))
	     (unwind-protect
		(let loop ((str  (read-line port))
			   (lnum 1))
		   (cond
		      ((eof-object? str)
		       (string-append "Line number "
				      sline
				      " out of range; "
				      (basename file)
				      " has " (integer->string lnum) "lines."))
		      ((=fx lnum line)
		       (string-append sline
				      (if (< line 10) "  " " ")
				      str))
		      (else
		       (loop (read-line port) (+fx lnum 1)))))
		(close-input-port port))))))
   
;*---------------------------------------------------------------------*/
;*    dummy ...                                                        */
;*    -------------------------------------------------------------    */
;*    Just to get rid of link problems.                                */
;*---------------------------------------------------------------------*/
(define (dummy l1 l2 l3)
   'nop)
