/*---------------------------------------------------------------------*/
/*    A practical implementation for the Scheme programming language   */
/*                                                                     */
/*                                    ,--^,                            */
/*                              _ ___/ /|/                             */
/*                          ,;'( )__, ) '                              */
/*                         ;;  //   L__.                               */
/*                         '   \\   /  '                               */
/*                              ^   ^                                  */
/*                                                                     */
/*               Copyright (c) 1992-1999 Manuel Serrano                */
/*                                                                     */
/*     Bug descriptions, use reports, comments or suggestions are      */
/*     welcome. Send them to                                           */
/*       bigloo@kaolin.unice.fr                                        */
/*       http://kaolin.unice.fr/bigloo                                 */
/*                                                                     */
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bdk/kbdb/kbdb/Tools/tty.c                   */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Thu Aug 12 08:11:44 1999                          */
/*    Last change :  Mon Aug 23 16:59:04 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    TTY management.                                                  */
/*=====================================================================*/
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <termios.h>

/*---------------------------------------------------------------------*/
/*    The standard bdb getchar function.                               */
/*---------------------------------------------------------------------*/
#if( defined( fgetc ) )
int my_fgetc( FILE *stream ) {
   return fgetc( stream );
}
int (*bdb_getc)(FILE *) = my_fgetc;
#else
int (*bdb_getc)(FILE *) = fgetc;
#endif

/*---------------------------------------------------------------------*/
/*    static char                                                      */
/*    tty_bufname ...                                                  */
/*---------------------------------------------------------------------*/
static char tty_bufname[ 20 ];

/*---------------------------------------------------------------------*/
/*    The file descriptors of the tty                                  */
/*---------------------------------------------------------------------*/
static int tty_fd[ 2 ];

/*---------------------------------------------------------------------*/
/*    Control characters                                               */
/*---------------------------------------------------------------------*/
static char vintr, vquit;

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    open_tty ...                                                     */
/*---------------------------------------------------------------------*/
static int
open_tty( int fd[ 2 ] ) {
   int i, c;
   struct termios term;
   
   fd[ 0 ] = fd[ 1 ] = -1;
   
   for( c = 'p'; c <= 's'; c++ ) {
      for( i = 0; i < 16; i++ ) {
	 sprintf( tty_bufname, "/dev/pty%c%x", c, i );
	 
	 if( (fd[ 0 ] = open( tty_bufname, O_RDWR|O_NONBLOCK|O_NDELAY ) ) >= 0 ) {
	    
	    sprintf( tty_bufname, "/dev/tty%c%x", c, i );
	    
	    if( (fd[ 1 ] = open( tty_bufname, O_RDWR ) ) < 0 ) {
	       /* close the master because we cannot open the slave */
	       close( fd[ 0 ] );
	       fd[ 0 ] = -1;
	       continue;
	    }
	    /* ok we have found and opened a tty */
	    /* we change the terminal behavior to get ride of CR */
      
	    if( tcgetattr( fd[ 0 ], &term ) < 0 )
	       goto err;
	    
	    term.c_oflag &= ~ONLCR;
	    term.c_iflag |= IGNCR;

	    vintr = term.c_cc[ VINTR ];
	    vquit = term.c_cc[ VQUIT ];

	    if( tcsetattr( fd[ 0 ], TCSANOW, &term ) < 0 )
	       goto err;
	    
	    return 0;
	 }
      }
   }

 err:
   /* Error, we cannot open a tty */
  if( fd[ 0 ] != -1 ) close( fd[ 0 ] );
  if( fd[ 1 ] != -1 ) close( fd[ 1 ] );
  return -1;
}

/*---------------------------------------------------------------------*/
/*    char *                                                           */
/*    tty_init ...                                                     */
/*    -------------------------------------------------------------    */
/*    Initializes a TTY and returns the name of the opened TTY.        */
/*    Returns the empty string if failure.                             */
/*---------------------------------------------------------------------*/
char *
tty_init() {
   if( open_tty( tty_fd ) )
      return "";
   else {
      return tty_bufname;
   }
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    get_tty0 ...                                                     */
/*---------------------------------------------------------------------*/
int
get_tty0() {
   return tty_fd[ 0 ];
}

/*---------------------------------------------------------------------*/
/*    char *                                                           */
/*    tty_read ...                                                     */
/*    -------------------------------------------------------------    */
/*    !!! WARNING !!! WARNING !!! WARNING !!! WARNING !!! WARNING !!!  */
/*    -------------------------------------------------------------    */
/*    This function is left here for the example. It is actually       */
/*    _NEVER CALLED_ because the actual reading on a tty is only       */
/*    operated on by kbdb (see the Kbdb/ttyk.c file).                  */
/*    -------------------------------------------------------------    */
/*    Tries to read something on a tty. If there is something to       */
/*    be read, return it in a NULL terminated string. Otherwise,       */
/*    returns the empty string.                                        */
/*---------------------------------------------------------------------*/
char *
tty_read() {
   static int   read_buf_len = 1024;
   static char *read_buf     = 0;
   int n, m = 0;
   int raw = read_buf_len;

   if( !read_buf ) read_buf = (char *)malloc( read_buf_len + 1 );

   while( (n = read( tty_fd[ 0 ], read_buf, raw )) == raw ) {
      m += n;
      read_buf_len += raw;
      read_buf = (char *)realloc( read_buf, read_buf_len + 1 );
   }

   if( (n + m) <= 0 ) {
      return "";
   }
   else {
      read_buf[ m + n ] = 0;
      return read_buf;
   }
}
   
/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    tty_write ...                                                    */
/*---------------------------------------------------------------------*/
int
tty_write( char *str, int len ) {
   return write( tty_fd[ 0 ], str, len );
}
   
