;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bdb/blib/bdb.scm                     */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Jul 28 07:10:43 1999                          */
;*    Last change :  Thu Oct 14 14:37:16 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bdb library                                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __bdb
   
   (extern (bdb-heap-size::int ()                      "bdb_heap_size")
	   (bdb-alloc-gc::int ()                       "bdb_alloc_gc")
	   (bdb-gc-number::int ()                      "bdb_gc_number")
	   (bdb-gc-explore::int (::int ::obj ::int)         "bdb_gc_explore")
	   (bdb-gc-explore-offset::int (::int ::long ::int) "bdb_gc_explore_offset")
	   (bdb-gc-dump::int (::int ::string ::int ::int ::int ::int) "bdb_gc_dump")

	   (macro puts::int (::string) "puts")
	   
	   (export bdb-close-client!   "bdb_close_client")
	   (export mangle              "bdb_mangle")
	   (export mangle-for-funcall  "bdb_mangle_for_funcall")
	   (export mangle2             "bdb_mangle2")
	   (export demangle            "bdb_demangle")
	   (export demangle2           "bdb_demangle2")
	   (export output-value        "bdb_output_value")
	   (export heap-info           "bdb_heap_info")
	   (export heap-explore        "bdb_heap_explore")
	   (export heap-explore-offset "bdb_heap_explore_offset")
	   (export heap-explore-send   "bdb_heap_explore_send")
	   (export heap-dump           "bdb_heap_dump")
	   (export heap-dump-send      "bdb_heap_dump_send")
	   (export bdb-print           "bdb_print")
	   (export bdb-funcall         "bdb_funcall")
	   (export bdb-bint            "bdb_bint")
	   (export bdb-bchar           "bdb_bchar")
	   (export bdb-true            "bdb_true")
	   (export bdb-false           "bdb_false")
	   (export bdb-nil             "bdb_nil")
	   (export bdb-find-type       "bdb_find_type"))
   
   (import __bdb_env
	   __bdb_dump)
   
   (export (bdb-initial-breakpoint)
	   (bdb-close-client!)
	   (mangle::int ::int ::string)
	   (mangle-for-funcall::int ::int ::string)
	   (mangle2::int ::int ::string ::string)
	   (demangle::int ::int ::string)
	   (demangle2::int ::int ::string ::string)
	   (output-value::int ::int ::obj ::bool)
	   (heap-info::int ::int)
	   (heap-explore::int ::int ::obj ::int)
	   (heap-explore-offset::int ::int ::long ::int)
	   (heap-explore-send::int ::int ::int ::int ::int ::string ::int ::string ::pair-nil ::int)
	   (heap-dump::int ::int ::string ::int ::int ::int ::int)
	   (heap-dump-send::int ::int ::obj)
	   (bdb-print::string ::obj ::bool)
	   (bdb-funcall::obj ::procedure ::pair-nil)
	   (bdb-bint ::bint)
	   (bdb-bchar ::bchar)
	   (bdb-true)
	   (bdb-false)
	   (bdb-nil::obj)
	   (bdb-find-type::string ::obj)))

;*---------------------------------------------------------------------*/
;*    bdb-initial-breakpoint ...                                       */
;*---------------------------------------------------------------------*/
(define (bdb-initial-breakpoint)
   #unspecified)
 
;*---------------------------------------------------------------------*/
;*    *client* ...                                                     */
;*---------------------------------------------------------------------*/
(define *client* #unspecified)

;*---------------------------------------------------------------------*/
;*    bdb:init-client! ...                                             */
;*    -------------------------------------------------------------    */
;*    This function is called by the server each time the server       */
;*    wants to communicate. There is no way to improve that because    */
;*    there is no way for the server to know that the client is        */
;*    already initialized. In consequence, we have to take care        */
;*    here not to initalize several time the same client.              */
;*---------------------------------------------------------------------*/
(define (bdb:init-client! port-number)
   ;; if we have already opened a client but from a different port-number
   ;; we start closing the previous client
   (if (and (socket? *client*)
	    (not (=fx (socket-port-number *client*) port-number)))
       (bdb-close-client!))
   (if (not (socket? *client*))
       (begin
	  ;; we setup the socket client
	  (set! *client* (make-client-socket "localhost" port-number))
	  (if (not (socket? *client*))
	      (error "bdb:init-client" "Can't setup client" *client*))
	  ;; we register its termination. This is mandatory to have
	  ;; a smooth termination of the client
	  (pragma "atexit( (void (*)(void))bdb_close_client )" )))
   0)
       
;*---------------------------------------------------------------------*/
;*    bdb-close-client! ...                                            */
;*---------------------------------------------------------------------*/
(define (bdb-close-client!)
   (if (socket? *client*)
       (begin
	  (socket-shutdown *client* #f)
 	  (set! *client* #unspecified))))

;*---------------------------------------------------------------------*/
;*    bdb-invoke ...                                                   */
;*---------------------------------------------------------------------*/
(define (bdb-invoke port-number kind obj)
   (bdb:init-client! port-number)
   ;; the kind of connection
   (write kind (socket-output *client*))
   (newline (socket-output *client*))
   ;; the value to be sent
   (write obj (socket-output *client*))
   (newline (socket-output *client*))
   ;; we flush the all transfer
   (flush-output-port (socket-output *client*)))

;*---------------------------------------------------------------------*/
;*    *mangling-cache* ...                                             */
;*---------------------------------------------------------------------*/
(define *demangling-cache* #unspecified)
(define *mangling-cache* #unspecified)

;*---------------------------------------------------------------------*/
;*    reset-demangling-cache! ...                                      */
;*---------------------------------------------------------------------*/
(define (reset-demangling-cache!)
   (set! *demangling-cache*
         (make-hash-table 1024
                          (lambda (o) (string->0..2^x-1 o 10))
                          car
                          string=?
                          64))
   (set! *mangling-cache*
         (make-hash-table 1024
                          (lambda (o) (string->0..2^x-1 o 10))
                          car
                          string=?
                          64)))

;*---------------------------------------------------------------------*/
;*    *bdb-module-info* ...                                            */
;*---------------------------------------------------------------------*/
(define *bdb-module-info* '())

;*---------------------------------------------------------------------*/
;*    mangle ...                                                       */
;*    -------------------------------------------------------------    */
;*    Mangle the identifier and returns to bdb.                        */
;*---------------------------------------------------------------------*/
(define (mangle port-number::int bgl::string)
   (let ((c (bgl->c bgl)))
      (bdb-invoke port-number #t c))
   0)

;*---------------------------------------------------------------------*/
;*    mangle2 ...                                                      */
;*    -------------------------------------------------------------    */
;*    Mangle the identifier and returns to bdb.                        */
;*---------------------------------------------------------------------*/
(define (mangle2 port-number::int bgl::string loc::string)
   (let ((c (c-bgl->c bgl loc)))
      (bdb-invoke port-number #t c))
   0)

;*---------------------------------------------------------------------*/
;*    mangle-for-funcall ...                                           */
;*    -------------------------------------------------------------    */
;*    Mangle the identifier and returns to bdb.                        */
;*    The returned identifier is the identifier of the closure         */
;*    associated to the Bigloo identifier.                             */
;*---------------------------------------------------------------------*/
(define (mangle-for-funcall port-number::int bgl::string)
   (let ((c (bgl->c-funcall bgl)))
      (bdb-invoke port-number #t c))
   0)

;*---------------------------------------------------------------------*/
;*    demangle ...                                                     */
;*    -------------------------------------------------------------    */
;*    Demangles a C identifier.                                        */
;*---------------------------------------------------------------------*/
(define (demangle port-number::int c::string)
   (let ((c (c->bgl c)))
      (bdb-invoke port-number #t c))
   0)

;*---------------------------------------------------------------------*/
;*    demangle2 ...                                                    */
;*    -------------------------------------------------------------    */
;*    Demangles a C identifier.                                        */
;*---------------------------------------------------------------------*/
(define (demangle2 port-number::int c::string loc::string)
   (let ((c (c-c->bgl c loc)))
      (bdb-invoke port-number #t c))
   0)
   
;*---------------------------------------------------------------------*/
;*    output-value ...                                                 */
;*---------------------------------------------------------------------*/
(define (output-value port-number::int value circle?::bool)
   (let ((port (open-output-string)))
      (if circle?
	  (write-circle value port)
	  (let ((len (get-write-length)))
	     (set-write-length! 10)
	     (write value port)
	     (set-write-length! len)))
      (let ((c (cons (find-runtime-type value)
		     (close-output-port port))))
	 (bdb-invoke port-number #t c)
	 0)))

;*---------------------------------------------------------------------*/
;*    *info-chunk* ...                                                 */
;*    -------------------------------------------------------------    */
;*    We must avoid allocating object when an HEAP-INFO request        */
;*    is received in order to deliver revelant informations.           */
;*---------------------------------------------------------------------*/
(define *info-chunk* (list #unspecified
			   #unspecified
			   #unspecified))

;*---------------------------------------------------------------------*/
;*    heap-info ...                                                    */
;*    -------------------------------------------------------------    */
;*    This function deliver to the debugger informations relative      */
;*    to the GC state (e.g., the heap, the number of allocations, ...) */
;*---------------------------------------------------------------------*/
(define (heap-info port-number::int)
   ;; the gc number
   (set-car! *info-chunk* (bdb-gc-number))
   ;; the heap size
   (set-car! (cdr *info-chunk*) (bdb-heap-size))
   ;; the number of allocation since last gc
   (set-car! (cddr *info-chunk*) (bdb-alloc-gc))
   ;; the serialization
   (let ((c *info-chunk*))
      (bdb-invoke port-number #t c)
      0))

;*---------------------------------------------------------------------*/
;*    heap-explore ...                                                 */
;*    -------------------------------------------------------------    */
;*    Explore an object in the heap.                                   */
;*---------------------------------------------------------------------*/
(define (heap-explore port-number::int obj verbose)
   (bdb-gc-explore port-number obj verbose))

;*---------------------------------------------------------------------*/
;*    heap-explore-offset ...                                          */
;*    -------------------------------------------------------------    */
;*    Explore an object in the heap.                                   */
;*---------------------------------------------------------------------*/
(define (heap-explore-offset port-number::int offset verbose)
   (bdb-gc-explore-offset port-number offset verbose))

;*---------------------------------------------------------------------*/
;*    heap-explore-send ...                                            */
;*---------------------------------------------------------------------*/
(define (heap-explore-send port-number::int
			   kind::int gen::int size::int
			   fun::string age::int
			   type::string link::pair-nil
			   base::int)
   (let* ((source (bgl-source fun))
	  (c (list kind gen size fun age type link base source)))
      (bdb-invoke port-number #t c)
      0))

;*---------------------------------------------------------------------*/
;*    heap-dump ...                                                    */
;*    -------------------------------------------------------------    */
;*    Dump an object in the heap.                                      */
;*---------------------------------------------------------------------*/
(define (heap-dump port-number::int fname width depth mode hide)
   (bdb-gc-dump port-number fname width depth mode hide))

;*---------------------------------------------------------------------*/
;*    heap-dump-send ...                                               */
;*---------------------------------------------------------------------*/
(define (heap-dump-send port-number::int legend)
   (let ((c legend))
      (bdb-invoke port-number #t c)
      0))

;*---------------------------------------------------------------------*/
;*    bdb-print ...                                                    */
;*    -------------------------------------------------------------    */
;*    This function returns a string that is the result of the         */
;*    printing. This function does not perform any output. This        */
;*    function is called when gdb wants a value. For instance, it is   */
;*    used by the PRINT and the DISPLAY commands.                      */
;*---------------------------------------------------------------------*/
(define (bdb-print::string value::obj circle?::bool)
   (let ((port (open-output-string)))
      (if circle?
	  (write-circle value port)
	  (let ((len (get-write-length)))
	     (set-write-length! 10)
	     (write value port)
	     (set-write-length! len)))
      (close-output-port port)))

;*---------------------------------------------------------------------*/
;*    bdb-true ...                                                     */
;*---------------------------------------------------------------------*/
(define (bdb-true)
   #t)

;*---------------------------------------------------------------------*/
;*    bdb-false ...                                                    */
;*---------------------------------------------------------------------*/
(define (bdb-false)
   #f)

;*---------------------------------------------------------------------*/
;*    bdb-bchar ...                                                    */
;*---------------------------------------------------------------------*/
(define (bdb-bchar char)
   (pragma::obj "BCHAR( $1 )" (char->integer char)))
   
;*---------------------------------------------------------------------*/
;*    bdb-bint ...                                                     */
;*---------------------------------------------------------------------*/
(define (bdb-bint int)
   (pragma::obj "BINT( $1 )" int))
   
;*---------------------------------------------------------------------*/
;*    bdb-nil ...                                                      */
;*---------------------------------------------------------------------*/
(define (bdb-nil)
   '())

;*---------------------------------------------------------------------*/
;*    bdb-funcall ...                                                  */
;*---------------------------------------------------------------------*/
(define (bdb-funcall fun::procedure pair::pair-nil)
   (apply fun pair))

;*---------------------------------------------------------------------*/
;*    bdb-find-type ...                                                */
;*---------------------------------------------------------------------*/
(define (bdb-find-type obj)
   (find-runtime-type obj))
		       
