/*---------------------------------------------------------------------*/
/*    A practical implementation for the Scheme programming language   */
/*                                                                     */
/*                                    ,--^,                            */
/*                              _ ___/ /|/                             */
/*                          ,;'( )__, ) '                              */
/*                         ;;  //   L__.                               */
/*                         '   \\   /  '                               */
/*                              ^   ^                                  */
/*                                                                     */
/*               Copyright (c) 1992-1999 Manuel Serrano                */
/*                                                                     */
/*     Bug descriptions, use reports, comments or suggestions are      */
/*     welcome. Send them to                                           */
/*       bigloo@kaolin.unice.fr                                        */
/*       http://kaolin.unice.fr/bigloo                                 */
/*                                                                     */
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bigloo/bdb/blib/gc_dump.c                   */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Tue Aug  3 10:13:43 1999                          */
/*    Last change :  Thu Oct 14 14:49:42 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The graphical dump of a heap. This produces a pixmap             */
/*    representing the heap. Each memory word is represented by one    */
/*    pixel in that image.                                             */
/*=====================================================================*/
#define I_HIDE_POINTERS
#include <gc_priv.h>
#include <backptr.h>
#include <bigloo2.1a.h>

/*---------------------------------------------------------------------*/
/*    Dump modes                                                       */
/*---------------------------------------------------------------------*/
#define DUMP_REFERENCES   -1
#define DUMP_GENERATIONS  -2
#define DUMP_TYPES        -3

/*---------------------------------------------------------------------*/
/*    Strings allocation file                                          */
/*---------------------------------------------------------------------*/
#define OH_FUNCTION( x ) \
   (SYMBOLP( x ) ? \
     BSTRING_TO_STRING( SYMBOL_TO_STRING( x ) ) : \
     "BDB:SYSTEM")

#define BDB_PREFIX "BDB:"
#define STRING_ALLOC_FILE "Clib/cstring.c"

/*---------------------------------------------------------------------*/
/*    Importations                                                     */
/*---------------------------------------------------------------------*/
extern char *bdb_find_type( obj_t );

/*---------------------------------------------------------------------*/
/*    Boehm's GC header description                                    */
/*---------------------------------------------------------------------*/
typedef struct {
   ptr_t oh_back_ptr;
#ifdef ALIGN_DOUBLE
   word oh_dummy;
#endif
   char * oh_string;		/* object descriptor string	*/
   word oh_int;		        /* object descriptor integers	*/
#ifdef NEED_CALLINFO
   struct callinfo oh_ci[NFRAMES];
#endif
   word oh_sz;			/* Original malloc arg.		*/
   word oh_sf;			/* start flag */
} oh;

/*---------------------------------------------------------------------*/
/*    OH_TO_OBJ ...                                                    */
/*    -------------------------------------------------------------    */
/*    Convert an OH object into a Bigloo object.                       */
/*---------------------------------------------------------------------*/
#define OH_TO_OBJ( _oh ) (((obj_t)((((oh *)(_oh)))+1)))

/*---------------------------------------------------------------------*/
/*    Standard colors                                                  */
/*---------------------------------------------------------------------*/
#define COLOR0 "#ffffff"
#define COLOR1 "#ffff00"
#define COLOR2 "#ff0000"
#define COLOR3 "#00cd00"
#define COLOR4 "#6a5acd"
#define COLOR5 "#ffa500"
#define COLOR6 "#00ffff"
#define COLOR7 "#d02090"

#define ROOT_CHAR      'c'
#define LEAK_CHAR      'b'
#define LEAK_ROOT_CHAR 'd'

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    GC_print_debug_info ...                                          */
/*---------------------------------------------------------------------*/
static void
GC_print_debug_info( oh *base, void *to_ptr, long offset ) {
   printf( "----- base: %p\n", base );
   printf( "    marked: %d\n", GC_is_marked( base ) );
   printf( "  dbg size: %d\n", GC_size( base ) );
   printf( "      size: %d\n", GC_size( base ) - sizeof( oh ) );
   printf( "     oh-sz: %d\n", base->oh_sz );
   printf( "      file: %s\n", OH_FUNCTION( base->oh_string ) );
   printf( "      line: %d\n", base->oh_int & 0xffff );
   printf( "       gen: %d\n", (base->oh_int & ~0xffff) >> 16 );
   printf( "       ptr: %p\n", to_ptr );
   printf( "    offset: %p\n", offset );
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    dump_addr ...                                                    */
/*---------------------------------------------------------------------*/
static int
dump_addr( long heap_offset, void *addr ) {
   size_t       offset = 0;
   void        *base = 0;
   void        *gc_addr;
   GC_ref_kind  kind;

   gc_addr = GC_base( addr );
   kind = GC_get_back_ptr_info( gc_addr, &base, &offset );
   printf( "inspecting address: %p -> %p\n", heap_offset, gc_addr );
   printf( "              size: %d\n", GC_size( addr ) );

   switch( kind ) {
      case GC_UNREFERENCED:
	 break;
	 
      case GC_NO_SPACE:
	 printf( "----- base: %p\n", gc_addr );
	 printf( "  no space: \n" );
	 break;

      case GC_REFD_FROM_ROOT:
      case GC_REFD_FROM_HEAP:
	 ;
      case GC_FINALIZER_REFD: {
	 int type = TYPE( OH_TO_OBJ( gc_addr ) );
	 
	 printf( "              type: %d\n", type );
	 GC_print_debug_info( (oh *)gc_addr, base, (long)offset );
	 break;
      }
      
   }
}

/*---------------------------------------------------------------------*/
/*    void *                                                           */
/*    GC_heap_start_addr ...                                           */
/*    -------------------------------------------------------------    */
/*    The first address of the GC heap.                                */
/*---------------------------------------------------------------------*/
void *
GC_heap_start_addr() {
   return GC_least_plausible_heap_addr;
}

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    the_leak_culprit_size ...                                        */
/*---------------------------------------------------------------------*/
static int the_leak_culprit_size = ALIGNMENT_VALUE;

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    leak_culprit_size ...                                            */
/*---------------------------------------------------------------------*/
static int
leak_culprit_size() {
   return the_leak_culprit_size;
}

/*---------------------------------------------------------------------*/
/*    void *                                                           */
/*    leak_culprit_addr ...                                            */
/*    -------------------------------------------------------------    */
/*    The address of the root that makes the leak.                     */
/*---------------------------------------------------------------------*/
void *
leak_culprit_addr( void *current ) {
   void *base;
   size_t offset;
   GC_ref_kind source;
   void *gc_addr = GC_base( current );

   source = GC_get_back_ptr_info( gc_addr, &base, &offset );

   switch( source ) {
      case GC_UNREFERENCED:
	 the_leak_culprit_size = ALIGNMENT_VALUE;
	 return 0L;
	 
      case GC_NO_SPACE:
	 the_leak_culprit_size = ALIGNMENT_VALUE;
	 return 0L;
	    
      case GC_REFD_FROM_ROOT:
	 the_leak_culprit_size = GC_size( gc_addr );
	 return current;
	 
      case GC_FINALIZER_REFD:
	 the_leak_culprit_size = GC_size( gc_addr );
	 return current;
	 
      case GC_REFD_FROM_HEAP: {
	 return leak_culprit_addr( GC_base( base ) );
      }
   }
}
   
/*---------------------------------------------------------------------*/
/*    static void *                                                    */
/*    GC_get_offset_addr ...                                           */
/*    -------------------------------------------------------------    */
/*    Find an address is the heap located at [heap_offset] from the    */
/*    heap starting address.                                           */
/*---------------------------------------------------------------------*/
static void *
GC_get_offset_addr( long heap_offset ) {
   int i;
   
   for( i = 0; i < GC_n_heap_sects; i++ ) {
      int size = GC_heap_sects[ i ].hs_bytes;
      if( heap_offset < size ) {
	 return GC_base( GC_heap_sects[ i ].hs_start + heap_offset );
      } else {
	 heap_offset -= size;
      }
   }
   
   return 0;
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    get_reference_pixel_color ...                                    */
/*---------------------------------------------------------------------*/
static int
get_reference_pixel_color( void *addr, int hide_bdb ) {
   size_t      offset = 0;
   void       *base = 0;
   void       *gc_addr;
   GC_ref_kind kind;
   void       *heap_offset = (void *)((unsigned long)addr & (~TAG_MASK));
   char       *s;

   gc_addr = GC_base( addr );
   s = ((oh *)gc_addr)->oh_string;
   kind = GC_get_back_ptr_info( gc_addr, &base, &offset );

   if( hide_bdb &&
       (!SYMBOLP( s ) || !strncmp( OH_FUNCTION( s ), BDB_PREFIX, 4 ) ) ) {
      return '-';
   }
   
   switch( kind ) {
      case GC_UNREFERENCED:
	 return 'a';
	 
      case GC_NO_SPACE:
	 return 'b';

      case GC_REFD_FROM_ROOT:
	 return 'c';
	 
      case GC_REFD_FROM_HEAP:
	 return 'd';

      case GC_FINALIZER_REFD:
	 return 'e';
   }
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    get_generation_pixel_color ...                                   */
/*---------------------------------------------------------------------*/
static int
get_generation_pixel_color( void *addr, int hide_bdb ) {
   size_t      offset = 0;
   void       *base = 0;
   void       *gc_addr;
   GC_ref_kind kind;
   void       *heap_offset = (void *)((unsigned long)addr & (~TAG_MASK));

   gc_addr = GC_base( addr );
   kind = GC_get_back_ptr_info( gc_addr, &base, &offset );

   switch( kind ) {
      case GC_UNREFERENCED:
	 return 'a';
	 
      case GC_NO_SPACE:
	 return 'b';

      case GC_REFD_FROM_ROOT:
      case GC_REFD_FROM_HEAP:
      case GC_FINALIZER_REFD: {
	 int age = ((((oh *)gc_addr)->oh_int & ~0xffff) >> 16);
	 char *s = ((oh *)gc_addr)->oh_string;

	 if( hide_bdb &&
	     (!SYMBOLP( s ) || !strncmp( OH_FUNCTION(s), BDB_PREFIX, 4 ) ) ) {
	    return '-';
	 }
	 
	 switch ( GC_gc_no - age ) {
		  
	    case 1:
	       /* 1 gc old */
	       return 'c';
	       
	    case 2:
	       /* 2 gc old */
	       return 'd';
	       
	    case 3:
	    case 4:
	    case 5:
	       /* 3-5 gc old */
	       return 'e';

	    case 6:
	    case 7:
	    case 8:
	    case 9:
	    case 10:
	       /* 6-10 gc old */
	       return 'f';

	    default:
	       /* very old */
	       return 'g';
	 }
      }
   }
}

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    leak_gc_num ...                                                  */
/*---------------------------------------------------------------------*/
static int leak_gc_num = 0;

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    get_leak_pixel_color ...                                         */
/*---------------------------------------------------------------------*/
static int
get_leak_pixel_color( void *addr, int hide_bdb ) {
   size_t      offset = 0;
   void       *base = 0;
   void       *gc_addr;
   GC_ref_kind kind;
   void       *heap_offset = (void *)((unsigned long)addr & (~TAG_MASK));

   gc_addr = GC_base( addr );
   kind = GC_get_back_ptr_info( gc_addr, &base, &offset );

   switch( kind ) {
      case GC_UNREFERENCED:
      case GC_NO_SPACE:
	 return '-';

      case GC_REFD_FROM_ROOT:
      case GC_REFD_FROM_HEAP:
      case GC_FINALIZER_REFD: {
	 int age = ((((oh *)gc_addr)->oh_int & ~0xffff) >> 16);

	 return ( age >= leak_gc_num ) ? LEAK_CHAR : '-';
      }
   }
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    get_type_pixel_color ...                                         */
/*---------------------------------------------------------------------*/
static int
get_type_pixel_color( void *addr, int hide_bdb ) {
   size_t      offset = 0;
   void       *base = 0;
   void       *gc_addr;
   GC_ref_kind kind;
   void       *heap_offset = (void *)((unsigned long)addr & (~TAG_MASK));

   gc_addr = GC_base( addr );
   kind = GC_get_back_ptr_info( gc_addr, &base, &offset );

   switch( kind ) {
      case GC_UNREFERENCED:
      case GC_NO_SPACE:
	 return 'a';

      case GC_REFD_FROM_ROOT:
      case GC_REFD_FROM_HEAP:
      case GC_FINALIZER_REFD: {
	 obj_t tobj = OH_TO_OBJ( gc_addr );
	 hdr *hhdr = GC_find_header( tobj );
	 int type = TYPE( tobj );
	 oh *o = gc_addr;
	 char *s = o->oh_string;

	 if( hide_bdb &&
	     (!SYMBOLP( s ) || !strncmp( OH_FUNCTION(s), BDB_PREFIX, 4 ) ) ) {
	    return '-';
	 }
	 
	 if( hhdr->hb_obj_kind ) {
	    /* type is >0 */
	    return 'b' + (type % 6);
	 } else {
	    if( 00 && !strcmp( s, STRING_ALLOC_FILE ) )
	       return 'b' + (type % 6);
	    else
	       return 'a';
	 }
      }
   }
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    address_to_offset ...                                            */
/*    -------------------------------------------------------------    */
/*    From an [address] in the heap return the corresponding offset.   */
/*    -------------------------------------------------------------    */
/*    This function is the inverse of [GC_get_offset_addr].            */
/*---------------------------------------------------------------------*/
long
address_to_offset( void *addr ) {
   int i;
   long heap_offset = 0;
   
   for( i = 0; i < GC_n_heap_sects; i++ ) {
      int size = GC_heap_sects[ i ].hs_bytes;
      void *start = GC_base( GC_heap_sects[ i ].hs_start );
			     
      if( (start <= addr) && ((start + size) > addr) )
	 return (heap_offset + (addr - start)) / ALIGNMENT_VALUE;
      else
	 heap_offset += size;
   }
   
   return -1;
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    set_pixel_culprit_color ...                                      */
/*    -------------------------------------------------------------    */
/*    In the file FOUT, change the pixel color the be the color of a   */
/*    leak root.                                                       */
/*---------------------------------------------------------------------*/
void
set_pixel_culprit_color( FILE *fout,
			 long fpos,
			 int width,
			 void *culprit_addr,
			 char color ) {
   long delta = address_to_offset( culprit_addr );

   if( delta >= 0 ) {
      long lnum  = delta / width;
      long cnum  = delta % width;
      /* to get the correct char pos, we have to add 4 ('"' + '"' + ','   */
      /* + '\n' for each line that we must skip, plus 1 ('"') for the     */
      /* beginning of the current line.                                   */
      long char_pos = delta + 1 + (lnum * 4) + fpos;
      int size = leak_culprit_size();
      fseek( fout, char_pos, SEEK_SET );

/*       printf( "set_pixel_culprit_color: %d %d %p  %p %p init: %d  width: %d  cnum: %d psize: %d\n", */
/* 	      size, char_pos, culprit_addr, culprit_addr, start_addr, fpos, */
/* 	      width, cnum, size / ALIGNMENT_VALUE );                   */
	      
      while( size > 0 ) {
	 if( cnum == width ) {
	    /* we have to jump over one line end */
	    fseek( fout, 4, SEEK_CUR );
	    cnum = 1;
	 } else
	    cnum++;
	 
	 size -= ALIGNMENT_VALUE;
	 fputc( color, fout );
      }
   }
}

/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    make_gc_sectors_list ...                                         */
/*    -------------------------------------------------------------    */
/*    Make a Bigloo list of strings that is composed of the GC         */
/*    sectors description. We use strings instead of number is order   */
/*    to avoid Bigloo arithmetic limitations.                          */
/*---------------------------------------------------------------------*/
static obj_t
make_gc_sectors_list( int i ) {
   return BNIL;
/*    if( i == GC_n_heap_sects )                                       */
/*       return BNIL;                                                  */
/*    else {                                                           */
/*       char addr1[ 20 ], addr2[ 20 ];                                */
/*       sprintf( addr1, "%ld", GC_heap_sects[ i ].hs_start );         */
/*       sprintf( addr2, "%ld", GC_heap_sects[ i ].hs_bytes );         */
/*       return MAKE_PAIR( MAKE_PAIR( string_to_bstring( addr1 ),      */
/* 				   string_to_bstring( addr2 ) ),       */
/* 			make_gc_sectors_list( i + 1 ) );               */
/*    }                                                                */
}
   
/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    make_legend ...                                                  */
/*---------------------------------------------------------------------*/
static obj_t
make_legend( int mode, long leak_size ) {
   switch( mode ) {
      case DUMP_REFERENCES:
	 return 
 	    MAKE_PAIR(
	       make_gc_sectors_list( 0 ),
	       MAKE_PAIR(
		  BINT( leak_size ),
		  MAKE_PAIR( 
		     string_to_bstring( "Reference" ),
		     MAKE_PAIR( 
			MAKE_PAIR( string_to_bstring( COLOR0 ),
				   string_to_bstring( "Free" ) ),
			MAKE_PAIR( 
			   MAKE_PAIR( string_to_bstring( COLOR1 ),
				      string_to_bstring( "Unreferenced" ) ),
			   MAKE_PAIR( 
			      MAKE_PAIR( string_to_bstring( COLOR2 ),
					 string_to_bstring( "Undebugged" ) ),
			      MAKE_PAIR( 
				 MAKE_PAIR( string_to_bstring( COLOR3 ),
					    string_to_bstring( "Root" ) ),
				 MAKE_PAIR( 
				    MAKE_PAIR( string_to_bstring( COLOR4 ),
					       string_to_bstring( "Heap" ) ),
				    MAKE_PAIR( 
				       MAKE_PAIR( string_to_bstring( COLOR5 ),
						  string_to_bstring( "Finalizer" ) ),
				       BNIL ) ) ) ) ) ) ) ) );
      case DUMP_GENERATIONS:
	 return
	    MAKE_PAIR(
	       make_gc_sectors_list( 0 ),
	       MAKE_PAIR(
		  BINT( leak_size ),
		  MAKE_PAIR(
		     string_to_bstring( "Age (in GCs)" ),
		     MAKE_PAIR( 
			MAKE_PAIR( string_to_bstring( COLOR0 ),
				   string_to_bstring( "Free" ) ),
			MAKE_PAIR( 
			   MAKE_PAIR( string_to_bstring( COLOR1 ),
				      string_to_bstring( "Unreferenced" ) ),
			   MAKE_PAIR( 
			      MAKE_PAIR( string_to_bstring( COLOR2 ),
					 string_to_bstring( "Nodebugged" ) ),
			      MAKE_PAIR( 
				 MAKE_PAIR( string_to_bstring( COLOR3 ),
					    string_to_bstring( "1" ) ),
				 MAKE_PAIR( 
				    MAKE_PAIR( string_to_bstring( COLOR4 ),
					       string_to_bstring( "2" ) ),
				    MAKE_PAIR( 
				       MAKE_PAIR( string_to_bstring( COLOR5 ),
						  string_to_bstring( "3-5" ) ),
				       MAKE_PAIR( 
					  MAKE_PAIR( string_to_bstring( COLOR6 ),
						     string_to_bstring( "6-10" ) ),
					  MAKE_PAIR( 
					     MAKE_PAIR( string_to_bstring( COLOR7 ),
							string_to_bstring( "> 10" ) ),
					     BNIL ) ) ) ) ) ) ) ) ) ) );
	 
      case DUMP_TYPES:
	 return 
	    MAKE_PAIR(
	       make_gc_sectors_list( 0 ),
	       MAKE_PAIR( BINT( leak_size ),
			  MAKE_PAIR(
			     string_to_bstring( "Type" ),
			     MAKE_PAIR( 
				MAKE_PAIR( string_to_bstring( COLOR0 ),
					   string_to_bstring( "Free" ) ),
				MAKE_PAIR( 
				   MAKE_PAIR( string_to_bstring( COLOR1 ),
					      string_to_bstring( "string buffer" ) ),
				   MAKE_PAIR( 
				      MAKE_PAIR( string_to_bstring( COLOR2 ),
						 string_to_bstring( "type 1" ) ),
				      MAKE_PAIR( 
					 MAKE_PAIR( string_to_bstring( COLOR3 ),
						    string_to_bstring( "type 2" ) ),
					 MAKE_PAIR( 
					    MAKE_PAIR( string_to_bstring( COLOR4 ),
						       string_to_bstring( "type 3" ) ),
					    MAKE_PAIR( 
					       MAKE_PAIR( string_to_bstring( COLOR5 ),
							  string_to_bstring( "type 4" ) ),
					       MAKE_PAIR( 
						  MAKE_PAIR( string_to_bstring( COLOR6 ),
							     string_to_bstring( "type 5" ) ),
						  MAKE_PAIR( 
						     MAKE_PAIR( string_to_bstring( COLOR7 ),
								string_to_bstring( "type 6" ) ),
						     BNIL ) ) ) ) ) ) ) ) ) ) );
      default:
	 return 
	    MAKE_PAIR(
	       make_gc_sectors_list( 0 ),
	       MAKE_PAIR(
		  BINT( leak_size ),
		  MAKE_PAIR(
		     string_to_bstring( "Leaks" ),
		     MAKE_PAIR( 
			MAKE_PAIR( string_to_bstring( COLOR0 ),
				   string_to_bstring( "No leak" ) ),
			MAKE_PAIR( 
			   MAKE_PAIR( string_to_bstring( COLOR2 ),
				      string_to_bstring( "Leak" ) ),
			   MAKE_PAIR( 
			      MAKE_PAIR( string_to_bstring( COLOR4 ),
					 string_to_bstring( "Root" ) ),
			      MAKE_PAIR( 
				 MAKE_PAIR( string_to_bstring( COLOR3 ),
					    string_to_bstring( "Leak root" ) ),
				 BNIL ) ) ) ) ) ) );
   }
}
	 
/*---------------------------------------------------------------------*/
/*    static void                                                      */
/*    bdb_dump_root_leak ...                                           */
/*    -------------------------------------------------------------    */
/*    When we want to see the memory leaking we have to complete       */
/*    two traversal. The first one finds the leaks and the second      */
/*    one finds the culprit roots. The BDB_DUMP_ROOT_LEAK is the       */
/*    second traversal.                                                */
/*    -------------------------------------------------------------    */
/*    The parameter of DUMP are:                                       */
/*       FOUT: the file to be modified.                                */
/*       FPOS: the position in the file where the heap starts          */
/*       WIDTH: the horizontal width of the pixmap.                    */
/*       DEPTH: the depth of the pixmap.                               */
/*---------------------------------------------------------------------*/
static long
bdb_dump_root_leak( FILE *fout, long fpos, int width, int depth ) {
   int height;
   long heap_offset = 0;
   int line = 0, col = 0;
   long heap_size = GC_get_heap_size();
   long leak_size = 0;
   
   /* we scan the whole heap */
   while( heap_offset < heap_size ) {
      void *gc_addr = GC_get_offset_addr( heap_offset );

      if( !gc_addr || !GC_is_marked( gc_addr ) ) {
	 heap_offset += ALIGNMENT_VALUE;
      } else {
	 int size = GC_size( gc_addr );

	 if( get_leak_pixel_color( gc_addr, 0 ) == LEAK_CHAR ) {
	    void *culprit_addr = leak_culprit_addr( gc_addr );

	    if( culprit_addr ) {
	       int col;
	       leak_size += size;

	       if( (culprit_addr == gc_addr) ||
		   (get_leak_pixel_color( culprit_addr, 0 ) == LEAK_CHAR ) )
		  col = LEAK_ROOT_CHAR;
	       else
		  col = ROOT_CHAR;

	       set_pixel_culprit_color( fout, fpos, width, culprit_addr, col );
	    }
	 }

	 heap_offset += size;
      }
   }

   /* we go the end of the file */
   fseek( fout, 0, SEEK_END );

   /* we return the global leak size */
   return leak_size;
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    bdb_dump_heap ...                                                */
/*    -------------------------------------------------------------    */
/*    The parameter of DUMP are:                                       */
/*       FILE-NAME: the file to be produced.                           */
/*       WIDTH: the horizontal width of the pixmap.                    */
/*       DEPTH: the depth of the pixmap.                               */
/*       MODE: a flag telling how to compute the color of each cell    */
/*---------------------------------------------------------------------*/
obj_t
bdb_dump_heap( char *fname, int width, int depth, int mode, int hide_bdb ) {
   int height;
   long heap_offset = 0;
   int line = 0, col = 0;
   long heap_size = GC_get_heap_size();
   FILE *fout;
   char *swidth = malloc( width + 1 );
   int (*f)( void *, int );
   long ftell_heap;
   long leak_size = -1;

   swidth[ width ] = 0;
   
/*    printf( "bdb_dump_heap [%s] [%d] [%d]\n", fname, width, depth ); */
/*    printf( "GC_heapsize : %db  %dMb\n", heap_size, heap_size / (1024 * 1024) ); */

   if( fout = fname ? fopen( fname, "w" ) : stdout ) {

      /* we compute the height of the pixmap */
      height = (heap_size / width) / ALIGNMENT_VALUE;

      /* we emit the xpm header */
      fprintf( fout, "/* XPM */\nstatic char *heap[] = {\n" );
      fprintf( fout, "/* width height num_colors chars_per_pixel */\n" );
      if( mode > 0 ) {
	 /* leak mode */
	 fprintf( fout, "\"   %d    %d     4          1\",\n", width, height );
	 fprintf( fout, "/* colors */\n" );
	 fprintf( fout, "\"-  c %s\",\n", COLOR0 );
	 fprintf( fout, "\"%c c %s\",\n", LEAK_CHAR, COLOR2 );
	 fprintf( fout, "\"%c c %s\",\n", LEAK_ROOT_CHAR, COLOR3 );
	 fprintf( fout, "\"%c c %s\",\n", ROOT_CHAR, COLOR4 );
	 fprintf( fout, "/* pixels */\n" );

	 /* we store the position in the file for the second traversal */
	 fflush( fout );
	 ftell_heap = ftell( fout );
      }
      else {
	 /* regular mode */
	 fprintf( fout, "\"   %d    %d     8          1\",\n", width, height );
	 fprintf( fout, "/* colors */\n" );
	 fprintf( fout, "\"- c %s\",\n", COLOR0 );
	 fprintf( fout, "\"a c %s\",\n", COLOR1 );
	 fprintf( fout, "\"b c %s\",\n", COLOR2 );
	 fprintf( fout, "\"c c %s\",\n", COLOR3 );
	 fprintf( fout, "\"d c %s\",\n", COLOR4 );
	 fprintf( fout, "\"e c %s\",\n", COLOR5 );
	 fprintf( fout, "\"f c %s\",\n", COLOR6 );
	 fprintf( fout, "\"g c %s\",\n", COLOR7 );
	 fprintf( fout, "/* pixels */\n" );
      }

      /* before entering the loop we setup the get_pixel_color function */
      switch( mode ) {
	 case DUMP_REFERENCES:
	    f = get_reference_pixel_color;
	    break;
	 case DUMP_GENERATIONS:
	    f = get_generation_pixel_color;
	    break;
	 case DUMP_TYPES:
	    f = get_type_pixel_color;
	    break;
	 default:
	    leak_gc_num = mode;
	    f = get_leak_pixel_color;
      }
	    
      /* we scan all the heap */
      while( line < height ) {
	 putc( '"', fout );
	 while( col < width ) {
	    if( heap_offset >= heap_size ) {
	       while( col++ < width )
		  swidth[ col ] = '-';
	       goto done;
	    } else {
	       void *gc_addr = GC_get_offset_addr( heap_offset );

	       if( !gc_addr || !GC_is_marked( gc_addr ) ) {
		  swidth[ col++ ] = '-';
		  heap_offset += ALIGNMENT_VALUE;
	       } else {
		  int size = GC_size( gc_addr );
		  int c = f( gc_addr, hide_bdb );
/* 		  printf( "offset: %p addr: %p  size: %d  color: %c (%d:%d)\n", */
/* 			  heap_offset, gc_addr, size, c, line, col );  */
		  heap_offset += size;
		  while( size > 0 ) {
		     size -= ALIGNMENT_VALUE;
		     swidth[ col++ ] = (!size ? '-' : c);
		     if( col >= width ) {
			/* this is a premature end of line */
			fputs( swidth, fout );
			fputs( "\",\n\"", fout );
			line++;
			col = 0;
		     }
		  }
	       }
	    }
	 }
	 fputs( swidth, fout );
	 fputs( "\",\n", fout );
	 col = 0;
	 line++;
      }

   done:
      fprintf( fout, "};\"\n\n " );

      /* when producing a leak dump we have to make a second traversal */
      /*   of the heap in order to print the roots of the leaks.       */
      if( mode > 0 )
	 leak_size = bdb_dump_root_leak( fout, ftell_heap, width, depth );

      free( swidth );
      if( fname ) fclose( fout );
      return make_legend( mode, leak_size );
   }
   return BNIL;
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    bdb_gc_dump ...                                                  */
/*---------------------------------------------------------------------*/
int
bdb_gc_dump( int port, char *file, int width, int depth, int mode, int hide ) {
   obj_t legend = bdb_dump_heap( file, width, depth, mode, hide );
   return bdb_heap_dump_send( port, legend );
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    bdb_heap_size ...                                                */
/*    -------------------------------------------------------------    */
/*    The size of the current heap in byte.                            */
/*---------------------------------------------------------------------*/
long
bdb_heap_size() {
   return (long)GC_get_heap_size();
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    bdb_alloc_gc ...                                                 */
/*    -------------------------------------------------------------    */
/*    The number of allocation since last GC.                          */
/*---------------------------------------------------------------------*/
long
bdb_alloc_gc() {
   return GC_get_bytes_since_gc();
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    bdb_gc_number ...                                                */
/*    -------------------------------------------------------------    */
/*    The number of gc executed.                                       */
/*---------------------------------------------------------------------*/
long
bdb_gc_number() {
   extern word GC_gc_no;
   return (long)GC_gc_no;
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    make_stop_chain ...                                              */
/*---------------------------------------------------------------------*/
obj_t
make_stop_chain( char *s ) {
   return MAKE_PAIR( string_to_bstring( s ), BNIL );
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    make_bstring_info ...                                            */
/*---------------------------------------------------------------------*/
obj_t
make_bstring_info( void *current, void *base, size_t offset, int verbose ) {
   void *new_base = GC_base( base );
   oh *o = current;
   char s[ 256 ];
   void *tmp_base;
   size_t tmp_offset;
   GC_ref_kind tmp_source;
   char *type;
   hdr *hhdr;
   obj_t tobj;
	 
   tobj = OH_TO_OBJ( current );
   hhdr = GC_find_header( tobj );
	 
   /* in order to request for a Bigloo identifier, we have to check */
   /* first that the object has been allocated with debug info      */
	 
   tmp_source = GC_get_back_ptr_info( new_base, &tmp_base, &tmp_offset );
/* 	 printf( "   FROM_HEAP [%p:%p] [%d:%s:%d] %d\n", base, tobj,    */
/* 		 o->oh_sz, o->oh_string, o->oh_int & 0xffff, tmp_source ); */
   switch( tmp_source) {
      case GC_UNREFERENCED:
	 type = "<???>";
	 break;
	 
      case GC_NO_SPACE:
	 type = "<no_space>";
	 break;
	    
      case GC_REFD_FROM_ROOT:
      case GC_FINALIZER_REFD:
      case GC_REFD_FROM_HEAP:
	 if( verbose ) {
	    printf( "make_link_chain...\n" );
	    printf( "bdb_find_type: %p\n", tobj );
	    printf( "         file: %s\n", OH_FUNCTION( o->oh_string ) );
	    printf( "         line: %d\n", o->oh_int & 0xffff);
	    printf( "          hdr: %p\n", hhdr );
	    printf( "         free: %d\n", hhdr->hb_obj_kind );
	 }
	       
	 type = hhdr->hb_obj_kind
	    ? bdb_find_type( tobj )
	    : "<string buffer>";

	 if( verbose ) {
	    printf( "         type: %s\n", type );
	 }
   }

/* 	 printf( "   TYPE [%s]\n", type );                             */
/* 	                                                               */
   sprintf( s, "%p (offset: %p) [%s]", tobj, offset, type );

   return string_to_bstring( s );
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    make_root_chain ...                                              */
/*---------------------------------------------------------------------*/
obj_t
make_root_chain( void *current, void *base, size_t offset ) {
   if( GC_is_visible( current ) ) {
      obj_t tobj;
      char *type;
      hdr  *hhdr;
      char s[ 256 ];
      
      tobj = OH_TO_OBJ( current );
      type = bdb_find_type( tobj );
      
      sprintf( s, "%p [%s]", tobj, type );
      
      return MAKE_PAIR( string_to_bstring( s ), BNIL );
   } else {
      char s[ 100 ];
      sprintf( s, "%p (offset: %p)", base, offset );
      return MAKE_PAIR( string_to_bstring( s ), BNIL );
   }
}

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    make_link_chain ...                                              */
/*    -------------------------------------------------------------    */
/*    Scan the entire link chain and make a Bigloo list of it.         */
/*---------------------------------------------------------------------*/
obj_t
make_link_chain( void *current, int verbose ) {
   void *base;
   size_t offset;
   GC_ref_kind source;

   source = GC_get_back_ptr_info( current, &base, &offset );
   
   switch( source ) {
      case GC_UNREFERENCED:
	 return make_stop_chain( "<???>" );
	 
      case GC_NO_SPACE:
	 return make_stop_chain( "<Undebugged>" );
	    
      case GC_REFD_FROM_ROOT:
	 return make_root_chain( current, base, offset );
	 
      case GC_FINALIZER_REFD:
	 return make_stop_chain( "<Finalizer>" );
	 
      case GC_REFD_FROM_HEAP: {
	 return MAKE_PAIR( make_bstring_info( current, base, offset, verbose ),
			   make_link_chain( GC_base( base ), verbose ) );
      }
   }
}

/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    bdb_gc_explore_internal ...                                      */
/*---------------------------------------------------------------------*/
static int
bdb_gc_explore_internal(  int port_num, void *gc_addr, int verbose ) {
   size_t      offset = 0;
   void       *base = 0;
   GC_ref_kind kind;
   oh         *o;
   char       *type;
   hdr        *hhdr;
   obj_t       tobj;

   if( gc_addr )
      kind = GC_get_back_ptr_info( (void *)gc_addr, &base, &offset );
   else
      return bdb_heap_explore_send( port_num,
				    0,          /* kind */
				    0,          /* gen  */
				    0,          /* size */
				    "",         /* fun  */
				    0,          /* age  */
				    "<???>",    /* type */
				    BNIL,       /* link */
				    gc_addr );

   switch( kind ) {
      case GC_UNREFERENCED:
	 return bdb_heap_explore_send( port_num,
				       0,       /* kind */
				       0,       /* gen  */
				       0,       /* size */
				       "",      /* fun  */
				       0,       /* age  */
				       "<???>", /* type */
				       BNIL,    /* link */
				       gc_addr );
	 break;
	 
      case GC_NO_SPACE:
	 return bdb_heap_explore_send( port_num,
				       1,       /* kind */
				       0,       /* gen  */
				       0,       /* size */
				       "",      /* fun  */
				       0,       /* age  */
				       "pair",  /* type */
				       BNIL,    /* link */
				       gc_addr );
	 break;

      case GC_REFD_FROM_ROOT:
 	 o    = gc_addr;
	 tobj = OH_TO_OBJ( gc_addr );

	 if( verbose ) {
	    printf( "bdb_gc_explore(ROOT)...\n" );
	    printf( "bdb_find_type: %p\n", tobj );
	    printf( "        tbase: %p\n", GC_base( tobj ) );
	    printf( "         file: %s\n", OH_FUNCTION( o->oh_string ) );
	    printf( "         line: %d\n", o->oh_int & 0xffff );
	    fflush( stdout );
	 }

	 type = bdb_find_type( tobj );

	 return bdb_heap_explore_send( port_num,
				       /* kind */
				       2,
				       /* gen  */
				       o->oh_int >> 16,
				       /* size */
				       o->oh_sz,
				       /* function */
				       OH_FUNCTION( o->oh_string ),
				       /* age */
				       o->oh_int & 0xffff,
				       /* type */
				       type,
				       /* link chain */
				       BNIL,
				       /* obj address */
				       tobj );
	 break;
	 
      case GC_REFD_FROM_HEAP:
      case GC_FINALIZER_REFD:
 	 o    = gc_addr;
	 tobj = OH_TO_OBJ( gc_addr );
	 hhdr = GC_find_header( tobj );

	 if( verbose ) {
	    printf( "bdb_gc_explore(HEAP)...\n" );
	    printf( "bdb_find_type: %p\n", tobj );
	    printf( "        tbase: %p\n", GC_base( tobj ) );
	    printf( "         file: %s\n", OH_FUNCTION( o->oh_string ) );
	    printf( "         line: %d\n", o->oh_int & 0xffff );
	    printf( "          hdr: %p\n", hhdr );
	    printf( "         free: %d\n", hhdr->hb_obj_kind );
	    fflush( stdout );
	 }

	 if( hhdr->hb_obj_kind )
	    type = bdb_find_type( tobj );
	 else {
	    if( 0 && !strcmp( o->oh_string, STRING_ALLOC_FILE ) )
	       type = bdb_find_type( tobj );
	    else
	       type = "<string buffer>";
	 }

	 return bdb_heap_explore_send( port_num,
				       /* kind */
				       2,
				       /* gen  */
				       o->oh_int >> 16,
				       /* size */
				       o->oh_sz,
				       /* file */
				       OH_FUNCTION( o->oh_string ),
				       /* lnum */
				       o->oh_int & 0xffff,
				       /* type */
				       type,
				       /* link chain */ 
				       make_link_chain( GC_base( base ),
							verbose ),
				       /* obj address */
				       tobj );
	 break;
   }
}


/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    bdb_gc_explore ...                                               */
/*---------------------------------------------------------------------*/
int
bdb_gc_explore( int port_num, obj_t obj, int verbose ) {
   void *gc_addr = (void *)(((unsigned long) obj) & (~TAG_MASK));

   return bdb_gc_explore_internal( port_num, GC_base( gc_addr ), verbose );
}
   
/*---------------------------------------------------------------------*/
/*    int                                                              */
/*    bdb_gc_explore_offset ...                                        */
/*---------------------------------------------------------------------*/
int
bdb_gc_explore_offset( int port_num, long heap_offset, int verbose ) {
   void *gc_addr = GC_get_offset_addr( heap_offset );

   return bdb_gc_explore( port_num, gc_addr, verbose );
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    GC_print_sect ...                                                */
/*---------------------------------------------------------------------*/
void
GC_print_sect() {
   int i;

   printf( "======= GC_print_sect ========================\n" );
   printf( "smallest : %p\n", GC_least_plausible_heap_addr );

   for( i = 0; i < GC_n_heap_sects; i++ ) {
      printf( "i: %d  size: %d (%dMg)  addr: %p  last addr: %p\n",
	      i,
	      GC_heap_sects[ i ].hs_bytes,
	      GC_heap_sects[ i ].hs_bytes / (1024 * 1024),
	      GC_heap_sects[ i ].hs_start,
	      (long)(GC_heap_sects[ i ].hs_start)
	      + GC_heap_sects[ i ].hs_bytes );
   }
}
