/*---------------------------------------------------------------------*/
/*    A practical implementation for the Scheme programming language   */
/*                                                                     */
/*                                    ,--^,                            */
/*                              _ ___/ /|/                             */
/*                          ,;'( )__, ) '                              */
/*                         ;;  //   L__.                               */
/*                         '   \\   /  '                               */
/*                              ^   ^                                  */
/*                                                                     */
/*               Copyright (c) 1992-1999 Manuel Serrano                */
/*                                                                     */
/*     Bug descriptions, use reports, comments or suggestions are      */
/*     welcome. Send them to                                           */
/*       bigloo@kaolin.unice.fr                                        */
/*       http://kaolin.unice.fr/bigloo                                 */
/*                                                                     */
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bigloo/bdb1/Lib/table.c                     */
/*    -------------------------------------------------------------    */
/*    Author      :  unknown                                           */
/*    Creation    :  Wed Jul 28 07:20:32 1999                          */
/*    Last change :  Wed Jul 28 07:30:23 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The mangling/demangling table management.                        */
/*    -------------------------------------------------------------    */
/*    The small C interface moslty used because Bigloo produces an     */
/*    huge C array that it is hard to parse in Scheme.                 */
/*=====================================================================*/
#include <bigloo2.1a.h>

/*---------------------------------------------------------------------*/
/*    The type used by Bigloo for the bdb table.                       */
/*---------------------------------------------------------------------*/
struct bdb_fun_info { char *sname, *cname; };

/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    bdb_table_to_list ...                                            */
/*    -------------------------------------------------------------    */
/*    This function construct a list of binding declarations from      */
/*    the table that has been build by the compiler in -gbdb mode.     */
/*---------------------------------------------------------------------*/
obj_t
bdb_table_to_list( obj_t bdb ) {
   obj_t glo_info  = BNIL;
   obj_t mod_info  = BNIL;
   obj_t init_info = BNIL;
   obj_t lnum_info = BNIL;
   struct bdb_fun_info *table_entry = (struct bdb_fun_info *)bdb;
   obj_t src_info = BNIL;
   long mod_lnum;
   
   /* we start fetching the module name and its            */
   /* C initialization function.                           */
   mod_info  = string_to_bstring( table_entry->sname );
   init_info = string_to_bstring( table_entry->cname );
   table_entry++;

   /* and the source files implementing this module.       */
   while( ((int *)table_entry->sname) ) {
      obj_t pair = MAKE_PAIR(string_to_bstring(table_entry->sname), src_info);
      src_info = pair;
      
      table_entry++;
   }
   
   mod_lnum  = (long)table_entry->cname;
   src_info  = MAKE_PAIR( init_info, src_info );
   lnum_info = MAKE_PAIR( BINT( mod_lnum ), src_info );
   mod_info  = MAKE_PAIR( mod_info, lnum_info );
   table_entry++;
   
   /* then we fetch global variables informations         */
   while( *((int *)table_entry) ) {
      char *fname, *sname, *cname;
      long lnum;
      obj_t pair = BNIL;
      obj_t entry = BNIL;

      /* we first fetch the source file name and line num */
      fname = table_entry->sname;
      lnum  = (long)table_entry->cname;
      table_entry++;

      /* now, we pickup the global scheme and C names     */
      sname = table_entry->sname;
      cname = table_entry->cname;

      /* is it a global function or a global variable ?   */
      if( !cname ) {
	 /* thie is a global function                     */
	 char *bp_cname;
	 obj_t pair2;

	 table_entry++;

	 cname    = table_entry->sname;
	 bp_cname = table_entry->cname;

	 pair2 = MAKE_PAIR( cname ? string_to_bstring( cname ) : BUNSPEC,
			    BINT( lnum ) );
	 pair2 = MAKE_PAIR( pair2, string_to_bstring( bp_cname ) );
	 table_entry++;
	 
	 /* this is a global function, we are now free    */
	 /* to parse the local variables                  */
	 while( table_entry->sname ) {
	    pair = MAKE_PAIR( string_to_bstring( table_entry->sname ),
			      string_to_bstring( table_entry->cname ) );

	    entry = MAKE_PAIR( pair, entry );

	    table_entry++;
	 }

	 pair2 = MAKE_PAIR( pair2, BNIL );
	 pair  = MAKE_PAIR( string_to_bstring( sname ), pair2 );
      } else {
	 /* this is a global variable.                    */
	 pair = MAKE_PAIR( string_to_bstring( sname ),
			   string_to_bstring( cname ) );
      }
      
      entry = MAKE_PAIR( pair, entry );
      entry = MAKE_PAIR( string_to_bstring( fname ), entry );
	 
      table_entry++;
      glo_info = MAKE_PAIR( entry, glo_info );
   }

   return MAKE_PAIR( mod_info, glo_info );
}
