;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bde/mco/mco.scm                      */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sun May 31 07:37:29 1998                          */
;*    Last change :  Wed Jun 30 08:49:18 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Bigloo module checksum generator.                            */
;*    -------------------------------------------------------------    */
;*    We have implemented that application in order to get rid of      */
;*    Bigloo to produce module checksum. Bigloo has a startup time     */
;*    that is too long and that make it not suitable to compute        */
;*    efficiently modules checksum.                                    */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module mco
   (include "include/args.sch")
   (main    main))

;*---------------------------------------------------------------------*/
;*    Global parameters                                                */
;*---------------------------------------------------------------------*/
(define *mco-version*   "0.0")
(define *mco-files*     '())
(define *mco-verbose*   #t)
(define *mco-output*    #f)

;*---------------------------------------------------------------------*/
;*    main ...                                                         */
;*---------------------------------------------------------------------*/
(define (main argv)
   ;; we parse command line arguments
   (parse-args argv)
   ;; we now start prof.
   (mco))

;*---------------------------------------------------------------------*/
;*    parse-args ...                                                   */
;*---------------------------------------------------------------------*/
(define (parse-args cmd-args)
   (define (usage args-parse-usage level)
      (print "usage: mco -o output files...")
      (newline)
      (args-parse-usage)
      (newline)
      (exit 0))
   (args-parse (cdr cmd-args)
      (("?")
       (usage args-parse-usage 1))
      (("-help" (synopsis "This help message"))
       (usage args-parse-usage 1))
      (("-v" (synopsis "Verbose mode"))
       (set! *mco-verbose* #t))
      (("-s" (synopsis "Silent mode"))
       (set! *mco-verbose* #f))
      (("-o" ?file (synopsis "mco file name"))
       (set! *mco-output* file))
      (else
       (set! *mco-files* (cons else *mco-files*))))
   ;; we clean up the configuration setting
   (if (not (pair? *mco-files*))
       (error "mco" "No source file specified" #f))
   (set! *mco-files* (reverse! *mco-files*))
   ;; if not output file is specified, we use the basename of the
   ;; first source file
   (if (not (string? *mco-output*))
       (set! *mco-output* (string-append (prefix (car *mco-files*)) ".mco"))))

;*---------------------------------------------------------------------*/
;*    mco ...                                                          */
;*    -------------------------------------------------------------    */
;*    We open all the input file in order to find the module clause    */
;*---------------------------------------------------------------------*/
(define (mco)
   (if *mco-verbose*
       (hello-world *mco-files*))
   (let ((mod (find-module-clause *mco-files*)))
      (if (not mod)
	  (error "mco" "Can't find module clause" *mco-files*)
	  (let ((checksum (module-checksum mod)))
	     (if (not (mco-checksum=? checksum *mco-output*))
		 (emit-checksum checksum *mco-output*))))))

;*---------------------------------------------------------------------*/
;*    hello-world ...                                                  */
;*    -------------------------------------------------------------    */
;*    This function is called in verbose mode to display the files     */
;*    that are processed by this mco session.                          */
;*---------------------------------------------------------------------*/
(define (hello-world files)
   (if (pair? files)
       (let loop ((files files))
	  (cond
	     ((null? (cdr files))
	      (print (car files) ":"))
	     (else
	      (display (car files))
	      (display " ")
	      (loop (cdr files)))))))
   
;*---------------------------------------------------------------------*/
;*    mco-checksum=? ...                                               */
;*    -------------------------------------------------------------    */
;*    Compares if the computed checksum is the same that the one       */
;*    currently contained in the mco file.                             */
;*---------------------------------------------------------------------*/
(define (mco-checksum=? checksum file)
   (if (not (file-exists? file))
       #f
       (let* ((port (open-input-file file))
	      (num  (unwind-protect (read port)
				    (close-input-port port))))
	  (eq? num checksum))))

;*---------------------------------------------------------------------*/
;*    emit-checksum ...                                                */
;*---------------------------------------------------------------------*/
(define (emit-checksum checksum file)
   (if (file-exists? file)
       (delete-file file))
   (let ((port (open-output-file file)))
      (fprint port checksum)
      (close-output-port port)))

;*---------------------------------------------------------------------*/
;*    find-module-clause ...                                           */
;*    -------------------------------------------------------------    */
;*    Open all the SRC files in order to find the module clause.       */
;*---------------------------------------------------------------------*/
(define (find-module-clause src)
   (define module?
      (match-lambda
	 ((module (? symbol?) . ?-)
	  #t)
	 (else
	  #f)))
   (let loop ((src src))
      (if (null? src)
	  #f
	  (if (file-exists? (car src))
	      (let* ((port (open-input-file (car src)))
		     (mod  (unwind-protect (let ((exp (read port)))
					      (if (module? exp)
						  exp
						  #f))
					   (close-input-port port))))
		 (if mod
		     mod
		     (loop (cdr src))))))))

;*---------------------------------------------------------------------*/
;*    module-checksum ...                                              */
;*    -------------------------------------------------------------    */
;*    Compute a checksum number for a module. The checksum is          */
;*    computed strictly using the exportations, includes and extern    */
;*    clauses.                                                         */
;*---------------------------------------------------------------------*/
(define (module-checksum module)
   (clause-checksum (symbol->number (cadr module)) (cddr module)))

;*---------------------------------------------------------------------*/
;*    include-checksum ...                                             */
;*---------------------------------------------------------------------*/
(define (include-checksum include checksum)
   (define directives?
      (match-lambda
	 ((directives . ?-)
	  #t)
	 (else
	  #f)))
   (if (file-exists? include)
       (let* ((port (open-input-file include))
	      (dir  (unwind-protect (let ((exp (read port)))
				       (if (directives? exp)
					   exp
					   #f))
				    (close-input-port port))))
	  (if dir
	      (clause-checksum (atom->number checksum include) (cdr dir))
	      checksum))
       checksum))
	      
;*---------------------------------------------------------------------*/
;*    clause-checksum ...                                              */
;*---------------------------------------------------------------------*/
(define (clause-checksum checksum clauses)
   (let loop ((clauses  clauses)
	      (checksum checksum))
      (match-case clauses
	 (()
	  checksum)
	 ((((or extern foreign export) . ?export) . ?rest)
	  (loop rest
		(list->number checksum export)))
	 (((include . ?files) . ?rest)
	  (let laap ((files    files)
		     (checksum checksum))
	     (if (null? files)
		 (loop rest
		       checksum)
		 (laap (cdr files)
		       (include-checksum (car files) checksum)))))
	 (else
	  (loop (cdr clauses)
		checksum)))))

;*---------------------------------------------------------------------*/
;*    symbol->number ...                                               */
;*---------------------------------------------------------------------*/
(define (symbol->number symbol)
   (string->0..2^x-1 (symbol->string symbol) 16))

;*---------------------------------------------------------------------*/
;*    atom->number ...                                                 */
;*---------------------------------------------------------------------*/
(define (atom->number checksum clause)
   (cond
      ((fixnum? clause)
       (bit-xor checksum clause))
      ((flonum? clause)
       (atom->number checksum (real->string clause)))
      ((char? clause)
       (bit-xor checksum (char->integer clause)))
      ((cnst? clause)
       (bit-xor checksum (cnst->integer clause)))
      ((string? clause)
       (bit-xor checksum (string->0..2^x-1 clause 16)))
      ((symbol? clause)
       (bit-xor checksum (symbol->number clause)))
      ((pair? clause)
       (list->number checksum clause))))

;*---------------------------------------------------------------------*/
;*    list->number ...                                                 */
;*---------------------------------------------------------------------*/
(define (list->number checksum clause)
   (cond
      ((null? clause)
       checksum)
      ((pair? clause)
       (list->number (atom->number checksum (car clause))
		     (cdr clause)))
      (else
       (atom->number checksum clause))))
	 
   
