;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/bmacs/cee/cee-config.el              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon May 25 07:28:09 1998                          */
;*    Last change :  Tue Aug 24 07:41:32 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The Cee configuration file.                                      */
;*    -------------------------------------------------------------    */
;*    Bdb configuration is set into cee-bdb.el                         */
;*=====================================================================*/
 
;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(provide 'cee-config)
(require 'ude-custom)
(require 'dbg-config)

;; cee version
(defconst cee-version "0.2"
  "*The Cee version.")

;; cee group
(defgroup cee nil
  "Bigloo Emacs Environment."
  :tag "Cee"
  :prefix "cee-"
  :group 'processes)

;; online documentation info pages
(defcustom cee-info-file-list '("libc.info" "ccp.info" "readline.info")
  "*The info docline pages."
  :group 'cee
  :type '(repeat (string)))

;; indentation
(defcustom cee-external-indent "indent"
  "*The external Cee indent program."
  :group 'cee
  :type 'string)

(defcustom cee-external-indent-opt "-st"
  "*The external Cee indent program options."
  :group 'cee
  :type 'string)

;; tags
(defcustom cee-make-tags (concat ude-make " " ude-tagsfile-name)
  "*The name of the command to build the tagsfile."
  :group 'cee
  :type 'string)

;; user menu
(defcustom cee-user-menu nil
  "*Non-nil means a popup menu user part."
  :group 'cee
  :type 'boolean)

;; tmp dir (for macro expansion)
(defcustom cee-tmp-dir (let ((dir (getenv "TMPDIR")))
		      (if (stringp dir)
			  dir
			"/tmp"))
  "*The tmp directory."
  :group 'cee
  :type 'string)

;; cee profiling face
(defcustom cee-profile-face 'font-lock-function-name-face
  "The face to display Bigloo identifiers on profile."
  :group 'cee
  :type 'symbol)

;; a name of a binary file that implements a profiler
(defcustom cee-profiler (file-installed-p "xprof" exec-path)
  "*The name of a visual profiler."
  :group 'cee
  :type '(choice (boolean) (string)))

;; the option to the profiler when called from the cee
(defcustom cee-profiler-cee-options '("-no-event-loop" "--emacs")
  "*The profiler cee options"
  :group 'cee
  :type '(repeat (string)))

;; a name of a binary file that implements the Bigloo debugger
(defcustom cee-kbdb (file-installed-p "kbdb" exec-path)
  "*The name of the Bigloo debugger."
  :group 'cee
  :type '(choice (boolean) (string)))

;; the options send to kbdb when called from the cee
(defcustom cee-kbdb-cee-options '("-no-event-loop"
				  "--bee"
				  "--no-heap-explorer"
				  "--no-demangling")
  "*The kbdb cee options"
  :group 'cee
  :type '(repeat (string)))

;*---------------------------------------------------------------------*/
;*    cee-customize ...                                                */
;*    -------------------------------------------------------------    */
;*    This function invoked CEE customization in a new frame.          */
;*---------------------------------------------------------------------*/
(defun cee-customize ()
  "Invokes Ude customization in a new frame."
  (interactive)
  (if (ude-empty-window-p)
      (customize-group 'cee)
    (let ((pop-up-frames t))
      (customize-group-other-window 'cee))))






