;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/comptime/Cfa/cinfo.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Jun 24 15:46:49 1996                          */
;*    Last change :  Thu Feb 25 13:51:52 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The cfa's information structures                                 */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module cfa_info
   
   (import type_type
	   type_cache
	   ast_var
	   ast_node)
   
   (export ;; the approximations
    (class approx
       ;; the type of the approximation. Just one type, we
       ;; do not compute several types approximation because
       ;; anything receiving more than one type is of type obj.
       (type::type (default *_*))
       ;; A type locked means that the type field can't be changed
       ;; (e.g. because the type has been set by the user, as for
       ;; variable).
       (type-locked?::bool read-only (default #f))
       ;; its allocations
       (allocs read-only)
       ;; or top
       (top?::bool (default #f))
       ;; a stamp to avoid useless multiple loose.
       (lost-stamp::long (default -1)))
    
    ;; function extensions
    (wide-class cfun/Cinfo::cfun (approx::approx read-only))
    (wide-class extern-sfun/Cinfo::sfun (approx::approx read-only))
    (wide-class intern-sfun/Cinfo::sfun
       (approx::approx read-only)
       (stamp::long (default -1)))
    
    ;; cnst extension
    (wide-class scnst/Cinfo::scnst (approx::approx read-only))
    
    ;; var extensions
    (wide-class pre-clo-env::svar)
    (wide-class svar/Cinfo::svar
       (approx::approx read-only)
       ;; is this variable holding a closure environement
       ;; (if it is it won't be lost when loosing the
       ;; function whose's this variable owner).
       (clo-env?::bool (default #f)))
    (wide-class cvar/Cinfo::cvar (approx::approx read-only))
    
    ;; exit extensions
    (wide-class sexit/Cinfo::sexit (approx::approx read-only))
    
    ;; global and local reshaping
    (wide-class reshaped-local::local
       (binding-value read-only (default #f)))
    (wide-class reshaped-global::global)
    
    ;; node extension
    (wide-class atom/Cinfo::atom (approx::approx read-only))
    (wide-class kwote/node::kwote (node::node read-only))
    (wide-class kwote/Cinfo::kwote (approx::approx read-only))
    (wide-class app-ly/Cinfo::app-ly (approx::approx read-only))
    (wide-class funcall/Cinfo::funcall
       (approx::approx read-only)
       (va-approx::approx read-only)
       (arity-error-noticed?::bool (default (not *warning*)))
       (type-error-noticed?::bool (default (not *warning*))))
    (wide-class pragma/Cinfo::pragma (approx::approx read-only))
    (wide-class setq/Cinfo::setq (approx::approx read-only))
    (wide-class conditional/Cinfo::conditional (approx::approx read-only))
    (wide-class fail/Cinfo::fail (approx::approx read-only))
    (wide-class select/Cinfo::select (approx::approx read-only))
    (wide-class set-ex-it/Cinfo::set-ex-it (approx::approx read-only))
    (wide-class jump-ex-it/Cinfo::jump-ex-it (approx::approx read-only))
    
    ;; boxes
    (wide-class pre-make-box::make-box)
    (wide-class make-box/Cinfo::make-box (approx::approx read-only))
    (wide-class make-box/O-Cinfo::make-box
       approx::approx
       (value-approx::approx read-only))
    (wide-class box-set!/Cinfo::box-set! (approx::approx read-only))
    (wide-class box-ref/Cinfo::box-ref (approx::approx read-only))
    (wide-class box-set!/O-Cinfo::box-set! (approx::approx read-only))
    (wide-class box-ref/O-Cinfo::box-ref (approx::approx read-only))))
   
