;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/comptime/Expand/srfi-0.scm           */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Feb 24 15:25:03 1999                          */
;*    Last change :  Fri Aug  6 12:33:42 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The expander for srfi form.                                      */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module expand_srfi-0
   (import engine_param
	   tools_misc)
   (export (expand-cond-expand x e)
	   (add-supported-srfi! srfi::symbol)))

;*---------------------------------------------------------------------*/
;*    *srfi-list* ...                                                  */
;*    -------------------------------------------------------------    */
;*    The list of supported srfi                                       */
;*    -------------------------------------------------------------    */
;*    The four initial supported srfi are:                             */
;*       - srfi-0                                                      */
;*       - bigloo                                                      */
;*       - bigloo<major-num>                                           */
;*       - bigloo<major-num><minor-num>                                */
;*    -------------------------------------------------------------    */
;*    When a library is used for compiling the name of that library    */
;*    is added to the supported srfis.                                 */
;*---------------------------------------------------------------------*/
(define *srfi-list*
   (let* ((bgl-major (string-append "BIGLOO" *bigloo-version*))
	  (common (list (string->symbol bgl-major)
			'bigloo
			'srfi-0)))
      (if (char? *bigloo-level*)
	  (cons (string->symbol (string-append bgl-major
					       (string
						(char-upcase *bigloo-level*))))
		common)
	  common)))

;*---------------------------------------------------------------------*/
;*    add-supported-srfi! ...                                          */
;*---------------------------------------------------------------------*/
(define (add-supported-srfi! srfi::symbol)
   (set! *srfi-list* (cons srfi *srfi-list*)))

;*---------------------------------------------------------------------*/
;*    expand-cond-expand ...                                           */
;*---------------------------------------------------------------------*/
(define (expand-cond-expand x e)
   (match-case x
      ((cond-expand)
       (error "cond-expand" "Illegal form" x))
      ((?- ((kwote else) . ?body))
       (e (epairify `(begin ,@body) x) e))
      ((?- (((kwote and)) . ?body) . ?-)
       (e (epairify `(begin ,@body) x) e))
      ((?- (((kwote and) ?req1) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 ,@body)
		      ,@else)
		    x)
	  e))
      ((?- (((kwote and) ?req1 ?req2 . ?reqs) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 (cond-expand
			      ((and ,req2 ,@reqs) ,@body)
			      ,@else)))
		    x)
	  e))
      ((?- (((kwote or)) . ?body) . ?else)
       (e (epairify `(cond-expand ,@else) x) e))
      ((?- (((kwote or) ?req1) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 ,@body)
		      ,@else)
		    x)
	  e))
      ((?- (((kwote or) ?req1 ?req2 . ?reqs) . ?body) . ?else)
       (let ((bd (gensym 'body)))
	  (e (epairify `(let ((,bd ,(epairify `(begin ,@body) body)))
			   (cond-expand
			    (,req1 ,bd)
			    (else
			     (cond-expand
			      ((or ,req2 ,@reqs)
			       ,bd)
			      ,@else))))
		       x)
	     e)))
      ((?- (((kwote not) ?req) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req (cond-expand ,@else))
		      (else ,@body))
		    x)
	  e))
      ((?- ((and (? symbol?) ?feature) . ?body) . ?else)
       (e (epairify (if (memq feature *srfi-list*)
			`(begin ,@body)
			`(cond-expand ,@else))
		    x)
	  e))
      (else
       (error "cond-expand" "Illegal form" x))))
   
