;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/comptime/Module/alibrary.scm         */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Sat Feb 28 10:20:55 1998                          */
;*    Last change :  Wed Jul 14 07:43:41 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The compiler library clause compilation                          */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module module_alibrary
   (include "Ast/unit.sch")
   (import  module_module
	    tools_error
	    engine_param
	    expand_srfi-0
	    (setup-library-values init_setrc))
   (export  (make-alibrary-compiler)
	    (make-library-name ::obj)
	    (use-library! library mode)))

;*---------------------------------------------------------------------*/
;*    use-library! ...                                                 */
;*---------------------------------------------------------------------*/
(define (use-library! library mode)
   (let ((lib-name (make-library-name library)))
      (if (not (member lib-name *additional-bigloo-libraries*))
	  (begin
	     (set! *additional-bigloo-libraries*
		   (cons lib-name *additional-bigloo-libraries*))
	     (let ((heap-name (string-append lib-name ".heap")))
		(add-supported-srfi! (string->symbol (string-upcase lib-name)))
		;; when use-library is called from the argument parsing we have
		;; to delay the library initialization until all arguments have
		;; been parsed. In consequence setup-library-values is called
		;; only when use-library is not called from argument parsing
		;; (which is denotes by the 'now mode value)
		(if (eq? mode 'now) (setup-library-values lib-name))
		(set! *additional-heap-names*
		      (cons heap-name *additional-heap-names*))
		lib-name)))))

;*---------------------------------------------------------------------*/
;*    make-alibrary-compiler ...                                       */
;*---------------------------------------------------------------------*/
(define (make-alibrary-compiler)
   (instantiate::ccomp
      (id 'library)
      (producer alibrary-producer)))

;*---------------------------------------------------------------------*/
;*    alibrary-producer ...                                            */
;*---------------------------------------------------------------------*/
(define (alibrary-producer clause)
   (match-case clause
      ((?- . ?protos)
       (for-each (lambda (x) (use-library! x 'now)) protos)
       '())
      (else
       (user-error "Parse error" "Illegal `library' clause" clause '()))))

;*---------------------------------------------------------------------*/
;*    make-library-name ...                                            */
;*---------------------------------------------------------------------*/
(define (make-library-name libname)
   (cond
      ((symbol? libname)
       (string-downcase (symbol->string libname)))
      ((string? libname)
       (let ((new-name (string-downcase libname)))
	  (if (not (string=? new-name libname))
	      (user-error "Parse error"
			  "Library name must be downcase"
			  libname '())
	      new-name)))
      (else
       (user-error "Parse error" "Illegal library" libname '()))))
       
