;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/comptime/Object/slots.scm            */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Jun 18 12:48:07 1996                          */
;*    Last change :  Fri May 28 21:33:51 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    We build the class slots                                         */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module object_slots
   (include "Object/class.sch"
	    "Tools/trace.sch")
   (import  tools_error
	    type_type
	    type_cache
	    ast_var
	    ast_ident
	    object_class
	    engine_param
	    (find-location tools_location))
   (export  (make-class-slots ::tclass ::obj ::obj ::int ::obj)
	    (get-local-virtual-slots-number ::tclass slots)))

;*---------------------------------------------------------------------*/
;*    ensure-type-defined! ...                                         */
;*---------------------------------------------------------------------*/
(define (ensure-type-defined! type::type src)
   (if (not (type-init? type))
       (user-error "Can't find type definition" (type-id type) src)))

;*---------------------------------------------------------------------*/
;*    make-class-slots ...                                             */
;*---------------------------------------------------------------------*/
(define (make-class-slots class slots super vnum src)
   (trace (ast 2) "make-class-slots: " slots " " vnum #\Newline)
   (define (find-default-attr attr)
      (if (not (pair? attr))
	  '(#f . #unspecified)
	  (match-case (car attr)
	     ((default ?value)
	      (cons #t value))
	     (else
	      (find-default-attr (cdr attr))))))
   (define (find-assert-attr attr)
      (if (not (pair? attr))
	  #f
	  (match-case (car attr)
	     ((assert ((? symbol?)) ?value)
	      (set-car! (car attr) 'lambda)
	      (car attr))
	     (else
	      (find-assert-attr (cdr attr))))))
   (define (find-virtual-attr attr)
      (let loop ((attr attr)
		 (get  #f)
		 (set  #f))
	 (if (not (pair? attr))
	     (cons get set)
	     (match-case (car attr)
		((get ?get)
		 (loop (cdr attr) get set))
		((set ?set)
		 (loop (cdr attr) get set))
		(else
		 (loop (cdr attr) get set))))))
   (define (find-set-attr attr)
      (if (not (pair? attr))
	  '(#f . #unspecified)
	  (match-case (car attr)
	     ((set ?value)
	      (cons #t value))
	     (else
	      (find-default-attr (cdr attr))))))
   (define (slot-member? slot slot-list)
      ;; this function returns true if there is a slot with the same name
      ;; in the list
      (let ((id (slot-id slot)))
	 (let loop ((slot-list slot-list))
	    (cond
	       ((null? slot-list)
		#f)
	       ((eq? (slot-id (car slot-list)) id)
		(car slot-list))
	       (else
		(loop (cdr slot-list)))))))
   (define (unique-virtual-slots slots)
      ;; this function removes overriden duplicated virtual slots
      (let loop ((slots slots)
		 (res   '()))
	 (cond
	    ((null? slots)
	     res)
	    ((slot-virtual? (car slots))
	     (let ((other (slot-member? (car slots) res)))
		(if (slot? other)
		    (let ((loc (find-location src))
			  (id  (slot-id (car slots))))
		       (if (eq? (slot-class-owner (car slots))
				(slot-class-owner other))
			   (user-error/location
			    loc
			    "Class declarator"
			    "Illegal duplicated virtual slot"
			    id)
			   (let ((new-num (slot-virtual-num (car slots))))
			      ;; we overriding a virtual slot we have to
			      ;; ajust the slot numbering
			      (slot-virtual-num-set! other new-num)
			      ;; ...and we emit a warning
			      (if (and (number? *warning*) (>= *warning* 2))
				  (user-warning/location
				   loc
				   "Class declarator"
				   "Overriden virtual slot"
				   id))
			      (loop (cdr slots) res))))
		    (loop (cdr slots) (cons (car slots) res)))))
	    (else
	     (loop (cdr slots) (cons (car slots) res))))))
   (let loop ((slots slots)
	      (res   (cond
			((not (type? super))
			 '())
			((not (tclass? super))
			 '())
			(else
			 ;; take care to the double reverse. Here we can't
			 ;; do in in-place reverse otherwise will be destroying
			 ;; super classes fields. This reverse allocates
			 ;; a fresh slots list.
			 (reverse (tclass-slots super)))))
	      (vnum  vnum))
      (if (null? slots)
	  ;; this is mandatory that unique-virtual-slots is called
	  ;; on a reversed list. This function fixs up virtual slots
	  ;; number. If the list is the wrong direction, numbers will get
	  ;; mixed up.
	  (unique-virtual-slots res)
	  (let ((s (car slots)))
	     (match-case s
		((* (id ?slot-id) . ?attr)
		 (let ((id      (symbol-append (car slot-id) '-len))
		       (default (find-default-attr attr))
		       (a-exp   (find-assert-attr attr))
		       (virtual (find-virtual-attr attr))
		       (type    (if (eq? (cdr slot-id) *_*)
				    *obj*
				    (cdr slot-id))))
		    (ensure-type-defined! type src)
		    (if (or (car virtual) (cdr virtual))
			(user-error "Parse error"
				    "virtual slot can't be indexed"
				    s
				    '()))
		    (loop (cdr slots)
			  (cons (slot (car slot-id)
				      (scheme-symbol->c-string (car slot-id))
				      type
				      #f
				      #unspecified
				      #t
				      #f
				      (memq 'read-only attr)
				      (car default)
				      (cdr default)
				      a-exp
				      #f
				      0
				      #f
				      #f
				      s
				      class)
				(cons 
				 (slot id
				       (scheme-symbol->c-string id)
				       *long*
				       #f
				       #unspecified
				       #f
				       #t
				       #t
				       #f
				       #unspecified
				       #f
				       #f
				       0
				       #f
				       #f
				       s
				       class)
				 res))
			  vnum)))
		((+ ?len (id ?slot-id) . ?attr)
		 (let ((id      (symbol-append (car slot-id) '-len))
		       (default (find-default-attr attr))
		       (a-exp   (find-assert-attr attr))
		       (virtual (find-virtual-attr attr))
		       (type    (if (eq? (cdr slot-id) *_*)
				    *obj*
				    (cdr slot-id))))
		    (ensure-type-defined! type src)
		    (if (or (car virtual) (cdr virtual))
			(user-error "Parse error"
				    "virtual slot can't be indexed"
				    s
				    '()))
		    (loop (cdr slots)
			  (cons (slot (car slot-id)
				      (scheme-symbol->c-string (car slot-id))
				      type
				      #t
				      len
				      #f
				      #f
				      (memq 'read-only attr)
				      (car default)
				      (cdr default)
				      a-exp
				      #f
				      0
				      #f
				      #f
				      s
				      class)
				res)
			  vnum)))
		(((id ?slot-id) . ?attr)
		 (let* ((default  (find-default-attr attr))
			(a-exp    (find-assert-attr attr))
			(virtual  (find-virtual-attr attr))
			(virtual? (or (car virtual) (cdr virtual)))
			(reado?   (memq 'read-only attr))
			(type     (if (eq? (cdr slot-id) *_*)
				      *obj*
				      (cdr slot-id))))
		    (cond
		       ((and (cdr virtual) (not (car virtual)))
			(user-error "Parse error"
				    "illegal virtual slot (missing getter)"
				    s
				    '()))
		       ((and (car virtual) (cdr virtual) reado?)
			(user-error "Parse error"
				    "illegal virtual slot (read-only)"
				    s
				    '()))
		       ((and (car virtual) (not (cdr virtual)) (not reado?))
			(user-error "Parse error"
				    "illegal virtual slot (missing setter)"
				    s
				    '())))
		    (ensure-type-defined! type src)
		    (loop (cdr slots)
			  (cons (slot (car slot-id)
				      (scheme-symbol->c-string (car slot-id))
				      type
				      #f
				      #unspecified
				      #f
				      #f
				      reado?
				      (car default)
				      (cdr default)
				      a-exp
				      virtual?
				      (if virtual? vnum 0)
				      (car virtual)
				      (cdr virtual)
				      s
				      class)
				res)
			  (if virtual? (+ vnum 1) vnum))))
		((id ?slot-id)
		 (let ((type (if (eq? (cdr slot-id) *_*)
				 *obj*
				 (cdr slot-id))))
		    (ensure-type-defined! type src)
		    (loop (cdr slots)
			  (cons (slot (car slot-id)
				      (scheme-symbol->c-string (car slot-id))
				      type
				      #f
				      #unspecified
				      #f
				      #f
				      #f
				      #f
				      #unspecified
				      #f
				      #f
				      0
				      #f
				      #f
				      s
				      class)
				res)
			  vnum)))
		(else
		 (user-error "Parse error"
			     "Unknown class slot type"
			     s
			     '())))))))

;*---------------------------------------------------------------------*/
;*    scheme-symbol->c-string ...                                      */
;*---------------------------------------------------------------------*/
(define (scheme-symbol->c-string symbol)
   (if *case-sensitive*
       (id->name symbol)
       (string-downcase (id->name symbol))))

;*---------------------------------------------------------------------*/
;*    get-local-virtual-slots-number ...                               */
;*    -------------------------------------------------------------    */
;*    This function takes a list of slots and a class and it computes  */
;*    how many virtual slots are created by this class.                */
;*    -------------------------------------------------------------    */
;*    The number of virtual slots is just the maximun virtual number   */
;*    found (+ 1). It is not the number of virtual slots because of    */
;*    inheritance mecanism. That is if a class override a virtual slot */
;*    definition. That slot does not count as a new entire virtual     */
;*    slot.                                                            */
;*---------------------------------------------------------------------*/
(define (get-local-virtual-slots-number class::tclass slots)
   (let loop ((slots slots)
	      (num   -1))
      (cond
	 ((null? slots)
	  (+fx num 1))
	 ((slot-virtual? (car slots))
	  (let ((lnum (slot-virtual-num (car slots))))
	     (loop (cdr slots)
		   (if (>fx lnum num)
		       lnum
		       num))))
	 (else
	  (loop (cdr slots) num)))))
