/*---------------------------------------------------------------------*/
/*    A practical implementation for the Scheme programming language   */
/*                                                                     */
/*                                    ,--^,                            */
/*                              _ ___/ /|/                             */
/*                          ,;'( )__, ) '                              */
/*                         ;;  //   L__.                               */
/*                         '   \\   /  '                               */
/*                              ^   ^                                  */
/*                                                                     */
/*               Copyright (c) 1992-1999 Manuel Serrano                */
/*                                                                     */
/*     Bug descriptions, use reports, comments or suggestions are      */
/*     welcome. Send them to                                           */
/*       bigloo@kaolin.unice.fr                                        */
/*       http://kaolin.unice.fr/bigloo                                 */
/*                                                                     */
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bigloo/runtime/Clib/custom.c                */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Mon Aug 16 16:56:24 1999                          */
/*    Last change :  Wed Aug 18 07:31:32 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The custom management.                                           */
/*=====================================================================*/
#include <bigloo2.1a.h>

/*---------------------------------------------------------------------*/
/*    static int                                                       */
/*    custom_default_equal ...                                         */
/*---------------------------------------------------------------------*/
static int
custom_default_equal( obj_t obj1, obj_t obj2 ) {
   return obj1 == obj2;
}
   
/*---------------------------------------------------------------------*/
/*    static long                                                      */
/*    custom_default_hash ...                                          */
/*---------------------------------------------------------------------*/
static long
custom_default_hash( obj_t obj ) {
   return (long)obj;
}
   
/*---------------------------------------------------------------------*/
/*    static char *                                                    */
/*    custom_default_to_string ...                                     */
/*---------------------------------------------------------------------*/
static char *
custom_default_to_string( obj_t obj, char *buffer, int len ) {
   if( len > 16 ) {
      sprintf( buffer, "<custom:%p>", obj );
      return buffer;
   } else {
      return "<custom>";
   }
}
   
/*---------------------------------------------------------------------*/
/*    static obj_t                                                     */
/*    custom_default_output ...                                        */
/*---------------------------------------------------------------------*/
static obj_t
custom_default_output( obj_t obj, FILE *file ) {
   fprintf( file, "<custom:%p>", obj );
   return obj;
}
   
/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    create_custom ...                                                */
/*    -------------------------------------------------------------    */
/*    LEN is number of additional bytes to be appended to the standard */
/*    custom structure.                                                */
/*---------------------------------------------------------------------*/
obj_t
create_custom( long len ) {
   obj_t custom;

   custom = GC_MALLOC_ATOMIC( CUSTOM_SIZE + len );

   custom->custom_t.header     = MAKE_HEADER( CUSTOM_TYPE, 0 );
   custom->custom_t.final      = 0L;
   custom->custom_t.identifier = 0L;
   custom->custom_t.equal      = custom_default_equal;
   custom->custom_t.hash       = custom_default_hash;
   custom->custom_t.to_string  = custom_default_to_string;
   custom->custom_t.output     = custom_default_output;
   
   return BREF( custom );
}

   
