/*---------------------------------------------------------------------*/
/*    A practical implementation for the Scheme programming language   */
/*                                                                     */
/*                                    ,--^,                            */
/*                              _ ___/ /|/                             */
/*                          ,;'( )__, ) '                              */
/*                         ;;  //   L__.                               */
/*                         '   \\   /  '                               */
/*                              ^   ^                                  */
/*                                                                     */
/*               Copyright (c) 1992-1999 Manuel Serrano                */
/*                                                                     */
/*     Bug descriptions, use reports, comments or suggestions are      */
/*     welcome. Send them to                                           */
/*       bigloo@kaolin.unice.fr                                        */
/*       http://kaolin.unice.fr/bigloo                                 */
/*                                                                     */
/*   This program is free software; you can redistribute it            */
/*   and/or modify it under the terms of the GNU General Public        */
/*   License as published by the Free Software Foundation; either      */
/*   version 2 of the License, or (at your option) any later version.  */
/*                                                                     */
/*   This program is distributed in the hope that it will be useful,   */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
/*   GNU General Public License for more details.                      */
/*                                                                     */
/*   You should have received a copy of the GNU General Public         */
/*   License along with this program; if not, write to the Free        */
/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
/*   MA 02111-1307, USA.                                               */
/*---------------------------------------------------------------------*/
/*=====================================================================*/
/*    serrano/prgm/project/bigloo/runtime/Clib/gc_profile.c            */
/*    -------------------------------------------------------------    */
/*    Author      :  Manuel Serrano                                    */
/*    Creation    :  Fri May 14 06:49:17 1999                          */
/*    Last change :  Thu Oct 14 14:49:56 1999 (serrano)                */
/*    -------------------------------------------------------------    */
/*    The GC profiler. This file is not specific to the Boehm's        */
/*    collector.                                                       */
/*=====================================================================*/
#include <bigloo2.1a.h>

/*---------------------------------------------------------------------*/
/*    MEGABYTE                                                         */
/*---------------------------------------------------------------------*/
#define MEGABYTE( val ) ((double)(val) / (1024.0 * 1024.0))

/*---------------------------------------------------------------------*/
/*    ppair_t ...                                                      */
/*    -------------------------------------------------------------    */
/*    This type is only used for profile stamp recording.              */
/*---------------------------------------------------------------------*/
typedef struct ppair {
   char *ident;
   struct ppair *next;
} *ppair_t;
  
/*---------------------------------------------------------------------*/
/*    obj_t                                                            */
/*    GC_profile_stamp ...                                             */
/*    -------------------------------------------------------------    */
/*    The stack of profile stamps.                                     */
/*---------------------------------------------------------------------*/
static ppair_t GC_profile_stamp = 0;
static ppair_t GC_profile_all_stamp = 0;

/*---------------------------------------------------------------------*/
/*    static char *                                                    */
/*    strdup ...                                                       */
/*---------------------------------------------------------------------*/
static char *strdup( char *old ) {
   int   len = strlen( old );
   char *new = malloc( len + 1 );

   strcpy( new, old );

   return new;
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    GC_profile_push ...                                              */
/*    -------------------------------------------------------------    */
/*    The dummy argument is used to break the Bigloo optimization.     */
/*    It is mandatory that the function introduced inside the          */
/*    profile form is really compiled into a C function. To get        */
/*    that, the function is passed as an argument in                   */
/*    GC_profile_push but not unsed here.                              */
/*    -------------------------------------------------------------    */
/*    When entering a profile, we force a GC.                          */
/*---------------------------------------------------------------------*/
long
GC_profile_push( char *ident, obj_t dummy ) {
   ppair_t new;

   new = malloc( sizeof( struct ppair ) );

   new->ident = strdup( ident );
   new->next  = GC_profile_stamp;

   GC_profile_all_stamp = GC_profile_stamp = new;
   
   return 0;
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    GC_collect_profile_push ...                                      */
/*    -------------------------------------------------------------    */
/*    The same function as GC_profile_push but force a collection.     */
/*---------------------------------------------------------------------*/
long
GC_collect_profile_push( char *ident, obj_t dummy ) {
   GC_profile_push( ident, dummy );
   
   GC_COLLECT();
   
   return 0;
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    GC_profile_pop ...                                               */
/*---------------------------------------------------------------------*/
long
GC_profile_pop() {
   if( GC_profile_stamp ) GC_profile_stamp = GC_profile_stamp->next;

   return 0;
}

/*---------------------------------------------------------------------*/
/*    gc_profile_info_t                                                */
/*---------------------------------------------------------------------*/
struct gc_profile_info_t {
   long          num;
   long          heap;
   long          alloc;
   unsigned long stack;
   unsigned long live;
   ppair_t       stamp;
};

/*---------------------------------------------------------------------*/
/*    get_top_of_stack                                                 */
/*    -------------------------------------------------------------    */
/*    This function is defined in the Call/cc implementation.          */
/*---------------------------------------------------------------------*/
extern unsigned long get_top_of_stack();

/*---------------------------------------------------------------------*/
/*    GC_PROFILE_VECTOR_INCREASE                                       */
/*    -------------------------------------------------------------    */
/*    How many gc_profile_info_t struct shall we add on each           */
/*    increase (when the vector is full).                              */
/*---------------------------------------------------------------------*/
#define GC_PROFILE_VECTOR_INCREASE 100

/*---------------------------------------------------------------------*/
/*    struct gc_profile_info_t *                                       */
/*    gc_profile_vector ...                                            */
/*---------------------------------------------------------------------*/
static struct gc_profile_info_t *gc_profile_vector = 0L;
static long   gc_profile_len = 0L;
static long   gc_profile_num;

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    gc_profile_realloc_vector ...                                    */
/*---------------------------------------------------------------------*/
static void
gc_profile_realloc_vector() {
   gc_profile_len += GC_PROFILE_VECTOR_INCREASE;

   gc_profile_vector = (struct gc_profile_info_t *)
      realloc( gc_profile_vector,
	       gc_profile_len * sizeof( struct gc_profile_info_t ) );
}

/*---------------------------------------------------------------------*/
/*    unsigned long                                                    */
/*    gc_profile_stack_base ...                                        */
/*---------------------------------------------------------------------*/
static unsigned long gc_profile_stack_base = 0;

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    GC_profile_start ...                                             */
/*---------------------------------------------------------------------*/
void GC_profile_start() {
   gc_profile_len = 0;
   gc_profile_num = 0;
   gc_profile_stack_base = get_top_of_stack();
}

/*---------------------------------------------------------------------*/
/*    unsigned long                                                    */
/*    GC_profile_stack_size ...                                        */
/*---------------------------------------------------------------------*/
unsigned long GC_profile_stack_size( unsigned long stack_top ) {
#if( STACK_GROWS_DOWN )
   return ( gc_profile_stack_base - stack_top );
#else 
   return ( stack_top - gc_profile_stack_base );
#endif
}
   
/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    GC_profile ...                                                   */
/*    -------------------------------------------------------------    */
/*    This function is called on every collection.                     */
/*       GC_NUM: the number of the collection                          */
/*       GC_HEAP: the size of the heap                                 */
/*       GC_ALLOCATED: the number of bytes allocated since last GC     */
/*---------------------------------------------------------------------*/
void GC_profile( long gc_num, long gc_heap, long gc_allocated, unsigned long live ) {
   if( gc_num >= gc_profile_len ) {
      gc_profile_realloc_vector();
   }

   gc_profile_vector[ gc_num ].num   = gc_num;
   gc_profile_vector[ gc_num ].heap  = gc_heap;
   gc_profile_vector[ gc_num ].alloc = gc_allocated;
   gc_profile_vector[ gc_num ].stack = get_top_of_stack();
   gc_profile_vector[ gc_num ].live  = live;
   gc_profile_vector[ gc_num ].stamp = GC_profile_stamp;

   gc_profile_num = gc_num;
}

/*---------------------------------------------------------------------*/
/*    long                                                             */
/*    GC_profile_stop ...                                              */
/*---------------------------------------------------------------------*/
long GC_profile_stop() {
   extern obj_t bprof_port;
   FILE *file = (FILE *)bprof_port;
   long i;
   ppair_t runner;

   fprintf( file, "\n#a012\n\n" );
   fputs( "(gc \n", file );
   for( i = 0; i < gc_profile_num; i++ ) {
      fprintf( file, "   (%d %#.2f %#.2f %#.2f %#.2f",
	       gc_profile_vector[ i ].num,
	       MEGABYTE( gc_profile_vector[ i ].heap ),
	       MEGABYTE( gc_profile_vector[ i ].alloc ),
	       MEGABYTE( GC_profile_stack_size( gc_profile_vector[ i ].stack )),
	       MEGABYTE( gc_profile_vector[ i ].live ) );

      for( runner = gc_profile_vector[i].stamp; runner; runner = runner->next )
	 fprintf( file, " %s", runner->ident );

      fputs( ")\n", file );
   }
   fputs( "   )\n", file );

   /* We now free the stamp list */
   runner = GC_profile_all_stamp;
   while( runner ) {
      ppair_t old = runner;
      free( old->ident );
      runner = runner->next;
      free( old );
   }
   GC_profile_all_stamp = 0L;
   
   /* And the profile vector */
   free( gc_profile_vector );
   gc_profile_len = gc_profile_num = 0;

   return 0;
}

/*---------------------------------------------------------------------*/
/*    void                                                             */
/*    GC_profile_init ...                                              */
/*---------------------------------------------------------------------*/
void
GC_profile_init() {
   static int init = 0;

   if( !init ) {
      init = 1;
      atexit( (void (*)(void))GC_profile_stop );
   }
}

		     
