;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Eval/expd-srfi-0.scm         */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Wed Feb 24 15:25:03 1999                          */
;*    Last change :  Sun May 23 08:17:39 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    The expander for srfi form.                                      */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __expander_srfi-0
   (import  (__error                   "Llib/error.scm")
	    (__bigloo                  "Llib/bigloo.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__structure               "Llib/struct.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__bexit                   "Llib/bexit.scm")
	    
	    (__r4_numbers_6_5          "Ieee/number.scm")
	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
	    (__r4_characters_6_6       "Ieee/char.scm")
	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
	    (__r4_booleans_6_1         "Ieee/boolean.scm")
	    (__r4_symbols_6_4          "Ieee/symbol.scm")
	    (__r4_strings_6_7          "Ieee/string.scm")
	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
	    (__r4_input_6_10_2         "Ieee/input.scm")
	    (__r4_control_features_6_9 "Ieee/control.scm")
	    (__r4_vectors_6_8          "Ieee/vector.scm")
	    (__r4_ports_6_10_1         "Ieee/port.scm")
	    (__r4_output_6_10_3        "Ieee/output.scm")
	    
	    (__progn                   "Eval/progn.scm"))
   
   (use     (__type                    "Llib/type.scm")
	    (__evenv                   "Eval/evenv.scm"))
   
   (export  (expand-cond-expand x e)
	    (register-eval-srfi! ::symbol)))

;*---------------------------------------------------------------------*/
;*    *srfi-eval-list* ...                                             */
;*    -------------------------------------------------------------    */
;*    The list of supported srfi by the interpreter                    */
;*    -------------------------------------------------------------    */
;*    The four initial supported srfi are:                             */
;*       - srfi-0                                                      */
;*       - bigloo                                                      */
;*       - bigloo<major-num>                                           */
;*       - bigloo<major-num><minor-num>                                */
;*    -------------------------------------------------------------    */
;*    When a library is used for compiling the name of that library    */
;*    is added to the supported srfis.                                 */
;*---------------------------------------------------------------------*/
(define *srfi-eval-list*
   '(bigloo-eval bigloo srfi-0))

;*---------------------------------------------------------------------*/
;*    register-eval-srfi! ...                                          */
;*---------------------------------------------------------------------*/
(define (register-eval-srfi! srfi::symbol)
   (set! *srfi-eval-list* (cons srfi *srfi-eval-list*)))

;*---------------------------------------------------------------------*/
;*    expand-cond-exapnd ...                                           */
;*---------------------------------------------------------------------*/
(define (expand-cond-expand x e)
   (match-case x
      ((cond-expand)
       (error "cond-expand" "Illegal form" x))
      ((?- ((kwote else) . ?body))
       (e (epairify `(begin ,@body) x) e))
      ((?- (((kwote and)) . ?body) . ?-)
       (e (epairify `(begin ,@body) x) e))
      ((?- (((kwote and) ?req1) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 ,@body)
		      ,@else)
		    x)
	  e))
      ((?- (((kwote and) ?req1 ?req2 . ?reqs) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 (cond-expand
			      ((and ,req2 ,@reqs) ,@body)
			      ,@else)))
		    x)
	  e))
      ((?- (((kwote or)) . ?body) . ?else)
       (e (epairify `(cond-expand ,@else) x) e))
      ((?- (((kwote or) ?req1) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req1 ,@body)
		      ,@else)
		    x)
	  e))
      ((?- (((kwote or) ?req1 ?req2 . ?reqs) . ?body) . ?else)
       (let ((bd (gensym 'body)))
	  (e (epairify `(let ((,bd ,(epairify `(begin ,@body) body)))
			   (cond-expand
			    (,req1 ,bd)
			    (else
			     (cond-expand
			      ((or ,req2 ,@reqs)
			       ,bd)
			      ,@else))))
		       x)
	     e)))
      ((?- (((kwote not) ?req) . ?body) . ?else)
       (e (epairify `(cond-expand
		      (,req (cond-expand ,@else))
		      (else ,@body))
		    x)
	  e))
      ((?- ((and (? symbol?) ?feature) . ?body) . ?else)
       (e (epairify (if (memq feature *srfi-eval-list*)
			`(begin ,@body)
			`(cond-expand ,@else))
		    x)
	  e))
      (else
       (error "cond-expand" "Illegal form" x))))
       
;*---------------------------------------------------------------------*/
;*    epairify ...                                                     */
;*    -------------------------------------------------------------    */
;*    If the struct definition was an extended pair (that is if we     */
;*    were tracking the source location of the structure), we          */
;*    propagate inside the generated function, the define-struct       */
;*    location.                                                        */
;*---------------------------------------------------------------------*/
(define (epairify pair epair)
   (if (epair? epair)
       (econs (car pair) (cdr pair) (cer epair))
       pair))
