;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Llib/custom.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Mon Aug 16 17:22:43 1999                          */
;*    Last change :  Wed Aug 25 14:07:26 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Scheme custom type manipulation.                                 */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __custom
   
   (import  (__error                   "Llib/error.scm"))
   
   (use     (__type                    "Llib/type.scm")
	    (__bigloo                  "Llib/bigloo.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__ucs2                    "Llib/ucs2.scm")
	    (__dsssl                   "Llib/dsssl.scm")
	    
	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
	    (__r4_vectors_6_8          "Ieee/vector.scm")
	    (__r4_booleans_6_1         "Ieee/boolean.scm")
	    (__r4_characters_6_6       "Ieee/char.scm")
	    (__r4_symbols_6_4          "Ieee/symbol.scm")
	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
	    (__r4_strings_6_7          "Ieee/string.scm")
	    (__r4_ports_6_10_1         "Ieee/port.scm")
	    (__r4_control_features_6_9 "Ieee/control.scm")

	    (__evenv                   "Eval/evenv.scm"))

   (extern  (macro c-custom?::bool (::obj) "CUSTOMP")
	    (macro c-custom-equal?::bool  (::custom ::custom) "CUSTOM_CMP")
	    (macro c-custom-hash::int (::custom) "CUSTOM_HASH_NUMBER")
	    
	    (macro c-custom-identifier::string (::custom)
		   "CUSTOM_IDENTIFIER")
   	    (macro c-custom-identifier-set!::obj (::custom ::string)
		   "CUSTOM_IDENTIFIER_SET"))
   
   (export  (inline custom?::bool ::obj)
	    (inline custom-equal?::bool ::custom ::custom)
	    (inline custom-identifier::string ::custom)
	    (inline custom-identifier-set! ::custom ::string)
	    (custom-hash ::custom ::int)))

;*---------------------------------------------------------------------*/
;*    custom? ...                                                      */
;*---------------------------------------------------------------------*/
(define-inline (custom? obj)
   (c-custom? obj))

;*---------------------------------------------------------------------*/
;*    custom-equal? ...                                                */
;*---------------------------------------------------------------------*/
(define-inline (custom-equal? obj1 obj2)
   (c-custom-equal? obj1 obj2))

;*---------------------------------------------------------------------*/
;*    custom-identifier ...                                            */
;*---------------------------------------------------------------------*/
(define-inline (custom-identifier::string custom::custom)
   (c-custom-identifier custom))

;*---------------------------------------------------------------------*/
;*    custom-identifier-set! ...                                       */
;*---------------------------------------------------------------------*/
(define-inline (custom-identifier-set! custom::custom str::string)
   (c-custom-identifier-set! custom str))

;*---------------------------------------------------------------------*/
;*    custom-hash ...                                                  */
;*---------------------------------------------------------------------*/
(define (custom-hash custom::custom mod::int)
   (let ((num (c-custom-hash custom)))
      (remainder num mod)))
