;*---------------------------------------------------------------------*/
;*    A practical implementation for the Scheme programming language   */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*               Copyright (c) 1992-1999 Manuel Serrano                */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo@kaolin.unice.fr                                        */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Read/reader.scm              */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Tue Dec 27 11:16:00 1994                          */
;*    Last change :  Sun Aug 15 17:34:11 1999 (serrano)                */
;*    -------------------------------------------------------------    */
;*    Bigloo's reader                                                  */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    Le module                                                        */
;*---------------------------------------------------------------------*/
(module __reader

   (import  (__error                   "Llib/error.scm"))

   (use     (__type                    "Llib/type.scm")
	    (__bigloo                  "Llib/bigloo.scm")
	    (__structure               "Llib/struct.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__dsssl                   "Llib/dsssl.scm")
	    (__ucs2                    "Llib/ucs2.scm")
	    (__unicode                 "Llib/unicode.scm")
	    (__bexit                   "Llib/bexit.scm")
	    (__binary                  "Llib/binary.scm")

	    (__rgc                     "Rgc/rgc.scm")

	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
	    (__r4_numbers_6_5          "Ieee/number.scm")
	    (__r4_characters_6_6       "Ieee/char.scm")
	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
	    (__r4_booleans_6_1         "Ieee/boolean.scm")
	    (__r4_symbols_6_4          "Ieee/symbol.scm")
	    (__r4_strings_6_7          "Ieee/string.scm")
	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
	    (__r4_control_features_6_9 "Ieee/control.scm")
	    (__r4_vectors_6_8          "Ieee/vector.scm")
	    (__r4_ports_6_10_1         "Ieee/port.scm")
	    (__r4_input_6_10_2         "Ieee/input.scm")
	    (__r4_output_6_10_3        "Ieee/output.scm")

	    (__evenv                   "Eval/evenv.scm"))

   (foreign (macro obj unspec               "BUNSPEC")
	    (macro obj boptional            "BOPTIONAL")
	    (macro obj brest                "BREST")
	    (macro obj bkey                 "BKEY")
	    (macro obj make-cnst (long)     "BCNST")
	    (export *bigloo-case-sensitive* "bigloo_case_sensitive"))
   
   (export  *bigloo-interpreter*
	    *bigloo-case-sensitive*
	    (read . port)
	    (read-case-sensitive . port)
	    (read-case-insensitive . port)
	    (reader-reset!)))

;*---------------------------------------------------------------------*/
;*    *bigloo-case-senstive* ...                                       */
;*---------------------------------------------------------------------*/
(define *bigloo-case-sensitive* *bigloo-case-sensitive*)
   
;*---------------------------------------------------------------------*/
;*    *bigloo-interpreter* ...                                         */
;*---------------------------------------------------------------------*/
(define *bigloo-interpreter* #f)

;*---------------------------------------------------------------------*/
;*    Les variables de control du lecteur                              */
;*---------------------------------------------------------------------*/
(define *par-open*  0)
(define *bra-open*  0)

;*---------------------------------------------------------------------*/
;*    Parenthesis mismatch (or unclosing) errors.                      */
;*---------------------------------------------------------------------*/
(define *list-error-level* 20)
(define *list-errors*      (make-vector *list-error-level* #unspecified))
(define *vector-errors*    (make-vector *list-error-level* #unspecified))

;*---------------------------------------------------------------------*/
;*    Control variables.                                               */
;*---------------------------------------------------------------------*/
(define *cyclic-references* '())
(define *position?*         #f)
(define *line-number*       1)
(define *end-of-list*       (cons 0 0))
(define *dotted-mark*       (cons 1 1))

;*---------------------------------------------------------------------*/
;*    reader-reset! ...                                                */
;*---------------------------------------------------------------------*/
(define (reader-reset!)
   (set! *line-number* 1)
   (set! *par-open* 0)
   (set! *bra-open* 0))

;*---------------------------------------------------------------------*/
;*    string-newline-number ...                                        */
;*    -------------------------------------------------------------    */
;*    The number of occurrence of the Newline character inside the     */
;*    string.                                                          */
;*---------------------------------------------------------------------*/
(define (string-newline-number string)
   (let loop ((i (-fx (string-length string) 1))
	      (occ 0))
      (cond
	 ((=fx i -1)
	  occ)
	 ((char=? (string-ref string i) #\Newline)
	  (loop (-fx i 1) (+fx occ 1)))
	 (else
	  (loop (-fx i 1) occ)))))

;*---------------------------------------------------------------------*/
;*    read-error ...                                                   */
;*---------------------------------------------------------------------*/
(define (read-error msg obj port)
   (let* ((obj-loc (if (epair? obj)
		       (match-case (cer obj)
			  ((at ?fname ?pos ?-)
			   pos)
			  (else
			   #f))
		       #f))
	  (loc (if (number? obj-loc)
		   obj-loc
		   (cond
		      ((>fx *par-open* 0)
		       (let ((open-key (-fx *par-open* 1)))
			  (if (<fx open-key (vector-length *list-errors*))
			      (c-vector-ref *list-errors* open-key)
			      #f)))
		      ((>fx *bra-open* 0)
		       (let ((open-key (-fx *bra-open* 1)))
			  (if (<fx open-key (vector-length *vector-errors*))
			      (c-vector-ref *vector-errors* open-key)
			      #f)))
		      (else
		       #f)))))
      (if (fixnum? loc)
	  (error/location "read" msg obj (input-port-name port) loc)
	  (error "read" msg obj))))

;*---------------------------------------------------------------------*/
;*    unreference! ...                                                 */
;*---------------------------------------------------------------------*/
(define (unreference! obj port)
   (let loop ((obj obj))
      (cond
	 ((procedure? obj)
	  (let* ((no   (obj))
		 (cell (assq no *cyclic-references*)))
	     (if (not (pair? cell))
		 (error/location "read"
				 "no target for graph reference"
				 no
				 (input-port-name port)
				 (input-port-position port))
		 (cdr cell))))
	 ((pair? obj)
	  (set-car! obj (loop (car obj)))
	  (set-cdr! obj (loop (cdr obj)))
	  obj)
	 ((vector? obj)
	  (let ((len (vector-length obj)))
	     (let laap ((i 0))
		(if (<fx i len)
		    (begin
		       (vector-set! obj i (loop (vector-ref obj i)))
		       (laap (+fx i 1)))
		    obj))))
	 ((struct? obj)
	  (let ((len (vector-length obj)))
	     (let laap ((i 0))
		(if (<fx i len)
		    (begin
		       (struct-set! obj i (loop (struct-ref obj i)))
		       (laap (+fx i 1)))
		    obj))))
	 (else
	  obj))))
   
;*---------------------------------------------------------------------*/
;*    make-list! ...                                                   */
;*---------------------------------------------------------------------*/
(define (make-list! l port)
   (define (reverse-proper-list! l)
      (let nr ((l l)
	       (r '()))
	 (cond
	    ((eq? (car l) *dotted-mark*)
	     (read-error "Illegal pair" r port))
	    ((null? (cdr l))
	     (set-cdr! l r)
	     l)
	    (else
	     (let ((cdrl (cdr l)))
		(nr cdrl
		    (begin (set-cdr! l r)
			   l)))))))
   (define (reverse-improper-list! l)
      (let nr ((l (cddr l))
	       (r (car l)))
	 (cond
	    ((eq? (car l) *dotted-mark*)
	     (read-error "Illegal pair" r port))
	    ((null? (cdr l))
	     (set-cdr! l r)
	     l)
	    (else
	     (let ((cdrl (cdr l)))
		(nr cdrl
		    (begin (set-cdr! l r)
			   l)))))))
   (cond
      ((null? l)
       l)
      ((and (pair? l) (pair? (cdr l)) (eq? (cadr l) *dotted-mark*))
       (reverse-improper-list! l))
      (else
       (reverse-proper-list! l)))) 
	   
;*---------------------------------------------------------------------*/
;*    collect-up-to ...                                                */
;*---------------------------------------------------------------------*/
(define-inline (collect-up-to ignore kind port)
   (let ((name (input-port-name port)))
      (let loop ((acc  '())
		 (pos  (input-port-position port))
		 (line *line-number*))
	 (let ((item (ignore)))
	    (if (eq? item *end-of-list*)
		acc
		(let ((new-pos  (input-port-position port))
		      (new-line *line-number*))
		   (loop (if *position?*
			     (econs item acc (list 'at name pos line))
			     (cons item acc))
			 new-pos
			 new-line)))))))
	     
;*---------------------------------------------------------------------*/
;*    *bigloo-grammar* ...                                             */
;*---------------------------------------------------------------------*/
(define *bigloo-grammar*
   (regular-grammar ((float    (or (: (* digit) "." (+ digit))
			 	   (: (+ digit) "." (* digit))))
		     (letter   (in ("azAZ") (#a128 #a255)))
		     (special  (in "!@~$%^&*></-_+\\|=?.:"))
		     (kspecial (in "!@~$%^&*></-_+\\|=?."))
		     (quote    (in "\",'`"))
		     (paren    (in "()[]{}"))
		     (id       (: (* digit)
				  (or letter special)
				  (* (or letter special digit (in ",'`")))))
		     (kid      (: (* digit)
				  (or letter kspecial)
				  (* (or letter kspecial digit (in ",'`")))))
		     (blank    (in #\Space #\Tab #a012 #a013)))
      
      ;; newlines
      ((+ #\Newline)
       (set! *line-number* (+fx *line-number* (the-length)))
       (ignore))
      
      ;; blank lines
      ((+ blank)
       (ignore))
      
      ;; comments
      ((: ";" (* all))
       (ignore))
      
      ;; the interpreter header or the dsssl named constants
      ((: "#!" (+ (in digit letter special ",'`")))
       (let* ((str (the-string)))
	  (cond
	     ((string=? str "#!optional")
	      boptional)
	     ((string=? str "#!rest")
	      brest)
	     ((string=? str "#!key")
	      bkey)
	     (else
	      (set! *bigloo-interpreter* #t)
	      (ignore)))))
      
      ;; characters
      ((: "#a" (= 3 digit))
       (let ((string (the-string)))
	  (if (not (=fx (the-length) 5))
	      (error/location "read"
			      "Illegal ascii character"
			      string
			      (input-port-name     (the-port))
			      (input-port-position (the-port)))
	      (integer->char (string->integer (the-substring 2 5))))))
      ((: "#\\" (or letter digit special (in "#; " quote paren)))
       (string-ref (the-string) 2))
      ((: "#\\" (>= 2 letter))
       (let ((char-name (string->symbol
			 (string-upcase!
			  (the-substring 2 (the-length))))))
	  (case char-name
	     ((NEWLINE)
	      #\Newline)
	     ((TAB)
	      #\tab)
	     ((SPACE)
	      #\space)
	     ((RETURN)
	      (integer->char 13))
	     (else
	      (error/location "read"
			      "Illegal character"
			      (the-string)
			      (input-port-name     (the-port))
			      (input-port-position (the-port)))))))
      
      ;; pointers
      ((: "#p" (+ xdigit))
       (let ((address (the-substring 2 (the-length))))
	  (pragma "(obj_t)(strtoul(BSTRING_TO_STRING($1), 0, 16))" address)))
      
      ;; ucs-2 characters
      ((: "#u" (= 4 xdigit))
       (integer->ucs2 (string->integer (the-substring 2 6) 16)))
      
      ;; strings with newline in them in addition to compute
      ;; the string, we have to count the number of newline
      ;; in order to increment the line-num variable strings
      ((: "\"" (* (or (out #a000 #\\ #\") (: #\\ all))) "\"")
       (let ((str (the-substring 1 (-fx (the-length) 1))))
	  (set! *line-number* (+fx *line-number* (string-newline-number str)))
	  (escape-scheme-string str)))
      ;; foreign strings of char
      ((: "#\"" (* (or (out #a000 #\\ #\") (: #\\ all))) "\"")
       (let ((str (the-substring 1 (-fx (the-length) 1))))
	  (set! *line-number* (+fx *line-number* (string-newline-number str)))
	  (escape-C-string str)))
      ;; ucs2 strings
      ((: "#u\"" (* (or (out #a000 #\\ #\") (: #\\ all))) "\"")
       (let ((str (the-substring 2 (the-length))))
	  (set! *line-number* (+fx *line-number* (string-newline-number str)))
	  (utf8-string->ucs2-string str)))
      
      ;; fixnums
      ((: (? (in "-+")) (+ digit))
       (the-fixnum))
      ((: "#o" (? (in "-+")) (+ (in ("07"))))
       (string->integer (the-substring 2 (the-length)) 8))
      ((: "#d" (? (in "-+")) (+ (in ("09"))))
       (string->integer (the-substring 2 (the-length)) 10))
      ((: "#x" (? (in "-+")) (+ (in (uncase (in ("09af"))))))
       (string->integer (the-substring 2 (the-length)) 16))
      ((: "#e" (? (in "-+")) (+ digit))
       (string->elong (the-substring 2 (the-length)) 10))
      ((: "#l" (? (in "-+")) (+ digit))
       (string->llong (the-substring 2 (the-length)) 10))
      
      ;; flonum
      ((: (? (in "-+"))
	  (or float
	      (: (or float (+ digit)) (in "eE") (? (in "+-")) (+ digit))))
       (the-flonum))
      
      ;; doted pairs
      ("."
       (if (<=fx *par-open* 0)
	   (error/location "read"
			   "Illegal token"
			   #\.
			   (input-port-name     (the-port))
			   (input-port-position (the-port)))
	   *dotted-mark*))
      
      ;; unspecified and eof-object
      ((: "#" (in "ue") (+ (in "nspecified-objt")))
       (let ((symbol (string->symbol
		      (string-upcase!
		       (the-substring 1 (the-length))))))
	  (case symbol
	     ((UNSPECIFIED)
	      unspec)
	     ((EOF-OBJECT)
	      beof)
	     (else
	      (error/location "read"
			      "Illegal identifier"
			      symbol
			      (input-port-name     (the-port))
			      (input-port-position (the-port)))))))
      
      ;; booleans
      ((: "#" (uncase #\t))
       #t)
      ((: "#" (uncase #\f))
       #f)
      
      ;; constants
      ((: "#<" (+ (or digit (uncase (in "afAF")))) ">")
       (if (not (=fx (the-length) 7))
	   (error/location "read"
			   "Illegal constant"
			   (the-string)
			   (input-port-name     (the-port))
			   (input-port-position (the-port)))
	   (make-cnst (string->integer (the-substring 2 6) 16))))
      
      ;; keywords
      ((or (: ":" kid) (: kid ":"))
       ;; since the keyword expression is also matched by the id
       ;; rule, keyword rule has to be placed before the id rule.
       (the-keyword))
      
      ;; identifiers
      (id
       ;; this rule has to be placed after the rule matching the `.' char
       (the-symbol))
      
      ;; quotations 
      ("'"
       (if *position?*
	   (econs 'quote
		  (cons (ignore) '())
		  (list 'at
			(input-port-name (the-port))
			(input-port-position (the-port))
			*line-number*))
	   (cons 'quote (cons (ignore) '()))))
      ("`"
       (if *position?*
	   (econs 'quasiquote
		  (cons (ignore) '())
		  (list 'at
			(input-port-name (the-port))
			(input-port-position (the-port))
			*line-number*))
	   (cons 'quasiquote (cons (ignore) '()))))
      (","
       (if *position?*
	   (econs 'unquote
		  (cons (ignore) '())
		  (list 'at
			(input-port-name (the-port))
			(input-port-position (the-port))
			*line-number*))
	   (cons 'unquote (cons (ignore) '()))))
      (",@"
       (if *position?*
	   (econs 'unquote-splicing
		  (cons (ignore) '())
		  (list 'at
			(input-port-name (the-port))
			(input-port-position (the-port))
			*line-number*))
	   (cons 'unquote-splicing (cons (ignore) '()))))
      
      ;; lists
      ((in "([")
       ;; if possible, we store the opening parenthesis.
       (if (and (vector? *list-errors*)
		(<fx *par-open* (vector-length *list-errors*)))
	   (c-vector-set! *list-errors*
			  *par-open*
			  (input-port-position (the-port))))
       ;; we increment the number of open parenthesis
       (set! *par-open* (+fx 1 *par-open*))
       ;; and then, we compute the result list...
       (make-list! (collect-up-to ignore "list" (the-port)) (the-port)))
      ((in ")]")
       ;; we decrement the number of open parenthesis
       (set! *par-open* (-fx *par-open* 1))
       (if (<fx *par-open* 0)
	   (begin
	      (set! *par-open* 0)
	      (ignore))
	   *end-of-list*))
      
      ;; vectors
      ("#("
       ;; if possible, we store the opening parenthesis.
       (if (and (vector? *vector-errors*)
		(<fx *par-open* (vector-length *vector-errors*)))
	   (let ((pos (input-port-position (the-port))))
	      (c-vector-set! *vector-errors* *par-open* pos)))
       ;; we increment the number of open parenthesis
       (set! *par-open* (+fx 1 *par-open*))
       (list->vector (reverse! (collect-up-to ignore "vector" (the-port)))))
      ((: "#" (: letter (* id)) "(")
       ;; we increment the number of open parenthesis
       (set! *par-open* (+fx 1 *par-open*))
       (let* ((id  (let ((str (the-substring 1 (-fx (the-length) 1))))
		      (string->symbol (if (not (eq? *bigloo-case-sensitive*
						    #unspecified))
					  str
					  (string-upcase! str)))))
	      (l   (reverse! (collect-up-to ignore "vector" (the-port)))))
	  (list->tvector id l)))
      
      ;; structures
      ("#{"
       ;; if possible, we store the opening parenthesis.
       (if (and (vector? *vector-errors*)
		(<fx *bra-open* (vector-length *vector-errors*)))
	   (let ((pos (input-port-position (the-port))))
	      (c-vector-set! *vector-errors* *bra-open* pos)))
       ;; then, we compute the structure
       ;; we increment the number of open parenthesis
       (set! *bra-open* (+fx 1 *bra-open*))
       (let ((l (reverse! (collect-up-to ignore "structure" (the-port)))))
	  (cons '_structure_ l)))
      ("}"
       (set! *bra-open* (-fx *bra-open* 1))
       (if (<fx *bra-open* 0)
	   (begin
	      (set! *bra-open* 0)
	      (ignore))
	   *end-of-list*))
      
      ;; cyclic target mark
      ((: "#" (+ digit) "=")
       (let* ((no (string->integer (the-substring 1 (-fx (the-length) 1))))
	      (the-object (ignore)))
	  (set! *cyclic-references*
		(cons (cons no the-object) *cyclic-references*))
	  (unreference! the-object (the-port))))
      
      ;; cyclic target reference
      ((: "#" (+ digit) "#")
       (let ((no (string->integer (the-substring 1 (-fx (the-length) 1)))))
	  (lambda () no)))
      
      ;; error or eof
      (else
       (let ((char (the-failure)))
	  (if (eof-object? char)
	      (cond
		 ((>fx *par-open* 0)
		  (let ((open-key (-fx *par-open* 1)))
		     (reader-reset!)
		     (if (and (<fx open-key (vector-length *list-errors*))
			      (fixnum? (c-vector-ref *list-errors* open-key)))
			 (error/location "read"
					 "Unclosed list"
					 char
					 (input-port-name (the-port))
					 (c-vector-ref *list-errors* open-key))
			 (error "read"
				"Unexpected end-of-file"
				"Unclosed list"))))
		 ((>fx *bra-open* 0)
		  (let ((open-key (-fx *bra-open* 1)))
		     (reader-reset!)
		     (if (and (<fx open-key (vector-length *vector-errors*))
			      (fixnum? (c-vector-ref *vector-errors*
						     open-key)))
			 (error/location "read"
					 "Unclosed vector or structure"
					 char
					 (input-port-name (the-port))
					 (c-vector-ref *vector-errors*
						       open-key))
			 (error "read"
				"Unexpected end-of-file"
				"Unclosed vector or structure"))))
		 (else
		  (reset-eof (the-port))
		  char))
	      (error/location "read"
			      "Illegal char"
			      (illegal-char-rep char)
			      (input-port-name     (the-port))
			      (input-port-position (the-port))))))))

;*---------------------------------------------------------------------*/
;*    read ...                                                         */
;*---------------------------------------------------------------------*/
(define (read . input-port)
   (set! *position?* #f)
   (set! *cyclic-references* '())
   ;; read except an undocumented argument used by the compiler to
   ;; get line number associated with expressions.
   (cond
      ((null? input-port)
       (read/rp *bigloo-grammar* (current-input-port)))
      ((not (input-port? (car input-port)))
       (error "read" "type `input-port' expected" (car input-port)))
      ((null? (cdr input-port))
       (read/rp *bigloo-grammar* (car input-port)))
      (else
       (set! *position?* #t)
       (if (fixnum? (cadr input-port))
	   (set! *line-number* (cadr input-port)))
       (read/rp *bigloo-grammar* (car input-port)))))

;*---------------------------------------------------------------------*/
;*    read/case ...                                                    */
;*---------------------------------------------------------------------*/
(define (read/case case . input-port)
   (let ((old *bigloo-case-sensitive*))
      (set! *bigloo-case-sensitive* case)
      (unwind-protect
	 (apply read input-port)
	 (set! *bigloo-case-sensitive* old))))
   
;*---------------------------------------------------------------------*/
;*    read-case-sensitiie ...                                          */
;*    -------------------------------------------------------------    */
;*    Case sensitive read.                                             */
;*---------------------------------------------------------------------*/
(define (read-case-sensitive . input-port)
   (apply read/case (cons #t input-port)))

;*---------------------------------------------------------------------*/
;*    read-case-insensitive ...                                        */
;*    -------------------------------------------------------------    */
;*    Case unsensitive read.                                           */
;*---------------------------------------------------------------------*/
(define (read-case-insensitive . input-port)
   (apply read/case (cons #f input-port)))

